//Copyright (c) 2004 Aaron Sullivan. All rights reserved.
#include "g_MusicBox.h"
#include <iostream.h>

g_MusicBox::g_MusicBox()
{	
	int rate = G_DEFAULT_SOUND_RATE;
    Uint16 format = MIX_DEFAULT_FORMAT;
    int channels = G_DEFAULT_SOUND_CHANNELS;
    int buffers = G_DEFAULT_SOUND_BUFFERS;
	
	int frq, ch;  //these would be used to get information about the current Audio Device settings, but when sent
	Uint16 fmt;   //to Mix_QuerySpec(). We won't use the returned values, though, we just want to know if it fails.
					  

	if (!Mix_QuerySpec( &frq, &fmt, &ch ) ) //if the Audio device has not been opened yet, let's open it.
	{
		if(Mix_OpenAudio(rate, format, channels, buffers)) //open the device with our settings
		{
			cout << ("Unable to open audio! (in g_MusicBox)\n");
		}
	}

	int i;
	for (i=0; i<G_PLAYLISTMAX; i++)
	{	
		music[i] = NULL;
	}
	
	muted = false;
}

g_MusicBox::~g_MusicBox()
{
	stop();
	int i;
	for(i=0; i<5; i++)
	{
		free(i);
	}

	Mix_CloseAudio();
}

void g_MusicBox::load(string musicFilepath)
{
	if (music[0] != NULL)
	{
		free();
	}
	music[0] = Mix_LoadMUS(musicFilepath.c_str());
}

void g_MusicBox::load(string musicFilepath, int playlistNumber)
{
	if (playlistNumber >= 0 && playlistNumber < G_PLAYLISTMAX)
	{
		if (music[playlistNumber] != NULL)
		{
			free(playlistNumber);
		}
		music[playlistNumber] = Mix_LoadMUS(musicFilepath.c_str());
	}
}

void g_MusicBox::play()
{
	if (muted) return;
	if (Mix_PlayingMusic())
	{
		stop();
	}
	Mix_PlayMusic(music[0], -1); //-1 means continuously loop
	playing = 0;
	//Mix_HookMusicFinished(stop()); //can't seem to get the hook to work from within a class
}

void g_MusicBox::play(int playlistNumber)
{
	if (muted) return;
	if (playlistNumber < 0 || playlistNumber > G_PLAYLISTMAX) return;
	if (Mix_PlayingMusic())
	{
		stop();
	}
	playing = playlistNumber;
	Mix_PlayMusic(music[playlistNumber], -1); //-1 means continuously loop
	//Mix_HookMusicFinished(this->stop()); //can't seem to get the hook to work from within a class
}

void g_MusicBox::playOnce()
{
	if (muted) return;
	if (Mix_PlayingMusic())
	{
		stop();
	}
	Mix_PlayMusic(music[0], 0); //0 means loop zero times (play once)
	playing = 0;
	//Mix_HookMusicFinished(stop()); //can't seem to get the hook to work from within a class
}

void g_MusicBox::playOnce(int playlistNumber)
{
	if (muted) return;
	if (playlistNumber < 0 || playlistNumber > G_PLAYLISTMAX) return;
	if (Mix_PlayingMusic())
	{
		stop();
	}
	playing = playlistNumber;
	Mix_PlayMusic(music[playlistNumber], 0); //0 means loop zero times (play once)
	//Mix_HookMusicFinished(this->stop()); //can't seem to get the hook to work from within a class
}

void g_MusicBox::pause()
{
	if (muted) return;
	if(!Mix_PausedMusic())
	{
		Mix_PauseMusic();
	}
}

void g_MusicBox::unpause()
{
	if (muted) return;
	if(Mix_PausedMusic())
	{
		Mix_ResumeMusic();
	}
}

void g_MusicBox::pauseToggle()
{
	if (muted) return;
	if(Mix_PausedMusic())
	{
		Mix_ResumeMusic();
	}
	else
	{
		Mix_PauseMusic();
	}
}

void g_MusicBox::stop()
{
	playing = -1;
	Mix_HaltMusic();
}

void g_MusicBox::fadeOut()
{
	if (muted) return;
	//3 second fade by default
	Mix_FadeOutMusic(3000);
}

void g_MusicBox::fadeOut(int milliseconds)
{
	if (muted) return;
	Mix_FadeOutMusic(milliseconds);
}

void g_MusicBox::free()
{
	if (music[0] != NULL)
	{
		if (playing == 0)
		{
			stop();
		}
		Mix_FreeMusic(music[0]);
		music[0] = NULL;
	}
}

void g_MusicBox::free(int playlistNumber)
{
	if (playlistNumber >= 0 && playlistNumber < G_PLAYLISTMAX)
	{
		if (music[playlistNumber] != NULL)
		{
			if (playing == playlistNumber)
			{
				stop();
			}
			Mix_FreeMusic(music[playlistNumber]);
			music[playlistNumber] = NULL;
		}
	}
}

int g_MusicBox::isPlaying()
{
	return (Mix_PlayingMusic());
}

int g_MusicBox::isFadingOut()
{
	return (Mix_FadingMusic() == MIX_FADING_OUT);
}

void g_MusicBox::mute()
{
	stop();
	muted = true;
}

void g_MusicBox::unmute()
{
	muted = false;
}

int g_MusicBox::isMuted()
{
	return muted;
}