#include "g_angle.h"
#include <iostream>
using namespace std;

g_angle::g_angle()
{
	angle = 0.0f;
	//test();
}
g_angle::g_angle( float newAngle )
{
	newAngle = convertTo360Range(newAngle);
	angle = newAngle;	
}
    
void g_angle::set( float newAngle )
{
	newAngle = convertTo360Range(newAngle);
	angle = newAngle;	
}

void g_angle::rotate(float rotationAmt)
{
	angle+=rotationAmt;
	angle = convertTo360Range(angle);
}

//this is complicated by keeping everything inside the 360 degree rotation range.
int g_angle::rotateTowardsTarget(float rotationAmt, float targetAngle)
{
	if (rotationAmt >= 180)
	{
		angle = convertTo360Range(targetAngle);
		return true;
	}
	
	targetAngle = convertTo360Range(targetAngle);
	
	if (angle > targetAngle)
	{
		if (angle - targetAngle > 180)
		{
			if (rotationAmt >= targetAngle+360 - angle) //if the difference in our angle is less than the rotation amount
			{
				angle = targetAngle; //we've reached our target angle so make sure it matches exactly
				return true;
			}
			else //if we haven't made it to the target yet
			{
				rotate(rotationAmt); //rotate towards it
			}
		}
		else //same idea but going in the opposite rotation
		{
			if (rotationAmt >= angle - targetAngle)
			{
				angle = targetAngle;
				return true;
			}
			else
			{
				rotate(-rotationAmt);
			}
		}
	}
	else if (angle < targetAngle) //different rules apply if the angle is less than the target
	{
		if (targetAngle - angle < 180.0f)
		{
			if (rotationAmt >= targetAngle - angle) //if the difference in our angle is less than the rotation amount
			{
				angle = targetAngle; //we've reached our target angle so make sure it matches exactly
				return true;
			}
			else //if we haven't made it to the target yet
			{
				rotate(rotationAmt);
			}
		}
		else //same idea but going in the opposite rotation
		{
			if (rotationAmt >= angle + 360.0f - targetAngle)
			{
				angle = targetAngle;
				return true;
			}
			else
			{
				rotate(-rotationAmt);
			}
		}
	}
	else
	{
		return true; //only way to get here is if the angle and targetAngle are already equal
	}
	
	return false; //if the previous returns weren't called we didn't reach the target yet
	
}

float g_angle::distToTarget(float rotationAmt, float targetAngle)
{
	if (rotationAmt >= 180)
	{
		angle = convertTo360Range(targetAngle);
		return true;
	}
	
	targetAngle = convertTo360Range(targetAngle);
	
	if (angle > targetAngle)
	{
		if (angle - targetAngle > 180)
		{
			if (rotationAmt >= targetAngle+360 - angle) //if the difference in our angle is less than the rotation amount
			{
				angle = targetAngle; //we've reached our target angle so make sure it matches exactly
				return true;
			}
			else //if we haven't made it to the target yet
			{
				rotate(rotationAmt); //rotate towards it
			}
		}
		else //same idea but going in the opposite rotation
		{
			if (rotationAmt >= angle - targetAngle)
			{
				angle = targetAngle;
				return true;
			}
			else
			{
				rotate(-rotationAmt);
			}
		}
	}
	else if (angle < targetAngle) //different rules apply if the angle is less than the target
	{
		if (targetAngle - angle < 180.0f)
		{
			if (rotationAmt >= targetAngle - angle) //if the difference in our angle is less than the rotation amount
			{
				angle = targetAngle; //we've reached our target angle so make sure it matches exactly
				return true;
			}
			else //if we haven't made it to the target yet
			{
				rotate(rotationAmt);
			}
		}
		else //same idea but going in the opposite rotation
		{
			if (rotationAmt >= angle + 360.0f - targetAngle)
			{
				angle = targetAngle;
				return true;
			}
			else
			{
				rotate(-rotationAmt);
			}
		}
	}
	else
	{
		return true; //only way to get here is if the angle and targetAngle are already equal
	}
	
	return false; //if the previous returns weren't called we didn't reach the target yet
	
}

	
float g_angle::get()
{
	return angle;
}

inline float g_angle::convertTo360Range( float angleToConvert )
{
	if (angleToConvert >= 360.0f)
	{
		angleToConvert = (float)((int)angleToConvert%360);
	}
	else if (angleToConvert < 0.0f)
	{
		angleToConvert = (float)(360 + (int)angleToConvert%360);
	}
	
	return angleToConvert;
}

float g_angle::inRadians()
{
	return angle * g_angle_k_piDivBy180;
}

float g_angle::cosOf()
{
	if (angle == 90.0f || angle == 270.0f)
	{
		return 0.0f;
	}
	else if (angle == 0.0f)
	{
		return 1.0f;
	}
	else if (angle == 180.0f)
	{
		return -1.0f;
	}
	else
	{
		return cos(inRadians());
	}
}

float g_angle::sinOf()
{
	if (angle == 0.0f || angle == 180.0f)
	{
		return 0.0f;
	}
	else if (angle == 90.0f)
	{
		return 1.0f;
	}
	else if (angle == 270.0f)
	{
		return -1.0f;
	}
	else
	{
		return sin(inRadians());
	}
}

/*
int g_angle::test()
{
	int passed = true;
	if (330.0f != convertTo360Range(-30.0f)) 
	{
		passed = false;
		cout << "-30 failed" << endl;
	}
	if (330.0f != convertTo360Range(690.0f))
	{
		passed = false;
		cout << "690 failed" << endl;
	}
	angle = 330.0f;
	if (!rotateTowardsTarget(35.0f,5.0f)) 
	{
		passed = false;
		cout << "330 to 360 failed" << endl;
	}	
	angle = 30.0f;
	if (!rotateTowardsTarget(150.0f,180.0f)) 
	{
		passed = false;
		cout << "30 to 180 failed" << endl;
	}
	angle = 30.0f;
	if (!rotateTowardsTarget(40.0f,-10.0f)) 
	{
		passed = false;
		cout << "30 to -10 failed" << endl;
	}
	angle = 270.0f;
	if (!rotateTowardsTarget(10.0f,260.0f)) 
	{
		passed = false;
		cout << "270 to 260 failed" << endl;
	}
	if (!passed)
	{
		cout << "angle tests were not passed.";
	}
	angle = 0.0f;
}
*/