#include "sn_3Dmodel.h"
#include <iostream>
#include <fstream>
#include <string>
#include "sn_Game.h"
using namespace std;

sn_3Dmodel::sn_3Dmodel()
{
	pTextures = NULL;
	drawDL = 0;
	drawAlphaDL = 0;
	/*sn_quadFace tempFace;
	vector<sn_quadFace>::iterator itr = faces.begin();
	tempFace.x1 = -1.0f;
	tempFace.y1 = -1.0f;
	tempFace.z1 = 0.0f;
	tempFace.x2 = 1.0f;
	tempFace.y2 = -1.0f;
	tempFace.z2 = 0.0f;
	tempFace.x3 = 1.0f;
	tempFace.y3 = 1.0f;
	tempFace.z3 = 0.0f;
	tempFace.x4 = -1.0f;
	tempFace.y4 = 1.0f;
	tempFace.z4 = 0.0f;
	tempFace.textureID = 0;
	faces.push_back(tempFace);*/
}

sn_3Dmodel::~sn_3Dmodel()
{
	//if (drawDL > 0) glDeleteLists(drawDL, 1);
	//if (drawAlphaDL > 0) glDeleteLists(drawAlphaDL, 1);
}

void sn_3Dmodel::init(sn_TextureCollection * pTC, const string &filename)
{
	ifstream datafile;
	
	pTextures = pTC;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/models/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::in);
	
	float xl, yl, zl; 
	
	if (datafile.is_open())
	{
		char dotSeparator; //used to read in dot between each face
		sn_quadFace tempFace;
		string tempTextureFilename;
		int alpha;
		char quadOrTri;
		int hFlip;
		int vFlip;
		int repeating;
		float xOffset, yOffset, zOffset;
		
		datafile >> xOffset;
		datafile >> yOffset;
		datafile >> zOffset;
		datafile >> xl;
		datafile >> yl;
		datafile >> zl;
		
		collider.init(xOffset, yOffset, zOffset, xl, yl, zl);
		
		do
		{
			if (!(datafile >> alpha)) break;
			tempFace.alpha = alpha;
			
			datafile >> quadOrTri;
			if (quadOrTri == 'q')
				tempFace.isQuad = true;
			else
				tempFace.isQuad = false;
			
			datafile >> tempFace.x1;
			datafile >> tempFace.y1;
			datafile >> tempFace.z1;
			datafile >> tempFace.x2;
			datafile >> tempFace.y2;
			datafile >> tempFace.z2;
			datafile >> tempFace.x3;
			datafile >> tempFace.y3;
			datafile >> tempFace.z3;
			if (tempFace.isQuad)
			{
				datafile >> tempFace.x4;
				datafile >> tempFace.y4;
				datafile >> tempFace.z4;
			}
			
			datafile >> hFlip;
			datafile >> vFlip;
			datafile >> repeating;
			tempFace.hFlip = (bool)hFlip;
			tempFace.vFlip = (bool)vFlip;
			tempFace.repeating = (bool)repeating;
			
			datafile.ignore(80,'\n');
			getline(datafile, tempTextureFilename);
			if (!repeating)
			{
				pTextures->load(tempTextureFilename);
			}
			else
			{
				pTextures->loadRepeating(tempTextureFilename);
			}
			tempFace.textureID = pTextures->getID(tempTextureFilename);

			if (tempFace.alpha == 255)
				faces.push_back(tempFace);
			else
				alphaFaces.push_back(tempFace);
			
		} while (datafile >> dotSeparator);
		datafile.close();

	}
	else
	{
		/*
		//if we can't load file we'll fill it with generic geometry and a 0 texture
		sn_quadFace tempFace;
		//vector<sn_quadFace>::iterator itr = faces.begin();
		tempFace.isQuad = true;
		tempFace.x1 = -1.0f;
		tempFace.y1 = -1.0f;
		tempFace.z1 = 0.0f;
		tempFace.x2 = 1.0f;
		tempFace.y2 = -1.0f;
		tempFace.z2 = 0.0f;
		tempFace.x3 = 1.0f;
		tempFace.y3 = 1.0f;
		tempFace.z3 = 0.0f;
		tempFace.x4 = -1.0f;
		tempFace.y4 = 1.0f;
		tempFace.z4 = 0.0f;
		tempFace.hFlip = false;
		tempFace.vFlip = false;
		tempFace.textureID = 0;
		faces.push_back(tempFace);
		 */
	}
	
	createDrawDL(xl,yl,zl);
	createDrawAlphaDL(xl,yl,zl);
	
}


void sn_3Dmodel::draw(int facingDir) const
{
	glEnable(GL_CULL_FACE);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glRotatef(-90.0f*(float)facingDir,0.0f,0.0f,1.0f);
	
	glCallList(drawDL);
	
	glPopMatrix();
	glDisable(GL_CULL_FACE);
}

void sn_3Dmodel::createDrawDL( float xl, float yl, float zl)
{
	
	xl *= 0.5; yl *= 0.5; zl *= 0.5;
	drawDL = glGenLists(1);
	
	glNewList(drawDL, GL_COMPILE);
	
	float tx1, ty1, tx2, ty2, tx3, ty3, tx4, ty4;
	
	for (vector<sn_quadFace>::const_iterator itr = faces.begin(); itr != faces.end(); ++itr)
	{
		if (!itr->repeating)
		{
			if (itr->hFlip) { tx1 = 1; tx2 = 0; tx3 = 0; tx4 = 1; }
			else { tx1 = 0; tx2 = 1; tx3 = 1; tx4 = 0; }
		
			if (itr->vFlip) { ty1 = 1; ty2 = 1; ty3 = 0; ty4 = 0; }
			else { ty1 = 0; ty2 = 0; ty3 = 1; ty4 = 1; }
		}
		else
		{
			if (itr->hFlip) { tx1 = xl; tx2 = 0; tx3 = 0; tx4 = xl; }
			else { tx1 = 0; tx2 = xl; tx3 = xl; tx4 = 0; }
			
			if (itr->vFlip) { ty1 = yl; ty2 = yl; ty3 = 0; ty4 = 0; }
			else { ty1 = 0; ty2 = 0; ty3 = yl; ty4 = yl; }
		}
		
		glBindTexture(GL_TEXTURE_2D,itr->textureID);
		
		if (itr->isQuad)
		{
			glBegin(GL_QUADS);
			glTexCoord2f(tx1,ty1); glVertex3f(itr->x1, itr->y1, itr->z1);
			glTexCoord2f(tx2,ty2); glVertex3f(itr->x2, itr->y2, itr->z2);
			glTexCoord2f(tx3,ty3); glVertex3f(itr->x3, itr->y3, itr->z3);
			glTexCoord2f(tx4,ty4); glVertex3f(itr->x4, itr->y4, itr->z4);
			glEnd();
		}
		else
		{
			if (itr->hFlip)
			{
				glBegin(GL_TRIANGLES);
				glTexCoord2f(0,1); glVertex3f(itr->x1, itr->y1, itr->z1);
				glTexCoord2f(1,0); glVertex3f(itr->x2, itr->y2, itr->z2);
				glTexCoord2f(0,0); glVertex3f(itr->x3, itr->y3, itr->z3);
				glEnd();
			}
			else
			{
				glBegin(GL_TRIANGLES);
				glTexCoord2f(0,1); glVertex3f(itr->x1, itr->y1, itr->z1);
				glTexCoord2f(0,0); glVertex3f(itr->x2, itr->y2, itr->z2);
				glTexCoord2f(1,0); glVertex3f(itr->x3, itr->y3, itr->z3);
				glEnd();
			}
		}
		
	}
	
	glEndList();
	
}

void sn_3Dmodel::drawAlpha(int facingDir) const
{
	glEnable(GL_CULL_FACE);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glRotatef(-90.0f*(float)facingDir,0.0f,0.0f,1.0f);
	
	glCallList(drawAlphaDL);
	
	glPopMatrix();
	glDisable(GL_CULL_FACE);
	//glColor4f(1.0f,1.0f,1.0f,1.0f);
}

void sn_3Dmodel::createDrawAlphaDL( float xl, float yl, float zl )
{
	xl *= 0.5; yl *= 0.5; zl *= 0.5;
	
	drawAlphaDL = glGenLists(1);
	
	glNewList(drawAlphaDL, GL_COMPILE);
	
	float tx1, ty1, tx2, ty2, tx3, ty3, tx4, ty4;
	
	for (vector<sn_quadFace>::const_iterator itr = alphaFaces.begin(); itr != alphaFaces.end(); ++itr)
	{	
		if (!itr->repeating)
		{
			if (itr->hFlip) { tx1 = 1; tx2 = 0; tx3 = 0; tx4 = 1; }
			else { tx1 = 0; tx2 = 1; tx3 = 1; tx4 = 0; }
		
			if (itr->vFlip) { ty1 = 1; ty2 = 1; ty3 = 0; ty4 = 0; }
			else { ty1 = 0; ty2 = 0; ty3 = 1; ty4 = 1; }
		}
		else
		{
			if (itr->hFlip) { tx1 = xl; tx2 = 0; tx3 = 0; tx4 = xl; }
			else { tx1 = 0; tx2 = xl; tx3 = xl; tx4 = 0; }
			
			if (itr->vFlip) { ty1 = yl; ty2 = yl; ty3 = 0; ty4 = 0; }
			else { ty1 = 0; ty2 = 0; ty3 = yl; ty4 = yl; }
		}
		
		glBindTexture(GL_TEXTURE_2D,itr->textureID);
		
		if (itr->alpha < 254)
		{
			glColor4f(1.0f,1.0f,1.0f,itr->alpha/255.0f);
		}
		
		if (itr->isQuad)
		{
			glBegin(GL_QUADS);
			glTexCoord2f(tx1,ty1); glVertex3f(itr->x1, itr->y1, itr->z1);
			glTexCoord2f(tx2,ty2); glVertex3f(itr->x2, itr->y2, itr->z2);
			glTexCoord2f(tx3,ty3); glVertex3f(itr->x3, itr->y3, itr->z3);
			glTexCoord2f(tx4,ty4); glVertex3f(itr->x4, itr->y4, itr->z4);
			glEnd();
		}
		else
		{
			glBegin(GL_TRIANGLES);
			glTexCoord2f(tx4,ty4); glVertex3f(itr->x1, itr->y1, itr->z1);
			glTexCoord2f(tx1,ty1); glVertex3f(itr->x2, itr->y2, itr->z2);
			glTexCoord2f(tx2,ty2); glVertex3f(itr->x3, itr->y3, itr->z3);
			glEnd();
		}
		
	}
	
	glEndList();
}

sn_Collider sn_3Dmodel::getCollider()
{
	return collider;
}