#include "sn_AnimationSeqList.h"

void sn_AnimationSeqList::init( sn_TextureCollection * pTextureCollection )
{
	pTexCollection = pTextureCollection;
}

int sn_AnimationSeqList::load( string filename )
{
	ifstream datafile;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/animation/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		texSegList.clear();
		allFrames.clear();
		sequences.clear();
		
		string texFilename, texSegFilename;
		sn_TextureSegmentList tempTexSegList;
		getline(datafile, texFilename);
		
		while (texFilename != ".")
		{
			pTexCollection->load(texFilename);
			getline(datafile, texSegFilename);
			texSegList.push_back(tempTexSegList);
			texSegList.back().init(pTexCollection->getID(texFilename));
			texSegList.back().load(texSegFilename);
			getline(datafile, texFilename);
		}
		
		string sequenceName;
		int texIndex, texSeg, duration;
		sn_animationFrame tempAnimFrame;
		sn_sequenceInfo tempSeqInfo;
		
		while (getline(datafile, sequenceName))		
		{
			datafile >> texIndex;
			
			if (texIndex != -1)
			{
				tempSeqInfo.indexOfStart = allFrames.size();
				tempSeqInfo.length = 0;
			}
		
			while (texIndex != -1)
			{
				datafile >> texSeg;
				datafile >> duration;
				
				tempAnimFrame.texNumber = texIndex;
				tempAnimFrame.texSeg = texSeg;
				tempAnimFrame.duration = duration;
				allFrames.push_back(tempAnimFrame);
				tempSeqInfo.length++;
				datafile >> texIndex;
			}
			
			sequences[sequenceName] = tempSeqInfo;
			
			datafile.ignore(80,'\n');
		}
		datafile.close();
		return true;
			
	}
	else
	{
		cout << "Error on segment file opening" << endl;
		return false;
	}			
}

void sn_AnimationSeqList::draw( string sequenceName, int frameNumber )
{
    if (sequences.find(sequenceName) == sequences.end()) return;
    if (sequences[sequenceName].indexOfStart+frameNumber >= (int)allFrames.size()) return; 
	int texNumber = allFrames[sequences[sequenceName].indexOfStart+frameNumber].texNumber;
	int texSeg = allFrames[sequences[sequenceName].indexOfStart+frameNumber].texSeg;
	texSegList[texNumber].draw(texSeg);
}

void sn_AnimationSeqList::draw( string sequenceName, int frameNumber,  float scale, bool horizFlip, bool vertFlip )
{
    if (sequences.find(sequenceName) == sequences.end()) return;
    if (sequences[sequenceName].indexOfStart+frameNumber >= (int)allFrames.size()) return;
	int texNumber = allFrames[sequences[sequenceName].indexOfStart+frameNumber].texNumber;
	int texSeg = allFrames[sequences[sequenceName].indexOfStart+frameNumber].texSeg;
	texSegList[texNumber].draw(texSeg, scale, horizFlip, vertFlip);
}

unsigned int sn_AnimationSeqList::getDuration( string sequenceName, int frameNumber )
{
	if (sequences.find(sequenceName) == sequences.end())
	{
		return 0;
	}
	else
	{
		return allFrames[sequences[sequenceName].indexOfStart+frameNumber].duration;
	}
}

int sn_AnimationSeqList::getLength( string sequenceName )
{
	if (sequences.find(sequenceName) == sequences.end())
	{
		return 0;
	}
	else
	{
		return sequences[sequenceName].length;
	}
}
