#include "sn_Button.h"

sn_Button::sn_Button()
{
	x = 0;
	y = 0;
	width = 5;
	height = 5;
	on = false;
	over = false;
	type = kSBBT_boxed;
	label = "nil";
}

void sn_Button::init(sn_FontTyper * fontToUse, float xPosition, float yPosition,
					 sn_ButtonType buttonType, const char * name, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to 1024 characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,name);
    vsprintf(buffer,name,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)

	label = buffer;
	
	x = xPosition;
	y = yPosition;

	type = buttonType;
	
	pTyper = fontToUse;
	
	//determine width

	height = pTyper->getPixelHeight();
	width = pTyper->getLength2D( label.c_str() );
	/*	switch (type)
	{
		case kSBBT_boxed:
			for (i=0; i<strlen(label) && i < 32; i++)
			{
				width += vf_charwidth(label[i]) * 2.0f + 1.0f; //the extra 1 is for the tracking between each letter
			}
			height = 47;
			width+=8.0f;
			break;
		case kSBBT_checkbox:
			for (i=0; i<strlen(label) && i < 32; i++)
			{
				width += vf_charwidth(label[i]) * 2.0f + 1.0f; //the extra 1 is for the tracking between each letter
			}
			width += 30; //for the checkbox area on the left and an equal buffer on the right
			height = 45;
			break;
		case kSBBT_smallAndBlue:
			for (i=0; i<strlen(label) && i < 32; i++)
			{
				width += vf_charwidth(label[i]) * 1.5f + 1.0f; //the extra 1 is for the tracking between each letter
			}
			height = 18;
			width -= 1.5f; // remove the extra tracking bit at the end
			break;
		case kSBBT_label:
			for (i=0; i<strlen(label) && i < 32; i++)
			{
				width += vf_charwidth(label[i]) * 2.0f + 1.0f; //the extra 1 is for the tracking between each letter
			}
			height = 45;
			width -= 2.0f;
			break;
		case kSBBT_upArrow:
			width = 25;
			height = 25;
			break;
		case kSBBT_downArrow:
			width = 25;
			height = 25;
			break;
	}
	*/

}

void sn_Button::setLabel( string &newLabel )
{
	label = newLabel;
	width = pTyper->getLength2D( label.c_str() );
}

void sn_Button::setLabel( const char * name, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to 1024 characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,name);
    vsprintf(buffer,name,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	label = buffer;
	
	width = pTyper->getLength2D( label.c_str() );
}

void sn_Button::moveBy( float xChange, float yChange)
{
	x += xChange;
	y += yChange;
}

void sn_Button::moveTo( float newX, float newY )
{
	x = newX;
	y = newY;
}

int sn_Button::isOn()
{
	return on;
}

//changes state of button and returns true if click is inside button
int sn_Button::click( float clickX, float clickY )
{
	if (clickX >= x && clickX <= x+width && clickY >= y && clickY <= y+height)
	{
		if (on) {
			on = false;
		}
		else
		{
			on = true;
		}
		return true; //click was inside button so return true
	}
	//click was not inside button so return false
	return false;
}

int sn_Button::rollover( float clickX, float clickY )
{
	if (clickX >= x && clickX <= x+width && clickY >= y && clickY <= y+height)
	{
		over = true;
		return true;
	}
	else
	{
		over = false;
		return false;
	}
}

void sn_Button::turnOff()
{
	on = false;
}

void sn_Button::turnOn()
{
	on = true;
}

void sn_Button::draw()
{	
	if (over)
	{
		glColor4f(0.1f, 0.5f, 1.0f, 1.0f);
	}
	
	glMatrixMode(GL_MODELVIEW);
	
	if (type == kSBBT_checkbox)
	{
		glPushMatrix();
		if (on)
		{
			pTyper->typeChar2D(x,y,1.0,false,128);
		}
		else
		{
			pTyper->typeChar2D(x,y,1.0,false,127);
		}
		
		pTyper->type2D(x+pTyper->getLength2D("#")*2, y, 1.0, false, label.c_str());
		
		glPopMatrix();
	}
	else if (type == kSBBT_upArrow)
	{
		glPushMatrix();
		pTyper->typeChar2D(x,y,1.0,false,129);
		glPopMatrix();
	}
	else if (type == kSBBT_downArrow)
	{
		glPushMatrix();
		pTyper->typeChar2D(x,y,1.0,false,130);
		glPopMatrix();
	}
	else if (type == kSBBT_CW)
	{
		glPushMatrix();
		pTyper->typeChar2D(x,y,1.0,false,131);
		glPopMatrix();
	}
	else if (type == kSBBT_CC)
	{
		glPushMatrix();
		pTyper->typeChar2D(x,y,1.0,false,132);
		glPopMatrix();
	}
	else if (type == kSBBT_tinyAndBlue)
	{
		glPushMatrix();
		pTyper->type2D(x,y,0.8,false,label.c_str());
		glPopMatrix();
	}
	else
	{
		glPushMatrix();
		
		pTyper->type2D(x,y,1.0,false,label.c_str());
		
		glPopMatrix();
	}
	
	glColor4f(1.0f,1.0f,1.0f,1.0f);
}

void sn_Button::animate(Uint16 ticksPassed)
{
	
}

float sn_Button::getX()
{
	return x;
}
float sn_Button::getY()
{
	return y;
}
float sn_Button::getWidth()
{
	return width;
}
float sn_Button::getHeight()
{
	return height;
}
