#include "sn_FontTyper.h"

void sn_FontTyper::init( sn_TextureCollection * pTC )
{
	pTextures = pTC;
	scale = 1.0f;
	centered = false;
}

void sn_FontTyper::load( string fontTextureName )
{
	pTextures->load( fontTextureName );
	fontTextureID = pTextures->getID( fontTextureName );
	string justName = removeExtension( fontTextureName );
	letters.init( fontTextureID );
	letters.load( justName + ".tsg" );
	pixelHeight = letters.getPixelHeight();
	scale = 1.0f;
}

void sn_FontTyper::type( float x, float y, float z, float scaleMultiplier, bool center, const char * text, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	float spaceTranslateDistance = letters.getWidth(0) * scaleMultiplier;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);
	

	//find length (stops at newline... first line determines size of every line below	
	float totalLength = 0;
	for (int i=0; buffer[i] != '\0' && i<snft_MaxStringSize && buffer[i] != '`'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			totalLength += letters.getWidth(buffer[i]-33);
		}
		else
		{
			totalLength += spaceTranslateDistance;
		}
	}
		
	if (center) glTranslatef( -totalLength * 0.5 * scaleMultiplier, 0, 0 ); //move to the left half the total width of the text
	
	for (int i=0; buffer[i] != '\0'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			//draw the symbol/letter
			letters.draw(buffer[i]-33, scaleMultiplier, 0, 0 );
			glTranslatef( letters.getWidth(buffer[i]-33) * scaleMultiplier, 0, 0 );
		}
		else if (buffer[i] == '`')
		{
			glTranslatef( -totalLength * scaleMultiplier,0,0);
		}
		else
		{
			//move over enough for a space
			glTranslatef( spaceTranslateDistance, 0, 0 );
		}
		
	}
	glPopMatrix();
}

//use internal state to determine scale and centeredness
void sn_FontTyper::type( float x, float y, float z, const char * text, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	float spaceTranslateDistance = letters.getWidth(0) * scale;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);

	float totalLength = 0;
	for (int i=0; buffer[i] != '\0' && buffer[i] != '`'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			totalLength += letters.getWidth(buffer[i]-33);
		}
		else
		{
			totalLength += spaceTranslateDistance;
		}
	}
	if (centered) glTranslatef( -totalLength * 0.5 * scale, 0, 0 ); //move to the left half the total width of the text
	
	for (int i=0; buffer[i] != '\0'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			//draw the symbol/letter
			letters.draw(buffer[i]-33, scale, 0, 0 );
			glTranslatef( letters.getWidth(buffer[i]-33) * scale, 0, 0 );
		}
		else if (buffer[i] == '`')
		{
			glTranslatef( -totalLength,0,0);
		}
		else
		{
			//move over enough for a space
			glTranslatef( spaceTranslateDistance, 0, 0 );
		}
		
	}
	glPopMatrix();
}

void sn_FontTyper::type2D( float x, float y, float scaleMultiplier, bool center, const char * text, ... )
{
	float z = 0;
	
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	scaleMultiplier *= pixelHeight;
	
	float spaceTranslateDistance = letters.getWidth(0) * scaleMultiplier;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);
	
	float totalLength = 0;
	if (center)
	{
		for (int i=0; buffer[i] != '\0' && i<snft_MaxStringSize && buffer[i] != '`'; ++i)
		{
			if (buffer[i] >= 33 && buffer[i] <= 126)
			{
				totalLength += letters.getWidth(buffer[i]-33) * scaleMultiplier;
			}
			else
			{
				totalLength += spaceTranslateDistance;
			}
			glTranslatef( -totalLength * 0.5, 0, 0 ); //move to the left half the total width of the text
		}
	}
	
	totalLength = 0;
	for (int i=0; buffer[i] != '\0'; ++i)
	{
		if (buffer[i] == '`')
		{
			glTranslatef( -totalLength, pixelHeight + 10, 0 );
			totalLength = 0;
		}
		else if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			//draw the symbol/letter
			letters.draw(buffer[i]-33, scaleMultiplier, 0, 1 ); //flip vertically when drawing 2D
			glTranslatef( letters.getWidth(buffer[i]-33) * scaleMultiplier, 0, 0 );
			totalLength += letters.getWidth(buffer[i]-33) * scaleMultiplier;
		}
		else
		{
			//move over enough for a space
			glTranslatef( spaceTranslateDistance, 0, 0 );
			totalLength += spaceTranslateDistance;
		}
		
	}
	glPopMatrix();
	
	
}

//use internal state to determine scale and centeredness
void sn_FontTyper::type2D( float x, float y, const char * text, ... )
{
	float z = 0;
	
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	float scaleMultiplier = scale * pixelHeight;
	
	float spaceTranslateDistance = letters.getWidth(0) * scaleMultiplier;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);
	
	float totalLength = 0;
	for (int i=0; buffer[i] != '\0' && buffer[i] != '`'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			totalLength += letters.getWidth(buffer[i]-33) * scaleMultiplier;
		}
		else
		{
			totalLength += spaceTranslateDistance;
		}
	}
	if (centered) glTranslatef( -totalLength * 0.5 * scaleMultiplier, 0, 0 ); //move to the left half the total width of the text

	for (int i=0; buffer[i] != '\0'; ++i)
	{
		if (buffer[i] == '`')
		{
			glTranslatef( -totalLength * scaleMultiplier, pixelHeight + 10,0);
		}
		else if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			//draw the symbol/letter
			letters.draw(buffer[i]-33, scaleMultiplier, 0, 1 ); //flip verically when drawing in 2D
			glTranslatef( letters.getWidth(buffer[i]-33) * scaleMultiplier, 0, 0 );
		}
		else
		{
			//move over enough for a space
			glTranslatef( spaceTranslateDistance, 0, 0 );
		}
		
	}
	glPopMatrix();
}

void sn_FontTyper::typeChar( float x, float y, float z, float scaleMultiplier, bool center, int charNumber )
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);
			
	float totalLength = 0;
	
	if (charNumber >= 33 && charNumber <= 126)
	{
		totalLength = letters.getWidth(charNumber-33) * scaleMultiplier;
	}
	
	if (center) glTranslatef( -totalLength * 0.5, 0, 0 ); //move to the left half the total width of the text
	
	if (charNumber >= 33)
	{
		//draw the symbol/letter
		letters.draw(charNumber-33, scaleMultiplier, 0, 1 ); //flip vertically when drawing 2D
		glTranslatef( letters.getWidth(charNumber-33) * scaleMultiplier, 0, 0 );
	}
	
	glPopMatrix();
	
}

void sn_FontTyper::typeChar2D( float x, float y, float scaleMultiplier, bool center, int charNumber )
{
	float z = 0;
		
	scaleMultiplier *= pixelHeight;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x, y, z);
	
		
	float totalLength = 0;

	if (charNumber >= 33 && charNumber <= 126)
	{
		totalLength = letters.getWidth(charNumber-33) * scaleMultiplier;
	}

	if (center) glTranslatef( -totalLength * 0.5, 0, 0 ); //move to the left half the total width of the text
	
	if (charNumber >= 33)
	{
		//draw the symbol/letter
		letters.draw(charNumber-33, scaleMultiplier, 0, 1 ); //flip vertically when drawing 2D
		glTranslatef( letters.getWidth(charNumber-33) * scaleMultiplier, 0, 0 );
	}

	glPopMatrix();
		
}

float sn_FontTyper::getLength( const char * text, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	//measure length of each letter and space
	float totalLength = 0;
	float spaceTranslateDistance = letters.getWidth(0);
	for (int i=0; buffer[i] != '\0' && buffer[i] != '`'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			totalLength += letters.getWidth(buffer[i]-33);
		}
		else
		{
			totalLength += spaceTranslateDistance;
		}
	}
	
	return totalLength;
}

float sn_FontTyper::getLength2D( const char * text, ... )
{
	//start dealing with "..." (list of arguments like used in printf)
    va_list argumentList;
    char buffer[snft_MaxStringSize]; //limits size to snft_MaxStringSize characters
    memset(buffer,0,sizeof(buffer));
	
	va_start(argumentList,text);
    vsprintf(buffer,text,argumentList);
    va_end(argumentList);
    //stop dealing with ... (we now have a formatted string in buffer)
	
	//measure length of each letter and space
	float totalLength = 0;
	float spaceTranslateDistance = letters.getWidth(0);
	for (int i=0; buffer[i] != '\0' && buffer[i] != '`'; ++i)
	{
		if (buffer[i] >= 33 && buffer[i] <= 126)
		{
			totalLength += letters.getWidth(buffer[i]-33);
		}
		else
		{
			totalLength += spaceTranslateDistance;
		}
	}
	
	return totalLength * pixelHeight;
}

float sn_FontTyper::getPixelHeight()
{
	return pixelHeight;
}

void sn_FontTyper::setScale( float newScale )
{
	scale = newScale;
}

void sn_FontTyper::setCentered( bool torf)
{
	centered = torf;
}

string sn_FontTyper::removeExtension( const string &s )
{
	string newString;
	string::size_type i = 0;
	while (i != s.size() && s[i] != '.') 
		++i;
	
	newString = s.substr(0,i);

	return newString;
}