#include "sn_Game.h"

sn_Game::sn_Game()
{
	firstRelativeRead = false;
	
	//rng.init(SDL_GetTicks()); //seed the random number generator
	
	#if defined(__APPLE__)
	musicBox.load(GAMETITLE + ".app/Contents/Resources/" + "title_music.ogg", kMusic_title);
	musicBox.load(GAMETITLE + ".app/Contents/Resources/" + "victory.ogg", kMusic_victory);
	musicBox.load(GAMETITLE + ".app/Contents/Resources/" + "bummer.ogg", kMusic_failure);
	musicBox.load(GAMETITLE + ".app/Contents/Resources/" + "stage_music.ogg", kMusic_gentle);
	#else
	musicBox.load("data/music/title_music.ogg", kMusic_title);
	musicBox.load("data/music/victory.ogg", kMusic_victory);
	musicBox.load("data/music/bummer.ogg", kMusic_failure);
	musicBox.load("data/music/stage_music.ogg", kMusic_gentle);
	#endif
	
	/*
	noisemaker.load("data/soundfx/buzz.aiff", kFX_buzz);
	noisemaker.load("data/soundfx/missileHit.aiff", kFX_missileHit);
	noisemaker.load("data/soundfx/missileBlast.aiff", kFX_missileBlast);
	noisemaker.load("data/soundfx/missileFail.aiff", kFX_click);
	noisemaker.load("data/soundfx/missile.aiff", kFX_playerMissile);
	noisemaker.load("data/soundfx/SBTactical1.ogg", kFX_tactical1);
	noisemaker.load("data/soundfx/SBTactical2.ogg", kFX_tactical2);
	noisemaker.load("data/soundfx/SBTactical3.ogg", kFX_tactical3);
	noisemaker.load("data/soundfx/SBTactical4.ogg", kFX_tactical4);
	noisemaker.load("data/soundfx/shieldSection.aiff", kFX_shieldSection);
	noisemaker.load("data/soundfx/shieldBarrier.aiff", kFX_shieldBarrier);
	*/
	//noisemaker.load("data/soundfx/timer.ogg", kFX_timer);
	//noisemaker.load("data/soundfx/lose.ogg", kFX_lose);
	
	reset();
	
	ticksNow = SDL_GetTicks();
	
	screenResX = 800;
	screenResY = 600;
	gameResX = 1024; //arbitrary choice for the resolution the game logic will follow (regardless of actual window/screen resolution
	gameResY = 768;
	useWindow = true;
	musicOn = true;
	soundOn = true;
	totalPlayStages = 1;
	
	gameover = false;
	chooseOptions = false;
	changeScreen = false;
	quit = false;
	edit = false;
	playEditedStage = false;
	
	musicMuted = false;
	sfxMuted = false;
	muted = false;
	
	stageNames.push_back("title.stg");
	stageNames.push_back("title2.stg");

	currentStage = 0;
	
	typer.init(&textures);
	typer.load("main_font.png");
	
	pStage = new sn_Stage;
	pStage->init(&textures);
}

sn_Game::~sn_Game()
{
	delete pStage;
	pStage = NULL;
}

void sn_Game::setResolution(int xRes,int yRes)
{
	screenResX = xRes;
	screenResY = yRes;
	
	resMultiplierX = (float)gameResX / (float)xRes;
	resMultiplierY = (float)gameResY / (float)yRes;
	
	//pointerX = (Sint32)(pointerX * resMultiplierX);
	//pointerY = (Sint32)(pointerY * resMultiplierY);
	
}

void sn_Game::reset()
{
	gameover = false;
}

void sn_Game::preEvent()
{
	pointerRelX = 0;
	pointerRelY = 0;
}

void sn_Game::processEvent( SDL_Event &event )
{
	if  (event.type == SDL_MOUSEMOTION )
	{
		//the resMultiplier is defined when the resolution is set
		pointerX = (Sint32)(event.motion.x * resMultiplierX);
		pointerY = (Sint32)(event.motion.y * resMultiplierY);
		
		pointerRelX = (Sint32)(event.motion.xrel * resMultiplierX);
		pointerRelY = (Sint32)(event.motion.yrel * resMultiplierY);
		
		if (firstRelativeRead == false)
		{
			firstRelativeRead = true;
			pointerRelX = 0;
			pointerRelY = 0;
		}
	}

	
}

Uint32 sn_Game::updateTicksPassed()
{
	ticksNow = SDL_GetTicks();
	ticksSinceLastUpdate = ticksNow - ticksAtLastUpdate;
	ticksAtLastUpdate = ticksNow;
	
	return ticksAtLastUpdate;
}

Uint32 sn_Game::getTicksPassed()
{
	return ticksSinceLastUpdate;
}


void sn_Game::resetStage()
{
	delete pStage;
	pStage = NULL;
	pStage = new sn_Stage;
	pStage->init(&textures);
	pStage->load(stageNames[currentStage], currentStage);
}

void sn_Game::reloadStage()
{
	pStage->load(stageNames[currentStage], currentStage);
}

void sn_Game::loadTitleStage()
{
	pStage->load(stageNames[0], 0);
}

void sn_Game::loadNextStage()
{
	currentStage++;
	if (currentStage < (int)stageNames.size())
		pStage->load(stageNames[currentStage], currentStage);
	else 
	{
		currentStage = 0;
		pStage->load(stageNames[currentStage], currentStage);
	}
}
