#include "sn_MovementArray.h"

sn_MovementArray::sn_MovementArray()
{	
}

sn_MovementArray::sn_MovementArray( int x, int y, int z )
{
	resize( x, y, z );
}

void sn_MovementArray::resize( int x, int y, int z )
{
	moves.resize(x * y * z);
	xSize = x;
	ySize = y;
	zSize = z;
	
	int xi, yi, zi, di; //di = directionIndex 0=n,1=e,2=s,3=w,4=up,5=down
	
	for (zi = 0; zi < zSize; ++zi)
	{
		for (yi = 0; yi < ySize; ++yi)
		{
			for (xi = 0; xi < xSize; ++xi)
			{
				for (di = 0; di < 6; ++di)
				{
					setMovement(xi,yi,zi,di,snmt::free);
				}
			}
		}
	}
}

void sn_MovementArray::setMovement( int x, int y, int z, int dir, snmt::eMoveType mType )
{
	if (dir < 0 || dir > 5) return;
	int index = getIndex(x,y,z);
	if (index == -1) return; //if trying to put something outside the array make no changes
	moves[index].direction[dir] = mType;
}

snmt::eMoveType sn_MovementArray::getMovement( int x, int y, int z, int dir )
{
	if (dir < 0 || dir > 5) return snmt::blocked;
	int index = getIndex(x,y,z);
	if (index == -1) return snmt::blocked; //if it is outside of the array return blocked
	return moves[index].direction[dir];
}

int sn_MovementArray::getXSize() const
{
	return xSize;
}
int sn_MovementArray::getYSize() const
{
	return ySize;
}
int sn_MovementArray::getZSize() const
{
	return zSize;
}

void sn_MovementArray::draw()
{
	int xi, yi, zi, di; //di = directionIndex 0=n,1=e,2=s,3=w,4=up,5=down
	snmt::eMoveType m;

	glDisable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	for (zi = 0; zi < zSize; ++zi)
	{
		for (yi = 0; yi < ySize; ++yi)
		{
			for (xi = 0; xi < xSize; ++xi)
			{
				for (di = 0; di < 6; ++di)
				{
					m = getMovement(xi,yi,zi,di);
					glPushMatrix();
					glTranslatef(xi,yi,zi);
					
					if (m == snmt::rampUp)
					{ 
						glColor4f(0.0f,1.0f,0.0f,0.5f);
						glRotatef(di*-90.0f,0.0,0.0,1.0);
						glBegin(GL_TRIANGLES);
						glVertex3f(-0.3,0,0);
						glVertex3f(+0.3,0,0);
						glVertex3f(0,0,00.3);
						glEnd();
					}
					else if (m == snmt::rampDown)
					{
						glColor4f(0.0f,1.0f,0.0f,0.5f);
						glRotatef(di*-90.0f,0.0,0.0,1.0);
						glBegin(GL_TRIANGLES);
						glVertex3f(-0.3,0,0.3);
						glVertex3f(+0.3,0,0.3);
						glVertex3f(0,0,0);
						glEnd();
					}
					else if (m == snmt::free && di == 5)
					{
						glColor4f(1,0,1,0.2f);
						glBegin(GL_TRIANGLES);
						glVertex3f(-0.1, 0, 0.2);
						glVertex3f(0.1, 0, 0.2);
						glVertex3f(0,0,0);
						glEnd();
					}
					else if (m == snmt::blocked && di < 4)
					{
						glColor4f(1,0,0,0.2);
						glRotatef(di*-90.0f,0,0,1);
						glBegin(GL_QUADS);
						glVertex3f(-0.2,-0.2,0);
						glVertex3f(0.2,-0.2,0);
						glVertex3f(0.1,0.3,0);
						glVertex3f(-0.1,0.3,0);
						glEnd();
					}
					else if (m == snmt::eblocked && di < 4)
					{
						glColor4f(0,0,1,0.2);
						glRotatef(di*-90.0f,0,0,1);
						glBegin(GL_QUADS);
						glVertex3f(-0.2,-0.2,0);
						glVertex3f(0.2,-0.2,0);
						glVertex3f(0.1,0.3,0);
						glVertex3f(-0.1,0.3,0);
						glEnd();
					}
					glPopMatrix();
				}
			}
		}
	}
	
	//restore standard render settings
	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
}

void sn_MovementArray::setBorder()
{
	int xi, yi, zi; //di = directionIndex 0=n,1=e,2=s,3=w,4=up,5=down
	
	/*//floor
	zi = 0;
	for ( yi = 0; yi < ySize; ++yi )
	{
		for ( xi = 0; xi < xSize; ++xi )
		{
			setMovement(xi,yi,zi,5,snmt::blocked);
		}
	}
	*/
	
	//ceiling
	zi = zSize - 1;
	for ( yi = 0; yi < ySize; ++yi )
	{
		for ( xi = 0; xi < xSize; ++xi )
		{
			setMovement(xi,yi,zi,4,snmt::blocked);
		}
	}
	
	//north
	yi = ySize - 1;
	for ( zi = 0; zi < zSize; ++zi )
	{
		for ( xi = 0; xi < xSize; ++xi )
		{
			setMovement(xi,yi,zi,0,snmt::blocked);
		}
	}
	
	//south
	yi = 0;
	for ( zi = 0; zi < zSize; ++zi )
	{
		for ( xi = 0; xi < xSize; ++xi )
		{
			setMovement(xi,yi,zi,2,snmt::blocked);
		}
	}
	
	//east
	xi = xSize - 1;
	for ( zi = 0; zi < zSize; ++zi )
	{
		for ( yi = 0; yi < ySize; ++yi )
		{
			setMovement(xi,yi,zi,1,snmt::blocked);
		}
	}
	
	//west
	xi = 0;
	for ( zi = 0; zi < zSize; ++zi )
	{
		for ( yi = 0; yi < ySize; ++yi )
		{
			setMovement(xi,yi,zi,3,snmt::blocked);
		}
	}
}
