#include "sn_MovementDeltas.h"
#include "sn_global.h"
#include "g_angle.h"
#include <iostream>
using namespace std;

void sn_MovementDeltas::init( sn_MovementArray * movementArray )
{
	pMovementArray = movementArray;
}

int sn_MovementDeltas::load( string filename )
{
	ifstream datafile;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/models/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::in);
	
	sn_movementDelta tempPoint;
	char tempDirChar;
	string tempTypeString;
	
	if (datafile.is_open())
	{
		while (datafile >> tempPoint.xOffset)
		{
			datafile >> tempPoint.yOffset;
			datafile >> tempPoint.zOffset;
			datafile >> tempDirChar;
			tempPoint.movementDirection = charToDir(tempDirChar);
			datafile.ignore(80, '\n');
			getline(datafile, tempTypeString);
			tempPoint.newMoveType = stringToMoveType(tempTypeString);
			deltas.push_back(tempPoint);
		}
		datafile.close();
		return false;
	}
	else
	{
		return false;
	}
		
}

void sn_MovementDeltas::applyAt( unsigned int x, unsigned int y, unsigned int z, int facing )
{
	if (facing == 0)
	{
		int i;
		for (i=0; i<deltas.size(); ++i)
		{
			pMovementArray->setMovement(x+deltas[i].xOffset,
										y+deltas[i].yOffset,
										z+deltas[i].zOffset,
										deltas[i].movementDirection,
										deltas[i].newMoveType);
		}
	}
	else
	{
		int dxOffset, dyOffset, dmDirection;
		int i;
		g_angle facingAngle;
		facingAngle.set(facing * -90);

		for (i=0; i<deltas.size(); ++i)
		{
			dxOffset = deltas[i].xOffset * sb_cosD(facingAngle.get()) - deltas[i].yOffset * sb_sinD(facingAngle.get());
			dyOffset = deltas[i].yOffset * sb_cosD(facingAngle.get()) + deltas[i].xOffset * sb_sinD(facingAngle.get());
			
			if (deltas[i].movementDirection < 4) 
				dmDirection = (deltas[i].movementDirection + facing) % 4;  //keep inside 0 to 3 range
			else 
				dmDirection = deltas[i].movementDirection;
			
			pMovementArray->setMovement(x+dxOffset,
										y+dyOffset,
										z+deltas[i].zOffset,
										dmDirection,
										deltas[i].newMoveType);
		}
	}
}

//allows for deferring certain new values to old values (like not replacing blocked with eblocked)
void sn_MovementDeltas::applyTempAt( unsigned int x, unsigned int y, unsigned int z, int facing )
{
	if (facing == 0)
	{
		int i;
		for (i=0; i<deltas.size(); ++i)
		{
			//if we are trying to write an eblocked (enemy blocked only) over a blocked go on to the next delta without
			//making a change.  This way, pushable blocks, for instance, won't "open" paths into walls that should stay blocked
			if (deltas[i].newMoveType == snmt::eblocked &&
				pMovementArray->getMovement((int)x+deltas[i].xOffset,(int)y+deltas[i].yOffset,
											(int)z+deltas[i].zOffset,deltas[i].movementDirection) == snmt::blocked)
				continue;
			//update the Movement Array
			pMovementArray->setMovement(x+deltas[i].xOffset,
										y+deltas[i].yOffset,
										z+deltas[i].zOffset,
										deltas[i].movementDirection,
										deltas[i].newMoveType);
		}
	}
	else
	{
		int dxOffset, dyOffset, dmDirection;
		int i;
		g_angle facingAngle;
		facingAngle.set(facing * -90);
		
		for (i=0; i<deltas.size(); ++i)
		{
			dxOffset = deltas[i].xOffset * sb_cosD(facingAngle.get()) - deltas[i].yOffset * sb_sinD(facingAngle.get());
			dyOffset = deltas[i].yOffset * sb_cosD(facingAngle.get()) + deltas[i].xOffset * sb_sinD(facingAngle.get());
			
			if (deltas[i].movementDirection < 4) 
				dmDirection = (deltas[i].movementDirection + facing) % 4;  //keep inside 0 to 3 range
			else 
				dmDirection = deltas[i].movementDirection;
			
			//if we are trying to write an eblocked (enemy blocked only) over a blocked go on to the next delta without
			//making a change.  This way, pushable blocks, for instance, won't "open" paths into walls that should stay blocked
			if (deltas[i].newMoveType == snmt::eblocked &&
				pMovementArray->getMovement(x+dxOffset,y+dyOffset,z+deltas[i].zOffset,dmDirection) == snmt::blocked)
				continue;
			
			//update the movement array
			pMovementArray->setMovement(x+dxOffset,
										y+dyOffset,
										z+deltas[i].zOffset,
										dmDirection,
										deltas[i].newMoveType);
		}
	}
}

//goes through all the movement Deltas and changes the effected areas to free
void sn_MovementDeltas::applyFreeAt( unsigned int x, unsigned int y, unsigned int z, int facing )
{
	if (facing == 0)
	{
		int i;
		for (i=0; i<deltas.size(); ++i)
		{
			//update the Movement Array
			pMovementArray->setMovement(x+deltas[i].xOffset,
										y+deltas[i].yOffset,
										z+deltas[i].zOffset,
										deltas[i].movementDirection,
										snmt::free);
		}
	}
	else
	{
		int dxOffset, dyOffset, dmDirection;
		int i;
		g_angle facingAngle;
		facingAngle.set(facing * -90);
		
		for (i=0; i<deltas.size(); ++i)
		{
			dxOffset = deltas[i].xOffset * sb_cosD(facingAngle.get()) - deltas[i].yOffset * sb_sinD(facingAngle.get());
			dyOffset = deltas[i].yOffset * sb_cosD(facingAngle.get()) + deltas[i].xOffset * sb_sinD(facingAngle.get());
			
			if (deltas[i].movementDirection < 4) 
				dmDirection = (deltas[i].movementDirection + facing) % 4;  //keep inside 0 to 3 range
			else 
				dmDirection = deltas[i].movementDirection;
			
			//update the movement array
			pMovementArray->setMovement(x+dxOffset,
										y+dyOffset,
										z+deltas[i].zOffset,
										dmDirection,
										snmt::free);
		}
	}
}


snmt::eMoveType sn_MovementDeltas::stringToMoveType(const string & name)
{
	if (name == "free")
	{
		return snmt::free;
	}
	if (name == "blocked")
	{
		return snmt::blocked;
	}
	if (name == "rampUp")
	{
		return snmt::rampUp;
	}
	if (name == "rampDown")
	{
		return snmt::rampDown;
	}
	if (name == "eblocked")
	{
		return snmt::eblocked;
	}
	return snmt::free;
}

int sn_MovementDeltas::charToDir( const char dirChar )
{
	switch(dirChar)
	{
	case 'n':
	case 'N':
		return 0;
		break;
	case 'e':
	case 'E':
		return 1;
		break;
	case 's':
	case 'S':
		return 2;
		break;
	case 'w':
	case 'W':
		return 3;
		break;
	case 'u':
	case 'U':
		return 4;
		break;
	case 'd':
	case 'D':
		return 5;
		break;
	default:
		return 0;
	}
}

vector<sn_movementDelta>::const_iterator sn_MovementDeltas::getDeltasBeginItr()
{
	return deltas.begin();
}

vector<sn_movementDelta>::const_iterator sn_MovementDeltas::getDeltasEndItr()
{
	return deltas.end();
}

void sn_MovementDeltas::add(sn_movementDelta md)
{
	deltas.push_back(md);
}

void sn_MovementDeltas::clear()
{
	deltas.clear();
}
