//Copyright (c) 2004 Aaron Sullivan. All rights reserved.
#include "sn_Phase_Continue.h"
#include <iostream>
using namespace std;

sn_Phase_Continue::sn_Phase_Continue( sn_Game * gameToUse) 
: sn_GamePhase( gameToUse ) //call sb_GamePhase constructor
{
	game = gameToUse;
	
	startButton.init(&game->typer, game->gameResX - 100, 700, kSBBT_smallAndBlue, "Play"); //start the button near the center
	startButton.moveBy(-(startButton.getWidth()/2), 0.0f); //finish centering it based on its width
														   //optionsButton.init(&game->typer, game->gameResX/2, 550, kSBBT_smallAndBlue, "OPTIONS");
														   //optionsButton.moveBy(-(optionsButton.getWidth()/2), 0.0f);
														   //quitButton.init(&game->typer, game->gameResX - 200, 750, kSBBT_smallAndBlue, "QUIT");
														   //quitButton.moveBy(-(quitButton.getWidth()/2), 0.0f);
	
	zCameraAngle.set(45.0f);
	zCameraAngleTarget = 45.0f;
	cameraRotatingCW = false;
	cameraRotatingCCW = false;
	zCameraRotationSpeed = 0.2f;
	
	cameraZoom = 425.0f;
	cameraZoomTarget = 425.0f;
	cameraZoomingIn = false;
	cameraZoomingOut = false;
	cameraZoomSpeed = 1.0f;
	
	stageNames.push_back("title.stg");
	stageNames.push_back("hole_in_one.stg");
	stageNames.push_back("two.stg");
	stageNames.push_back("jump.stg");
	stageNames.push_back("level1.stg");
	stageNames.push_back("ledges.stg");
	stageNames.push_back("bridge_the_gap.stg");
	stageNames.push_back("level3.stg");
	stageNames.push_back("phat_pharoah.stg");
	
	fanfarePlaying = false;
	
	reset();
}

sn_Phase_Continue::~sn_Phase_Continue()
{
	
}

void sn_Phase_Continue::start()
{
	done = false;
	time = 0;
	
	//title variables
	titleX = 285.0f, titleY = 300.0f;
	titleAlpha = 0.8f;
	titleScaleX = 50.0f;
	titleScaleY = 8.0f;
	titleGlow = 0.0f;
	
	//phase specific
	showingTitle = false;
	erasingTitle = false;
	
	for (int i=0; i<10; i++)
		keyhold[i] = false;
	
	game->pStage->load("standard.stg", 1);
	
	game->musicBox.play(kMusic_gentle);
	fanfarePlaying = false;
	
	//start keeping track of Ticks passed
	game->updateTicksPassed();
	
}

void sn_Phase_Continue::pause()
{
	game->musicBox.pauseToggle();
	paused = true;
	
}

void sn_Phase_Continue::unpause()
{
	game->musicBox.pauseToggle();
	
	game->updateTicksPassed();
	game->getTicksPassed();
	paused = false;
}

void sn_Phase_Continue::end()
{
	done = true;
}

void sn_Phase_Continue::reset()
{
	showingTitle = false;
	keyhold.clear();
	keyhold.resize(sn_totalPlayerControls);
	currentStage = 0;
}

void sn_Phase_Continue::processEvent( SDL_Event &event )
{
	switch (event.type)
	{
		case SDL_MOUSEMOTION:
			if (game->pointerX > game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayerEdit(sn_FUR);
			}
			else if (game->pointerX > game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayerEdit(sn_FLR);
			}
			else if (game->pointerX < game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayerEdit(sn_FUL);
			}
			else if (game->pointerX < game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayerEdit(sn_FLL);
			}
			startButton.rollover(game->pointerX, game->pointerY);
			break;
			
			//pointer position comes from game object ( which has its own processEvent function )
		case SDL_MOUSEBUTTONDOWN:
			if (showingTitle)
			{
				if ( event.button.button == SDL_BUTTON_LEFT )
				{
					if (startButton.click(game->pointerX, game->pointerY))
					{
						//erasingTitle = true; //when title is erased this phase will end()
						//showingTitle = false; 
						//timeTitleEnded = time;
						//game->musicBox.fadeOut(400);
						game->pStage->controlPlayerEdit(sn_PlayEditedStage);
						game->playEditedStage = true;
						
						//game->pStage->load(stageNames[currentStage], currentStage);
					}
					/*else if (optionsButton.click(game->pointerX, game->pointerY))
				{
						game->chooseOptions = true;
				}*/
					/*else if (quitButton.click(game->pointerX, game->pointerY))
				{
						game->quit = true;
				}*/
					else if (game->pointerX > game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
					{
						keyhold[sn_UR] = true;
					}
					else if (game->pointerX > game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
					{
						keyhold[sn_LR] = true;
					}
					else if (game->pointerX < game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
					{
						keyhold[sn_UL] = true;
					}
					else if (game->pointerX < game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
					{
						keyhold[sn_LL] = true;
					}
				}
				else if (event.button.button == SDL_BUTTON_RIGHT )
				{
					keyhold[sn_Shoot] = true;
					game->pStage->controlPlayerEdit(sn_Shoot);
				}
				
			}
			if( event.button.button == SDL_BUTTON_WHEELUP)
			{
				game->pStage->controlPlayerEdit(sn_NextModel);
			}
			if( event.button.button == SDL_BUTTON_WHEELDOWN)
			{
				game->pStage->controlPlayerEdit(sn_LastModel);
			}
			break;
		case SDL_MOUSEBUTTONUP:
			if ( event.button.button == SDL_BUTTON_LEFT )
			{
				keyhold[sn_UR] = false;
				keyhold[sn_LR] = false;
				keyhold[sn_UL] = false;
				keyhold[sn_LL] = false;
				keyhold[sn_Up] = false;
				keyhold[sn_Down] = false;
			}
			else if (event.button.button == SDL_BUTTON_RIGHT )
			{
				keyhold[sn_Shoot] = false;
			}
			break;
		case SDL_KEYDOWN:
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				keyhold[sn_Shoot] = true;
				game->pStage->controlPlayerEdit(sn_Shoot);
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FUL]=true;
				}
				else
				{
					keyhold[sn_UL] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FUR]=true;
				}
				else
				{
					keyhold[sn_UR] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FLR]=true;
				}
				else
				{
					keyhold[sn_LR] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FLL]=true;
				}
				else if (event.key.keysym.mod & KMOD_META || event.key.keysym.mod & KMOD_CTRL)
				{
					game->pStage->controlPlayerEdit(sn_SaveStage);
				}
				else
				{
					keyhold[sn_LL] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_r)
			{
				keyhold[sn_Up] = true; 
			}
			else if (event.key.keysym.sym == SDLK_f)
			{
				keyhold[sn_Down] = true;
			}
			else if (event.key.keysym.sym == SDLK_RIGHT)
			{
				if (!cameraRotatingCW)
				{
					zCameraAngleTarget -= 90.0f;
					zCameraAngleHalfwayToTarget = zCameraAngleTarget + 45.0f;
					if (zCameraAngleTarget < 0)
					{
						zCameraAngleTarget = 315;
					}
					cameraRotatingCW = true;
					cameraRotatingCCW = false;
				}
			}
			else if (event.key.keysym.sym == SDLK_LEFT)
			{
				if (!cameraRotatingCCW)
				{
					zCameraAngleTarget += 90.0f;
					zCameraAngleHalfwayToTarget = zCameraAngleTarget - 45.0f;
					cameraRotatingCCW = true;
					cameraRotatingCW = false;
				}
			}
			else if (event.key.keysym.sym == SDLK_UP)
			{
				if (!cameraZoomingIn)
				{
					cameraZoomTarget += 50;
					if (cameraZoomTarget <= 500)
					{
						cameraZoomingIn = true;
						cameraZoomingOut = false;
					}
					else cameraZoomTarget -= 50;
				}
				
			}
			else if (event.key.keysym.sym == SDLK_DOWN)
			{
				if (!cameraZoomingOut)
				{
					cameraZoomTarget -= 50;
					if (cameraZoomTarget >= 0)
					{
						cameraZoomingIn = false;
						cameraZoomingOut = true;
					}
					else cameraZoomTarget += 50;
				}
			}
			else if (event.key.keysym.sym == SDLK_c)
			{
				game->pStage->toggleDrawColliders();
			}
			else if (event.key.keysym.sym == SDLK_m)
			{
				game->pStage->toggleDrawMovementArray();
			}
			else if (event.key.keysym.sym == SDLK_PERIOD )
			{
				//keyhold[sn_NextModel] = true;
				game->pStage->controlPlayerEdit(sn_NextModel);
			}
			else if (event.key.keysym.sym == SDLK_COMMA )
			{
				//keyhold[sn_LastModel] = true;
				game->pStage->controlPlayerEdit(sn_LastModel);
			}
			else if (event.key.keysym.sym == SDLK_BACKSPACE)
			{
				game->pStage->controlPlayerEdit(sn_EraseBlock);
			}
			else if (event.key.keysym.sym == SDLK_1)
			{
				game->pStage->controlPlayerEdit(sn_AddPushableBlock);
			}
			else if (event.key.keysym.sym == SDLK_2)
			{
				game->pStage->controlPlayerEdit(sn_AddPlayerStart);
			}
			else if (event.key.keysym.sym == SDLK_l)
			{
				if (event.key.keysym.mod & KMOD_META || event.key.keysym.mod & KMOD_CTRL)
				{
					game->pStage->controlPlayerEdit(sn_LoadStage);
				}
			}
			else if (event.key.keysym.sym == SDLK_p)
			{
				if (event.key.keysym.mod & KMOD_META || event.key.keysym.mod & KMOD_CTRL)
				{
					game->pStage->controlPlayerEdit(sn_PlayEditedStage);
					game->playEditedStage = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_3)
			{
				game->pStage->controlPlayerEdit(sn_AddOtherSnowball);
			}
			break;
		case SDL_KEYUP:
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				keyhold[sn_Shoot] = false;
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				keyhold[sn_UL] = false;
				keyhold[sn_FUL] = false;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				keyhold[sn_UR] = false;
				keyhold[sn_FUR] = false;
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				keyhold[sn_LR] = false;
				keyhold[sn_FLR] = false;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				keyhold[sn_LL] = false;
				keyhold[sn_FLL] = false;
			}
			else if (event.key.keysym.sym == SDLK_r)
			{
				keyhold[sn_Up] = false; 
			}
			else if (event.key.keysym.sym == SDLK_f)
			{
				keyhold[sn_Down] = false;
			}
			else if (event.key.keysym.sym == SDLK_PERIOD )
			{
				keyhold[sn_NextModel] = false;
			}
			else if (event.key.keysym.sym == SDLK_COMMA )
			{
				keyhold[sn_LastModel] = false;
			}
			else if (event.key.keysym.sym == SDLK_BACKSPACE)
			{
				keyhold[sn_EraseBlock] = false;
			}
			
			break;
		default:
			break;
	}
}

void sn_Phase_Continue::animate()
{
	Uint32 ticksPassed;
	game->updateTicksPassed();
	ticksPassed = game->getTicksPassed();
	
	//Time based events
	time = time + ticksPassed;
	
	Uint32 startTime, endTime;
	float transitionTime; //will hold amount 
	
	if (time > 500 && erasingTitle == false && showingTitle == false)
	{
		showingTitle = true;
		timeTitleStarted = 500;
	}
	
	if (fanfarePlaying && false == game->musicBox.isPlaying())
	{
		fanfarePlaying = false;
		game->musicBox.play(kMusic_title);
	}
	
	if (showingTitle)
	{
		
		if (keyhold[sn_UL] == true)
		{
			game->pStage->controlPlayerEdit(sn_UL);
		}
		else if (keyhold[sn_UR] == true)
		{
			game->pStage->controlPlayerEdit(sn_UR);
		}
		else if (keyhold[sn_LR] == true)
		{
			game->pStage->controlPlayerEdit(sn_LR);
		}
		else if (keyhold[sn_LL] == true)
		{
			game->pStage->controlPlayerEdit(sn_LL);
		}
		else if (keyhold[sn_FUL] == true)
		{
			game->pStage->controlPlayerEdit(sn_FUL);
		}
		else if (keyhold[sn_FUR] == true)
		{
			game->pStage->controlPlayerEdit(sn_FUR);
		}
		else if (keyhold[sn_FLR] == true)
		{
			game->pStage->controlPlayerEdit(sn_FLR);
		}
		else if (keyhold[sn_FLL] == true)
		{
			game->pStage->controlPlayerEdit(sn_FLL);
		}
		else if (keyhold[sn_Up] == true)
		{
			game->pStage->controlPlayerEdit(sn_Up);
		}
		else if (keyhold[sn_Down] == true)
		{
			game->pStage->controlPlayerEdit(sn_Down);
		}
		
		
		game->pStage->animateEdit( ticksPassed );
		
		if (game->pStage->isFailed())
		{
			if (!fanfarePlaying)
			{
				fanfarePlaying = true;
				game->musicBox.play(kMusic_failure);
			}
			game->pStage->load(stageNames[currentStage], currentStage);
			//playerTries -= 1;
			//if (playerTries < 0) game->gameover = true;
		}
		
		if (game->pStage->isCompleted())
		{
			fanfarePlaying = true;
			game->musicBox.play(kMusic_victory);
			currentStage++;
			if (currentStage < (int)stageNames.size())
				game->pStage->load(stageNames[currentStage], currentStage);
			else 
				done = true;
		}
		
		if (cameraRotatingCW || cameraRotatingCCW)
		{
			if (zCameraAngle.rotateTowardsTarget( zCameraRotationSpeed * ticksPassed, zCameraAngleTarget))
			{
				cameraRotatingCCW = false;
				cameraRotatingCW = false;
			}
		}
		if (cameraZoomingIn)
		{
			cameraZoom += cameraZoomSpeed * ticksPassed;
			if (cameraZoom > cameraZoomTarget)
			{
				cameraZoom = cameraZoomTarget;
				cameraZoomingIn = false;
			}
		}
		else if (cameraZoomingOut)
		{
			cameraZoom -= cameraZoomSpeed * ticksPassed;
			if (cameraZoom < cameraZoomTarget)
			{
				cameraZoom = cameraZoomTarget;
				cameraZoomingOut = false;
			}
		}
		//animate the title of the phase
		//"stretch in"
		startTime = timeTitleStarted; endTime = timeTitleStarted+500; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			//the total amount to change (over whole time) is divided total time of transition and multiplied by how many ticks passed 
			//then that portion (which is the fraction needed for THIS update) is subtracted from the current value
			titleScaleX -= 47.0f/transitionTime*ticksPassed;
		}
		if (time > endTime) titleScaleX = 3.0f; //take care of inaccuracy of method
		
		//glow the title
		startTime = timeTitleStarted+500; endTime = timeTitleStarted+1000; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			if (titleGlow == 0.0f)
			{
				titleGlow = 1.0f;
			}
			titleGlow -= 1.0/transitionTime*ticksPassed;
		}
		if (time > endTime) 
		{
			titleGlow = 0.0f;
		}
	}
	
	if (erasingTitle)
	{
		//raise and vanish title
		startTime = timeTitleEnded; endTime = timeTitleEnded+500; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			titleY -= 200.0f/transitionTime*ticksPassed; 
			titleAlpha -= 0.8f/transitionTime*ticksPassed;
			if (titleAlpha < 0.0f) titleAlpha = 0.0f;
			titleScaleY += 30.0f/transitionTime*ticksPassed;
		}
		if (time >= endTime) {
			titleY = 0.0;
			titleAlpha = 0.0f;
			titleScaleY = 4.0f;
			end();
		}
	}
}

void sn_Phase_Continue::draw()
{
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***standard settings for rendering***
	//***if changed in other functions  ***
	//***they should then also be reset ***
	//***from those same functions      ***
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	//***    end standard settings      ***
	
	//***setup 3D projection
	gluLookAt(0,0,500-cameraZoom,0,0,0,0,1,0);
	
	//***begin 3D drawing
	
	game->pStage->drawEdit(zCameraAngle.get());
	
	//***end 3D drawing
	
	glPopMatrix();
	
	//***setup ortho projection for 2D overlay
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, game->gameResX, game->gameResY, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***begin 2D drawing
	
	if (showingTitle || erasingTitle)
	{
		drawTitle();
	}
	
	startButton.draw();
	//optionsButton.draw();
	//quitButton.draw();
	
	drawPointer();
	
	//****end 2D drawing
	
	//***restore matrices
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();	
}


void sn_Phase_Continue::drawTitle()
{
	
}

void sn_Phase_Continue::drawPointer()
{
}