#include "sn_Phase_Options.h"
#include "sn_global.h"


sn_Phase_Options::sn_Phase_Options( sn_Game * gameToUse) 
: sn_GamePhase( gameToUse ) //call sb_GamePhase constructor first
{
	game = gameToUse;
	
	oldResX = game->screenResX;
	oldResY = game->screenResY;
	oldWindow = game->useWindow;
	
	label_window.init( &game->typer, 200,50,kSBBT_label, "Windowed");
	button_res[0].init( &game->typer, 200,100,kSBBT_checkbox,"1280 x 960");
	button_res[1].init( &game->typer, 200,150,kSBBT_checkbox,"1024 x 768");
	button_res[2].init( &game->typer, 200,200,kSBBT_checkbox,"800 x 600");
	button_res[3].init( &game->typer, 200,250,kSBBT_checkbox,"640 x 480");
	availableRes[0].x = 1280; availableRes[0].y = 960;
	availableRes[1].x = 1024; availableRes[1].y = 768;
	availableRes[2].x =  800; availableRes[2].y = 600;
	availableRes[3].x = 640; availableRes[3].y = 480;
	
	//deal with all possible fullscreen modes
	SDL_Rect **modes;
	int i;

	modes=SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_OPENGL); // Get available modes
	
	if(modes == (SDL_Rect **)0) // Check is there are any fullscreen opengl modes available
	{
		button_res[4].init( &game->typer, 600,100,kSBBT_checkbox,"NONE");
		totalResButtons = 4;
	}
	else if(modes == (SDL_Rect **)-1) //Check if our resolution is unrestricted (using fullscreen!)
	{
		button_res[4].init( &game->typer, 600,100,kSBBT_checkbox,"1600 x 1200"); //this will probably never happen, but...
		button_res[5].init( &game->typer, 600,150,kSBBT_checkbox,"1280 x 960");
		button_res[6].init( &game->typer, 600,200,kSBBT_checkbox,"1024 x 768");
		availableRes[4].x = 1600; availableRes[4].y = 1200;
		availableRes[5].x = 1280; availableRes[5].y = 960;
		availableRes[6].x = 1024; availableRes[6].y = 768;
		totalResButtons = 7;
	}
	else
	{
		//generate buttons for every available resolution (only a select 5 will show in option screen)
		for(i=totalWindowButtons; modes[i-totalWindowButtons] && i < maxResButtons; ++i)
		{
			availableRes[i].x = modes[i-totalWindowButtons]->w;
			availableRes[i].y = modes[i-totalWindowButtons]->h;
			button_res[i].init( &game->typer, 600, 100+(i-totalWindowButtons)*50, kSBBT_checkbox, 
							   "%d x %d", (int)availableRes[i].x, (int)availableRes[i].y);
		}
		totalResButtons = i;
	}
	
	firstFullButtonShowing = totalWindowButtons;
	
	if (totalResButtons != totalWindowButtons)
		label_full.init( &game->typer, 600,50,kSBBT_label, "Fullscreen");
	
	button_music.init( &game->typer, 200,500,kSBBT_checkbox,"Play Music");
	button_sound.init( &game->typer, 600,500,kSBBT_checkbox,"Use Sound FX");
	
	button_done.init( &game->typer, game->gameResX/2, 600, kSBBT_boxed, "DONE");
	button_done.moveBy(-(button_done.getWidth()/2), 0.0f);
	button_up.init( &game->typer, 820, 100, kSBBT_upArrow,"#");
	button_down.init( &game->typer, 820, 341, kSBBT_downArrow,"#");
	
	//turn on the correct button based on the preference file
	
	selectedButton = -1;
	oldSelectedButton = -1;
	if (pref.getUseWindow())
	{
		for(i=0; i<4; i++)
		{
			if (pref.getWidth() == availableRes[i].x &&
				pref.getHeight() == availableRes[i].y)
			{
				button_res[i].turnOn();
				selectedButton = i;
				oldSelectedButton = i;
				break;
			}
		}
	}
	else
	{
		for(i=totalWindowButtons; i<totalResButtons; i++)
		{
			if (pref.getWidth() == availableRes[i].x &&
				pref.getHeight() == availableRes[i].y)
			{
				button_res[i].turnOn();
				selectedButton = i;
				oldSelectedButton = i;
				break;
			}
		}
		//if we were trying fullscreen and the pref res is not available will go to the defaul windowed size
		//which should be available on any computer. (800x600 windowed)
		if (selectedButton == -1)
		{
			pref.changeToDefault();
			for(i=0; i<4; i++)
			{
				if (pref.getWidth() == availableRes[i].x &&
					pref.getHeight() == availableRes[i].y)
				{
					button_res[i].turnOn();
					selectedButton = i;
					oldSelectedButton = i;
					break;
				}
			}
		}
	}
	//now select or deselect the music and sound buttons accordin to prefs
	if (pref.getUseMusic())
	{
		button_music.turnOn();
	}
	if (pref.getUseSound())
	{
		button_sound.turnOn();
	}
	

	
	//confirming buttons for confirmation of whether new resolution worked
	label_ok.init( &game->typer, game->gameResX/2, game->gameResY/2, kSBBT_label, "OK?");
	label_ok.moveBy(-(label_ok.getWidth()/2), -(label_ok.getHeight()/2) - 50.0f);
	button_yes.init( &game->typer, game->gameResX/2 - 50.0f, game->gameResY/2, kSBBT_boxed, "Yes");
	button_yes.moveBy(-(button_yes.getWidth()/2), -(button_yes.getHeight()/2));
	button_no.init( &game->typer, game->gameResX/2 + 50.0f, game->gameResY/2, kSBBT_boxed, "No");
	button_no.moveBy(-(button_no.getWidth()/2), -(button_no.getHeight()/2));
	label_timer.init( &game->typer, game->gameResX/2, game->gameResY/2, kSBBT_label, "10");
	label_timer.moveBy(-(label_timer.getWidth()/2), -(label_timer.getHeight()/2) + 100.0f);
	
	
	button_UL.init(&game->typer, 50,500, kSBBT_tinyAndBlue, "a");
	button_LL.init(&game->typer, 70,500, kSBBT_tinyAndBlue, "s");
	
	
	reset();
	confirming = false;
	confirmTotalSeconds = 15;
}

sn_Phase_Options::~sn_Phase_Options()
{
	
}

void sn_Phase_Options::start()
{
	done = false;
	time = 0;
	
	//title variables
	titleX = 290.0f, titleY = 300.0f;
	titleAlpha = 0.8f;
	titleScaleX = 50.0f;
	titleScaleY = 8.0f;
	titleGlow = 0.0f;
	
	//phase specific
	showingTitle = false;
	erasingTitle = false;
	
	//start keeping track of Ticks passed
	game->updateTicksPassed();
}

void sn_Phase_Options::pause()
{
	game->musicBox.pauseToggle();
	paused = true;
	
}

void sn_Phase_Options::unpause()
{
	game->musicBox.pauseToggle();
	
	game->updateTicksPassed();
	game->getTicksPassed();
	paused = false;
}

void sn_Phase_Options::end()
{
	done = true;
}

void sn_Phase_Options::reset()
{
	showingTitle = false;
	confirming = false;
}

void sn_Phase_Options::processEvent( SDL_Event &event )
{
	int i;
	string keyPressed;
	
	if (totalResButtons > totalWindowButtons + 6)
	{
		if( event.button.button == SDL_BUTTON_WHEELUP)
		{
			if (firstFullButtonShowing > totalWindowButtons) //if we aren't already at the top
			{
				firstFullButtonShowing -= 1;
				for (i=totalWindowButtons; i<totalResButtons; i++)
				{
					button_res[i].moveBy(0.0f, 50.0f);
				}
			}
		}
		if( event.button.button == SDL_BUTTON_WHEELDOWN)
		{
			if (firstFullButtonShowing+6 < totalResButtons) //if we aren't already at the top
			{
				firstFullButtonShowing += 1;
				for (i=totalWindowButtons; i<totalResButtons; i++)
				{
					button_res[i].moveBy(0.0f, -50.0f);
				}
			}
		}
	}

	switch (event.type)
	{
		case SDL_MOUSEMOTION:
			if (!confirming)
			{
				for (i=0; i<totalWindowButtons; i++)
				{
					button_res[i].rollover(game->pointerX, game->pointerY);
				}
				{
					int lastButtonShowing = CLAMP(firstFullButtonShowing+6, firstFullButtonShowing, totalResButtons);
					for (i=firstFullButtonShowing; i<lastButtonShowing; i++)
					{
						button_res[i].rollover(game->pointerX, game->pointerY);
					}
				}
				button_music.rollover(game->pointerX, game->pointerY);
				button_sound.rollover(game->pointerX, game->pointerY);
				button_done.rollover(game->pointerX, game->pointerY);
				button_up.rollover(game->pointerX, game->pointerY);
				button_down.rollover(game->pointerX, game->pointerY);
			}
			else
			{
				button_yes.rollover(game->pointerX, game->pointerY);
				button_no.rollover(game->pointerX, game->pointerY);
			}
			break;
		//pointer position comes from game object ( which has its own processEvent function )
		case SDL_MOUSEBUTTONDOWN:
			if (!confirming)
			{
				if (event.button.button == SDL_BUTTON_LEFT)
				{
					if (showingTitle)
					{
						oldResX = game->screenResX;
						oldResY = game->screenResY;
						oldWindow = game->useWindow;
						
						int resButtonClicked = -1;
						for (i=0; i<totalWindowButtons; i++)
						{
							if (button_res[i].click(game->pointerX, game->pointerY))
							{
								resButtonClicked = i;
							}
						}
						int lastButtonShowing = CLAMP(firstFullButtonShowing+6, firstFullButtonShowing, totalResButtons);
						for (i=firstFullButtonShowing; i<lastButtonShowing; i++)
						{
							if (button_res[i].click(game->pointerX, game->pointerY))
							{
								resButtonClicked = i;
							}
						}
							
						if (resButtonClicked > -1) //if a resolution button was clicked
						{							
							selectedButton = resButtonClicked;
							
							for (i=0; i < totalResButtons; i++) //turn off all the checkboxes
							{
								button_res[i].turnOff();
							}
							
							
							button_res[resButtonClicked].turnOn(); //turn on the one clicked
							
							if (resButtonClicked < totalWindowButtons) //if it's a windowed option use window
							{
								game->setResolution(availableRes[resButtonClicked].x,
													availableRes[resButtonClicked].y);
								game->useWindow = true;
							}
							else //anything else is a fullscreen option
							{
								game->setResolution(availableRes[resButtonClicked].x,
													availableRes[resButtonClicked].y);
								game->useWindow = false;
							}

							game->changeScreen = true; //this is set so the main loop will take care of the resolution switch
							confirming = true; //go into confirm mode (wait for user to okay new resolution)
							confirmSecondsLeft = confirmTotalSeconds; //reset timer
							timeConfirmStarted = time; //keep of track of when it started
						}
						
						if (button_music.click(game->pointerX, game->pointerY))
						{
							if (button_music.isOn())
							{
								game->musicMuted = false;
								game->musicBox.unmute();
								game->musicBox.play(kMusic_title);
								pref.changeMusic(true);
							}
							else
							{
								game->musicMuted = true;
								game->musicBox.mute();
								pref.changeMusic(false);
							}
						}
						
						if (button_sound.click(game->pointerX, game->pointerY))
						{
							if (button_sound.isOn())
							{
								game->sfxMuted = false;
								game->noisemaker.unmute();
								game->noisemaker.play(kFX_click);
								pref.changeSound(true);
							}
							else
							{
								game->sfxMuted = true;
								game->noisemaker.mute();
								pref.changeSound(false);
							}
						}
						
						if (button_done.click(game->pointerX, game->pointerY))
						{
							end();
						}
						
						if (totalResButtons > totalWindowButtons + 6)
						{
							if (button_up.click(game->pointerX, game->pointerY))
							{
								if (firstFullButtonShowing > totalWindowButtons) //if we aren't already at the top
								{
									firstFullButtonShowing -= 1;
									for (i=totalWindowButtons; i<totalResButtons; i++)
									{
										button_res[i].moveBy(0.0f, 50.0f);
									}
								}
							}
							
							if (button_down.click(game->pointerX, game->pointerY))
							{
								if (firstFullButtonShowing+6 < totalResButtons) //if we aren't already at the top
								{
									firstFullButtonShowing += 1;
									for (i=totalWindowButtons; i<totalResButtons; i++)
									{
										button_res[i].moveBy(0.0f, -50.0f);
									}
								}
							}
						}
					}
				} //end of mouse left button
				
				//erasingTitle = true; //when title is erased this phase will end()
				//showingTitle = false; 
				//timeTitleEnded = time;
				//game->musicBox.fadeOut(400);
			}//end of mousebutton when not confirming
			else //we are in confirmation mode
			{
				if (event.button.button == SDL_BUTTON_LEFT)
				{
					if (button_yes.click(game->pointerX, game->pointerY))
					{
						confirming = false;
						oldSelectedButton = selectedButton; //preserved so we can restore the check to the last button
						pref.changeWidth(availableRes[selectedButton].x);
						pref.changeHeight(availableRes[selectedButton].y);
						pref.changeWindow(game->useWindow);
					}
					if (button_no.click(game->pointerX, game->pointerY)) //if player doesn't like new screen
					{
						//go back to old resolution
						game->setResolution(oldResX, oldResY);
						game->useWindow = oldWindow;
						game->changeScreen = true; 
						confirming = false;
						int j;
						for (j=0; j < totalResButtons; j++) //turn off all the checkboxes
						{
							button_res[j].turnOff();
						}
						selectedButton = oldSelectedButton;
						button_res[selectedButton].turnOn();
					}
				}
				
			}
			break;
		case SDL_MOUSEBUTTONUP:
			break;
		case SDL_KEYDOWN:
			keyPressed = SDL_GetKeyName( event.key.keysym.sym );
			if(keyPressed == "left shift" || keyPressed == "left meta")
			{
				//play bzzt sound to signify these are reserved
			}
			else
			{
				//change label for selected button
				button_UL.setLabel(keyPressed);
				//change key for selected player control
			}
			//skip animated intro? 
			break;
		case SDL_KEYUP:
			break;
		default:
			break;
	}
}

void sn_Phase_Options::animate()
{
	Uint32 ticksPassed;
	game->updateTicksPassed();
	ticksPassed = game->getTicksPassed();
	
	//Time based events
	time = time + ticksPassed;
	
	Uint32 startTime, endTime;
	float transitionTime; //will hold amount 
	
	if (confirming)
	{
		//timer
		//*this phase is ended when the timer (secondsLeft) runs out
		Sint16 previousSL = confirmSecondsLeft;
		confirmSecondsLeft = confirmTotalSeconds - (time-timeConfirmStarted)/1000; //time/1000 turns milliseconds into seconds

		label_timer.init( &game->typer, game->gameResX/2, game->gameResY/2, kSBBT_label, "%d", confirmSecondsLeft);
		label_timer.moveBy(-(label_timer.getWidth()/2), -(label_timer.getHeight()/2) + 100.0f);
		
		if (confirmSecondsLeft < 0)
		{
			//go back to old resolution
			game->setResolution(oldResX, oldResY);
			game->useWindow = oldWindow;
			game->changeScreen = true; 
			confirming = false;
			int i;
			for (i=0; i < totalResButtons; i++) //turn off all the checkboxes
			{
				button_res[i].turnOff();
			}
			selectedButton = oldSelectedButton;
			button_res[selectedButton].turnOn();
		}
	}
	
	if (time > 500 && erasingTitle == false && showingTitle == false)
	{
		showingTitle = true;
		timeTitleStarted = 500;
	}
}

void sn_Phase_Options::draw()
{
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***standard settings for rendering***
	//***if changed in other functions  ***
	//***they should then also be reset ***
	//***from those same functions      ***
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	//***    end standard settings      ***
	
	//***setup 3D projection
	gluLookAt(0,0,500-425,0,0,0,0,1,0);
	
	//***begin 3D drawing
	
	game->pStage->draw(45.0);
	
	//***end 3D drawing
	
	glPopMatrix();
	
	//***setup ortho projection for 2D overlay
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, game->gameResX, game->gameResY, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***begin 2D drawing
	
	int i;
	//draw windowed resolution buttons
	label_window.draw();
	
	//button_UL.draw();
	
	for (i=0; i<totalWindowButtons; i++)
	{
		button_res[i].draw();
	}
	
	if (totalWindowButtons != totalResButtons)
	{
		label_full.draw();
		int lastButtonShowing = CLAMP(firstFullButtonShowing+6, firstFullButtonShowing, totalResButtons);
		for (i=firstFullButtonShowing; i<lastButtonShowing; i++)
		{
			button_res[i].draw();
		}
	}
	
	button_music.draw();
	button_sound.draw();
	button_done.draw();
	if (totalResButtons > totalWindowButtons + 6)
	{
		button_up.draw();
		button_down.draw();
	}
	
	if (confirming)
	{
		//make everything drawn before this dimmed
		glLoadIdentity();
		glDisable(GL_TEXTURE_2D);
		glColor4f(0.9f,0.9f,1.0f,0.9f);
		glBegin(GL_QUADS);
		glVertex2f(2.0f,2.0f);
		glVertex2f(game->gameResX-2, 2.0f);
		glVertex2f(game->gameResX-2, game->gameResY - 2);
		glVertex2f(2.0f, game->gameResY - 2);
		glEnd();
		//return to standard render settings
		glEnable(GL_TEXTURE_2D);
		glColor4f(1.0f,1.0f,1.0f,1.0f);
		
		label_ok.draw();
		button_yes.draw();
		button_no.draw();
		label_timer.draw();
		
	}
	
	//everything after this point is not effected by the dimming of the confirm mode
	
	//draw line around edge of screen
	
	drawPointer();
	
	//****end 2D drawing
	
	//***restore matrices
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


void sn_Phase_Options::drawTitle()
{/*
	vf_setpentrack(8.0f, 0.0f);
	if (titleGlow > 0.0f)
	{	
		glAAFlush();
		glLoadIdentity();
		glBindTexture(GL_TEXTURE_2D, glAA_texture[2]);
		vf_setpenscale(titleScaleX, titleScaleY);
		vf_setpenloc( titleX, titleY );
		glAALineWidth(10);
		glAAColor4f(1.0f, 1.0f, 0.0f, titleGlow);
		vecString("SPACE BARRAGE");
	}
	
	glAAFlush();
	glLoadIdentity();
	glBindTexture(GL_TEXTURE_2D, glAA_texture[0]);
	vf_setpenscale(titleScaleX, titleScaleY);
	vf_setpenloc( titleX, titleY );
	glAALineWidth(3);
	glAAColor4f(1.0f, 1.0f, 0.2f, titleAlpha);
	vecString("SPACE BARRAGE");
	
	glAAFlush();
	glLoadIdentity();
	glBindTexture(GL_TEXTURE_2D, glAA_texture[2]);
	vf_setpentrack(1.0f, 0.0f);
	vf_setpenscale(1.5f, 1.5f);
	vf_setpenloc( 410, 500 );
	glAALineWidth(2);
	glAAColor4f(0.5f, 0.5f, 1.0f, titleAlpha);
	vecString("CLICK TO BEGIN");
	
	glAAFlush();
	glLoadIdentity();
	glBindTexture(GL_TEXTURE_2D, glAA_texture[2]);
	vf_setpentrack(1.0f, 0.0f);
	vf_setpenscale(1.5f, 1.5f);
	vf_setpenloc( 221, 700 );
	glAALineWidth(2);
	glAAColor4f(0.5f, 0.5f, 1.0f, titleAlpha);
	vecString("www.iDevGames.com - 21 days later: Vectorized entry");
*/
}


void sn_Phase_Options::drawPointer()
{
	/*
	glAAFlush();
	glLoadIdentity();
	glBindTexture(GL_TEXTURE_2D, glAA_texture[0]);
	glTranslatef(game->pointerX, game->pointerY, 0.0f);
	glAALineWidth(3);
	glAAColor4f(1.0f, 1.0f, 0.0f, 0.6f);
	glAABegin(GL_LINE_STRIP);
	glAAVertex2f(0.0f,0.0f);
	glAAVertex2f(0.0f,15.0f);
	glAAVertex2f(4.0f,10.0f);
	glAAVertex2f(12.0f,12.0f);
	glAAVertex2f(0.0f,0.0f);
	glAAEnd();
	glAAColor4f(1.0f, 1.0f, 0.0f, 0.2f);
	 */
}