//Copyright (c) 2004 Aaron Sullivan. All rights reserved.
#include "sn_Phase_Play.h"
#include <iostream>
using namespace std;

sn_Phase_Play::sn_Phase_Play( sn_Game * gameToUse) 
: sn_GamePhase( gameToUse ) //call sb_GamePhase constructor
{
	game = gameToUse;
	
	startButton.init(&game->typer, game->gameResX - 100, 725, kSBBT_smallAndBlue, "Restart"); //start the button near the center
	startButton.moveBy(-(startButton.getWidth()/2), 0.0f); //finish centering it based on its width
	//optionsButton.init(&game->typer, game->gameResX/2, 550, kSBBT_smallAndBlue, "OPTIONS");
	//optionsButton.moveBy(-(optionsButton.getWidth()/2), 0.0f);
	quitButton.init(&game->typer, 65, 725, kSBBT_smallAndBlue, "Quit");
	quitButton.moveBy(-(quitButton.getWidth()/2), 0.0f);
	
	cwButton.init(&game->typer, game->gameResX/2 - 44, 725, kSBBT_CW, "# ");
	cwButton.moveBy(-(cwButton.getWidth()/2), 0.0f);
	ccButton.init(&game->typer, game->gameResX/2 + 39, 725, kSBBT_CC, "# ");
	ccButton.moveBy(-(ccButton.getWidth()/2), 0.0f);
	inButton.init(&game->typer, game->gameResX/2, 705, kSBBT_upArrow, "# ");
	inButton.moveBy(-(inButton.getWidth()/2), 0.0f);
	outButton.init(&game->typer, game->gameResX/2, 735, kSBBT_downArrow, "# ");
	outButton.moveBy(-(outButton.getWidth()/2), 0.0f);
	
	zCameraAngle.set(45.0f);
	zCameraAngleTarget = 45.0f;
	cameraRotatingCW = false;
	cameraRotatingCCW = false;
	zCameraRotationSpeed = 0.2f;
	
	cameraZoom = 425.0f;
	cameraZoomTarget = 425.0f;
	cameraZoomingIn = false;
	cameraZoomingOut = false;
	cameraZoomSpeed = 1.0f;
	
	stageNames.push_back("title.stg");
	stageNames.push_back("first.stg");
	stageNames.push_back("hole_in_one.stg");
	stageNames.push_back("two.stg");
	stageNames.push_back("around_and_around.stg");
	stageNames.push_back("maze.stg");
	stageNames.push_back("rampmaze.stg");
	stageNames.push_back("jump.stg");
	stageNames.push_back("level1.stg");
	stageNames.push_back("ledges.stg");
	stageNames.push_back("q_ball.stg");
	stageNames.push_back("level3.stg");
	stageNames.push_back("bridge_the_gap.stg");
	stageNames.push_back("pushingtwo.stg");
	stageNames.push_back("plump_or_chump.stg");
	stageNames.push_back("longwayaround.stg");
	stageNames.push_back("dealingwithsteps.stg");
	stageNames.push_back("ice_bridge.stg");
	stageNames.push_back("hiddenexit.stg");
	stageNames.push_back("deja_vu.stg");
	stageNames.push_back("phat_pharoah.stg");
	stageNames.push_back("bowling.stg");
	game->totalPlayStages = stageNames.size() - 1;
	
	fanfarePlaying = false;
	
	reset();
}

sn_Phase_Play::~sn_Phase_Play()
{
	
}

void sn_Phase_Play::start()
{
	done = false;
	time = 0;
	
	//title variables
	titleX = 20.0f, titleY = 50.0f;
	titleAlpha = 1.0f;
	
	//phase specific
	showingTitle = true;
	erasingTitle = false;
	fadeAlpha = 1.0f;
	fadingInNewStage = true;
	fadingOutFinishedStage = false;
	
	for (int i=0; i<10; i++)
		keyhold[i] = false;
	
	currentStage = 1;
	if (game->playEditedStage)
	{
		game->pStage->load("temp.stg",currentStage);
	}
	else
	{
		game->pStage->load(stageNames[currentStage], currentStage);
	}
	
	captionLabel.init(&game->typer, titleX, titleY, kSBBT_smallAndBlue, game->pStage->getCaption()->c_str());
	
	
	game->musicBox.play(kMusic_gentle);
	fanfarePlaying = false;
	mouseInCameraArea = false;
	
	//start keeping track of Ticks passed
	game->updateTicksPassed();

}

void sn_Phase_Play::pause()
{
	game->musicBox.pauseToggle();
	paused = true;

}

void sn_Phase_Play::unpause()
{
	game->musicBox.pauseToggle();

	game->updateTicksPassed();
	game->getTicksPassed();
	paused = false;
}

void sn_Phase_Play::end()
{
	done = true;
}

void sn_Phase_Play::reset()
{
	showingTitle = true;
	keyhold.clear();
	keyhold.resize(sn_totalPlayerControls);
	currentStage = 0;
}

void sn_Phase_Play::processEvent( SDL_Event &event )
{	
	switch (event.type)
	{
		case SDL_MOUSEMOTION:
			if (game->pointerX > game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayer(sn_FUR);
			}
			else if (game->pointerX > game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayer(sn_FLR);
			}
			else if (game->pointerX < game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayer(sn_FUL);
			}
			else if (game->pointerX < game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
			{
				game->pStage->controlPlayer(sn_FLL);
			}
			
			//if (mouseInCameraArea)
			//{
				cwButton.rollover(game->pointerX, game->pointerY);
				ccButton.rollover(game->pointerX, game->pointerY);
				inButton.rollover(game->pointerX, game->pointerY);
				outButton.rollover(game->pointerX, game->pointerY);
			//}

			startButton.rollover(game->pointerX, game->pointerY);
			quitButton.rollover(game->pointerX, game->pointerY);

			
			break;
			
		//pointer position comes from game object ( which has its own processEvent function )
		case SDL_MOUSEBUTTONDOWN:
			erasingTitle = true;

			if ( event.button.button == SDL_BUTTON_LEFT )
			{
				if (startButton.click(game->pointerX, game->pointerY))
				{
					//erasingTitle = true; //when title is erased this phase will end()
					//showingTitle = false; 
					//timeTitleEnded = time;
					//game->musicBox.fadeOut(400);
					stageLoad(currentStage);
				}
				/*else if (optionsButton.click(game->pointerX, game->pointerY))
				{
					game->chooseOptions = true;
				}*/
				else if (quitButton.click(game->pointerX, game->pointerY))
				{
					done = true;
				}
				else if (mouseInCameraArea)
				{
					if (cwButton.click(game->pointerX, game->pointerY))
					{
						rotateCW();
					}
					else if (ccButton.click(game->pointerX, game->pointerY))
					{
						rotateCC();
					}
					else if (inButton.click(game->pointerX, game->pointerY))
					{
						zoomIn();
					}
					else if (outButton.click(game->pointerX, game->pointerY))
					{
						zoomOut();
					}
				}
				else if (game->pointerX > game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
				{
					keyhold[sn_UR] = true;
				}
				else if (game->pointerX > game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
				{
					keyhold[sn_LR] = true;
				}
				else if (game->pointerX < game->gameResX/2 && game->pointerY < game->gameResY/2 + 20 )
				{
					keyhold[sn_UL] = true;
				}
				else if (game->pointerX < game->gameResX/2 && game->pointerY >= game->gameResY/2 + 20 )
				{
					keyhold[sn_LL] = true;
				}
			}
			else if (event.button.button == SDL_BUTTON_RIGHT )
			{
				keyhold[sn_Shoot] = true;
				game->pStage->controlPlayer(sn_Shoot);
			}
				
			if( event.button.button == SDL_BUTTON_WHEELUP)
			{
				zoomIn();
			}
			if( event.button.button == SDL_BUTTON_WHEELDOWN)
			{
				zoomOut();
			}
			break;
		case SDL_MOUSEBUTTONUP:
			if ( event.button.button == SDL_BUTTON_LEFT )
			{
				keyhold[sn_UR] = false;
				keyhold[sn_LR] = false;
				keyhold[sn_UL] = false;
				keyhold[sn_LL] = false;
			}
			else if (event.button.button == SDL_BUTTON_RIGHT )
			{
				keyhold[sn_Shoot] = false;
			}
			break;
		case SDL_KEYDOWN:
			erasingTitle = true;
			/*
			if (event.key.keysym.sym == SDLK_r)
			{
				game->pStage->toggleRecording();
			}
			else if (event.key.keysym.sym == SDLK_p)
			{
				game->pStage->startPlayback();
			}
			*/
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				keyhold[sn_Shoot] = true;
				game->pStage->controlPlayer(sn_Shoot);
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FUL]=true;
				}
				else
				{
					keyhold[sn_UL] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FUR]=true;
				}
				else
				{
					keyhold[sn_UR] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FLR]=true;
				}
				else
				{
					keyhold[sn_LR] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
				{
					keyhold[sn_FLL]=true;
				}
				else
				{
					keyhold[sn_LL] = true;
				}
			}
			else if (event.key.keysym.sym == SDLK_RIGHT)
			{
				rotateCW();
			}
			else if (event.key.keysym.sym == SDLK_LEFT)
			{
				rotateCC();
			}	
			else if (event.key.keysym.sym == SDLK_UP)
			{
				zoomIn();
			}
			else if (event.key.keysym.sym == SDLK_DOWN)
			{
				zoomOut();
			}
			else if (event.key.keysym.sym == SDLK_c)
			{
				game->pStage->toggleDrawColliders();
			}
			else if (event.key.keysym.sym == SDLK_m)
			{
				game->pStage->toggleDrawMovementArray();
			}
			break;
		case SDL_KEYUP:
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				keyhold[sn_Shoot] = false;
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				keyhold[sn_UL] = false;
				keyhold[sn_FUL] = false;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				keyhold[sn_UR] = false;
				keyhold[sn_FUR] = false;
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				keyhold[sn_LR] = false;
				keyhold[sn_FLR] = false;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				keyhold[sn_LL] = false;
				keyhold[sn_FLL] = false;
			}
			break;
		default:
			break;
	}
}

void sn_Phase_Play::animate()
{
	Uint32 ticksPassed;
	game->updateTicksPassed();
	ticksPassed = game->getTicksPassed();
	
	//Time based events
	time = time + ticksPassed;
	
	Uint32 startTime, endTime;
	float transitionTime; //will hold amount 
	
	if (fanfarePlaying && false == game->musicBox.isPlaying())
	{
		fanfarePlaying = false;
		game->musicBox.play(kMusic_gentle);
	}
	
	if (keyhold[sn_UL] == true)
	{
		game->pStage->controlPlayer(sn_UL);
	}
	else if (keyhold[sn_UR] == true)
	{
		game->pStage->controlPlayer(sn_UR);
	}
	else if (keyhold[sn_LR] == true)
	{
		game->pStage->controlPlayer(sn_LR);
	}
	else if (keyhold[sn_LL] == true)
	{
		game->pStage->controlPlayer(sn_LL);
	}
	else if (keyhold[sn_FUL] == true)
	{
		game->pStage->controlPlayer(sn_FUL);
	}
	else if (keyhold[sn_FUR] == true)
	{
		game->pStage->controlPlayer(sn_FUR);
	}
	else if (keyhold[sn_FLR] == true)
	{
		game->pStage->controlPlayer(sn_FLR);
	}
	else if (keyhold[sn_FLL] == true)
	{
		game->pStage->controlPlayer(sn_FLL);
	}

	if (!fadingOutFinishedStage)
		game->pStage->animate( ticksPassed );
	
	if (game->pStage->isFailed())
	{
		//if (!fanfarePlaying)
		//{
		//	fanfarePlaying = true;
		//	game->musicBox.play(kMusic_failure);
		//}
		stageLoad(currentStage);
	}
	
	if (game->pStage->isCompleted() && !fadingOutFinishedStage)
	{
		//fanfarePlaying = true;
		//game->musicBox.play(kMusic_victory);
		fadingOutFinishedStage = true;
		timeStageEnded = time;
	}
	
	
	if (cameraRotatingCW || cameraRotatingCCW)
	{
		if (zCameraAngle.rotateTowardsTarget( zCameraRotationSpeed * ticksPassed, zCameraAngleTarget))
		{
			cameraRotatingCCW = false;
			cameraRotatingCW = false;
		}
	}
	if (cameraZoomingIn)
	{
		cameraZoom += cameraZoomSpeed * ticksPassed;
		if (cameraZoom > cameraZoomTarget)
		{
			cameraZoom = cameraZoomTarget;
			cameraZoomingIn = false;
		}
	}
	else if (cameraZoomingOut)
	{
		cameraZoom -= cameraZoomSpeed * ticksPassed;
		if (cameraZoom < cameraZoomTarget)
		{
			cameraZoom = cameraZoomTarget;
			cameraZoomingOut = false;
		}
	}
	
	if (showingTitle && erasingTitle)
	{
		showingTitle = false;
		timeTitleEnded = time;
	}
	
	if (fadingInNewStage)
	{
		startTime = 0; endTime = 250; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			fadeAlpha -= 1.0f/transitionTime*ticksPassed;
		}
		if (fadeAlpha <= 0.0f || time > endTime) 
		{
			fadeAlpha = 0.0f;
			fadingInNewStage = false;
			//timeFadeInEnded = time;
		}
	}
	
	
	//if player is at exit fadingOutFinishedStage is activated
	if (fadingOutFinishedStage)
	{
		startTime = timeStageEnded; endTime = timeStageEnded + 250; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time <= endTime)
		{
			fadeAlpha += 1.0f/transitionTime*ticksPassed;
		}
		if (fadeAlpha >= 1.0f || time > endTime) 
		{
			fadeAlpha = 1.0f;
			currentStage++;
			if (currentStage < (int)stageNames.size())
			{
				stageLoad(currentStage);
			}
			else 
				done = true;
		}
		
	}
		
	if (erasingTitle)
	{
		//vanish title
		startTime = timeTitleEnded; endTime = timeTitleEnded+1000; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			titleAlpha -= 1.0f/transitionTime*ticksPassed;
			if (titleAlpha < 0.0f) titleAlpha = 0.0f;
		}
		if (time >= endTime) {
			titleAlpha = 0.0f;
			erasingTitle = false;
		}
	}
	
	if (game->pointerX > 400 && game->pointerX < 620 && game->pointerY > 675)
		mouseInCameraArea = true;
	else
		mouseInCameraArea = false;
}

void sn_Phase_Play::draw()
{
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***standard settings for rendering***
	//***if changed in other functions  ***
	//***they should then also be reset ***
	//***from those same functions      ***
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	//***    end standard settings      ***
	
	//***setup 3D projection
	gluLookAt(0,0,500-cameraZoom,0,0,0,0,1,0);
	
	//***begin 3D drawing
	
	game->pStage->draw(zCameraAngle.get());
	
	//***end 3D drawing
	
	glPopMatrix();
	
	//***setup ortho projection for 2D overlay
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, game->gameResX, game->gameResY, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***begin 2D drawing
	glDisable(GL_DEPTH_TEST);
	if (showingTitle || erasingTitle)
	{	
		drawTitle();
	}
	
	startButton.draw();
	//optionsButton.draw();
	quitButton.draw();
	if (!mouseInCameraArea) glColor4f(1.0f,1.0f,1.0f,0.3f);
	cwButton.draw();
	if (!mouseInCameraArea) glColor4f(1.0f,1.0f,1.0f,0.3f);
	ccButton.draw();
	if (!mouseInCameraArea) glColor4f(1.0f,1.0f,1.0f,0.3f);
	inButton.draw();
	if (!mouseInCameraArea) glColor4f(1.0f,1.0f,1.0f,0.3f);
	outButton.draw();
	glEnable(GL_DEPTH_TEST);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
		
	//drawPointer();
	if (fadeAlpha > 0.0f)
	{	
		glDisable(GL_TEXTURE_2D);
		glColor4f(0.0f,0.0f,0.0f,fadeAlpha);
		glPushMatrix();
		glBegin(GL_QUADS);
		glVertex3f(0,0,0);
		glVertex3f(game->gameResX,0,0);
		glVertex3f(game->gameResX,game->gameResY,0);
		glVertex3f(0,game->gameResY,0);
		glEnd();
		glPopMatrix();
		glEnable(GL_TEXTURE_2D);
		glColor4f(1,1,1,1);
	}
	//****end 2D drawing
	
	//***restore matrices
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();	
}


void sn_Phase_Play::drawTitle()
{
	glColor4f(1.0f,1.0f,1.0f,titleAlpha);
	captionLabel.draw();
	glColor4f(1.0f,1.0f,1.0f,1.0f);
}

void sn_Phase_Play::drawPointer()
{
}

void sn_Phase_Play::stageLoad( int stageNumber )
{
	game->pStage->load(stageNames[stageNumber], stageNumber);
	captionLabel.init(&game->typer, titleX, titleY, kSBBT_smallAndBlue, game->pStage->getCaption()->c_str());
	time = 0;
	showingTitle = true;
	erasingTitle = false;
	titleAlpha = 1.0;
	fadeAlpha = 1.0;
	fadingOutFinishedStage = false;
	fadingInNewStage = true;
	
	keyhold[sn_Shoot] = false;
	keyhold[sn_UL] = false;
	keyhold[sn_FUL] = false;
	keyhold[sn_UR] = false;
	keyhold[sn_FUR] = false;
	keyhold[sn_LR] = false;
	keyhold[sn_FLR] = false;
	keyhold[sn_LL] = false;
	keyhold[sn_FLL] = false;
	
	zCameraAngle.set(45.0f);
	zCameraAngleTarget = 45.0f;
	cameraRotatingCW = false;
	cameraRotatingCCW = false;
	zCameraRotationSpeed = 0.2f;
	
	cameraZoom = 425.0f;
	cameraZoomTarget = 425.0f;
	cameraZoomingIn = false;
	cameraZoomingOut = false;
	cameraZoomSpeed = 1.0f;
}

void sn_Phase_Play::zoomIn()
{
	if (!cameraZoomingIn)
	{
		cameraZoomTarget += 50;
		if (cameraZoomTarget <= 500)
		{
			cameraZoomingIn = true;
			cameraZoomingOut = false;
		}
		else cameraZoomTarget -= 50;
	}
}

void sn_Phase_Play::zoomOut()
{
	if (!cameraZoomingOut)
	{
		cameraZoomTarget -= 50;
		if (cameraZoomTarget >= 0)
		{
			cameraZoomingIn = false;
			cameraZoomingOut = true;
		}
		else cameraZoomTarget += 50;
	}
}

void sn_Phase_Play::rotateCW()
{
	if (!cameraRotatingCW)
	{
		zCameraAngleTarget -= 90.0f;
		zCameraAngleHalfwayToTarget = zCameraAngleTarget + 45.0f;
		if (zCameraAngleTarget < 0)
		{
			zCameraAngleTarget = 315;
		}
		cameraRotatingCW = true;
		cameraRotatingCCW = false;
	}
}

void sn_Phase_Play::rotateCC()
{
	if (!cameraRotatingCCW)
	{
		zCameraAngleTarget += 90.0f;
		zCameraAngleHalfwayToTarget = zCameraAngleTarget - 45.0f;
		cameraRotatingCCW = true;
		cameraRotatingCW = false;
	}
}