//Copyright (c) 2004 Aaron Sullivan. All rights reserved.
#include "sn_Phase_Title.h"
#include <iostream>
using namespace std;

sn_Phase_Title::sn_Phase_Title( sn_Game * gameToUse) 
: sn_GamePhase( gameToUse ) //call sb_GamePhase constructor
{
	game = gameToUse;
	
	startButton.init(&game->typer, game->gameResX/2, 500, kSBBT_smallAndBlue, "Start"); //start the button near the center
	startButton.moveBy(-(startButton.getWidth()/2), 0.0f); //finish centering it based on its width
	optionsButton.init(&game->typer, game->gameResX/2, 550, kSBBT_smallAndBlue, "Options");
	optionsButton.moveBy(-(optionsButton.getWidth()/2), 0.0f);
	quitButton.init(&game->typer, game->gameResX/2, 600, kSBBT_smallAndBlue, "Quit");
	quitButton.moveBy(-(quitButton.getWidth()/2), 0.0f);
	editButton.init(&game->typer, 170, 695, kSBBT_tinyAndBlue, "uDevGames 2004 entry- visit uDevGames.com for more games!");
	
	zCameraAngle.set(45.0f);
	zCameraAngleTarget = 45.0f;
	cameraRotatingCW = false;
	cameraRotatingCCW = false;
	zCameraRotationSpeed = 0.2f;
	
	cameraZoom = 425.0f;
	cameraZoomTarget = 425.0f;
	cameraZoomingIn = false;
	cameraZoomingOut = false;
	cameraZoomSpeed = 1.0f;
	
	reset();
}

sn_Phase_Title::~sn_Phase_Title()
{
	
}

void sn_Phase_Title::start()
{
	done = false;
	time = 0;
	
	//title variables
	titleX = 285.0f, titleY = 300.0f;
	titleAlpha = 0.8f;
	titleScaleX = 50.0f;
	titleScaleY = 8.0f;
	titleGlow = 0.0f;
	
	//phase specific
	showingTitle = true;
	erasingTitle = false;
	
	for (int i=0; i<sn_totalPlayerControls; i++)
		keyhold[i] = false;
	
	game->currentStage=0;
	game->loadTitleStage();
	
	game->textures.load("titlenew.png");
	titleTextureID = game->textures.getID("titlenew.png");
	titleSegList.init(titleTextureID);
	titleSegList.load("titlenew.tsg");
	
	udgAnim.init(&game->textures);
	udgAnim.load("udg.ans");
	udg.init(&udgAnim);
	udg.startSequence("still");
	
	game->musicBox.play(kMusic_title);
	
	//start keeping track of Ticks passed
	game->updateTicksPassed();
	
	game->pStage->startPlayback();
	
}

void sn_Phase_Title::pause()
{
	game->musicBox.pauseToggle();
	paused = true;
	
}

void sn_Phase_Title::unpause()
{
	game->musicBox.pauseToggle();
	
	game->updateTicksPassed();
	game->getTicksPassed();
	paused = false;
}

void sn_Phase_Title::end()
{
	done = true;
}

void sn_Phase_Title::reset()
{
	showingTitle = false;
	keyhold.clear();
	keyhold.resize(sn_totalPlayerControls);
}

void sn_Phase_Title::processEvent( SDL_Event &event )
{
	switch (event.type)
	{
		case SDL_MOUSEMOTION:
			startButton.rollover(game->pointerX, game->pointerY);
			optionsButton.rollover(game->pointerX, game->pointerY);
			quitButton.rollover(game->pointerX, game->pointerY);
			break;
		//pointer position comes from game object ( which has its own processEvent function )
		case SDL_MOUSEBUTTONDOWN:
			if (showingTitle)
			{
				if (startButton.click(game->pointerX, game->pointerY))
				{
					erasingTitle = true; //when title is erased this phase will end()
					showingTitle = false;
					game->pStage->stopPlayback();
					timeTitleEnded = time;
					game->musicBox.fadeOut(400);
				}
				if (optionsButton.click(game->pointerX, game->pointerY))
				{
					game->chooseOptions = true;
					game->pStage->stopPlayback();
				}
				if (quitButton.click(game->pointerX, game->pointerY))
				{
					game->quit = true;
				}
				if (editButton.click(game->pointerX, game->pointerY))
				{
					game->edit = true;
					game->pStage->stopPlayback();
					timeTitleEnded = time;
					game->musicBox.fadeOut(400);
				}
				
			}
			if (game->pStage->isRecording())
			{
				if( event.button.button == SDL_BUTTON_WHEELUP)
				{
					if (!cameraZoomingIn)
					{
						cameraZoomTarget += 50;
						if (cameraZoomTarget <= 500)
						{
							cameraZoomingIn = true;
							cameraZoomingOut = false;
						}
						else cameraZoomTarget -= 50;
					}
				}
				if( event.button.button == SDL_BUTTON_WHEELDOWN)
				{
					if (!cameraZoomingOut)
					{
						cameraZoomTarget -= 50;
						if (cameraZoomTarget >= -100)
						{
							cameraZoomingIn = false;
							cameraZoomingOut = true;
						}
						else cameraZoomTarget += 50;
					}
					
				}
			}
			break;
		case SDL_MOUSEBUTTONUP:
			break;
		case SDL_KEYDOWN:
			if (event.key.keysym.sym == SDLK_e && 
				(event.key.keysym.mod & KMOD_META || event.key.keysym.mod & KMOD_CTRL ))
			{
				game->edit = true;
				game->pStage->stopPlayback();
				timeTitleEnded = time;
				game->musicBox.fadeOut(400);
			}
			else if (event.key.keysym.sym == SDLK_r)
			{
				game->pStage->toggleRecording();
			}
			else if (event.key.keysym.sym == SDLK_p)
			{
				game->pStage->startPlayback();
			}
			else if (game->pStage->isRecording())
			{
				if (event.key.keysym.sym == SDLK_v) 
				{
					game->pStage->controlPlayer(sn_E_confident);
				}
				else if (event.key.keysym.sym == SDLK_b)
				{
					game->pStage->controlPlayer(sn_E_sad);
				}
				else if (event.key.keysym.sym == SDLK_n) 
				{
					game->pStage->controlPlayer(sn_E_scared);
				}
				else if (event.key.keysym.sym == SDLK_h)
				{
					game->pStage->controlPlayer(sn_E_happy);
				}
				if (event.key.keysym.sym == SDLK_SPACE)
				{
					//done = true;
					keyhold[sn_Shoot] = true;
					game->pStage->controlPlayer(sn_Shoot);
				}
				else if (event.key.keysym.sym == SDLK_a)
				{
					if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
					{
						keyhold[sn_FUL]=true;
					}
					else
					{
						keyhold[sn_UL] = true;
					}
				}
				else if (event.key.keysym.sym == SDLK_w)
				{
					if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
					{
						keyhold[sn_FUR]=true;
					}
					else
					{
						keyhold[sn_UR] = true;
					}
				}
				else if (event.key.keysym.sym == SDLK_d)
				{
					if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
					{
						keyhold[sn_FLR]=true;
					}
					else
					{
						keyhold[sn_LR] = true;
					}
				}
				else if (event.key.keysym.sym == SDLK_s)
				{
					if (event.key.keysym.mod & KMOD_LSHIFT || event.key.keysym.mod & KMOD_RSHIFT)
					{
						keyhold[sn_FLL]=true;
					}
					else
					{
						keyhold[sn_LL] = true;
					}
				}
				else if (event.key.keysym.sym == SDLK_RIGHT)
				{
					if (!cameraRotatingCW)
					{
						zCameraAngleTarget -= 90.0f;
						zCameraAngleHalfwayToTarget = zCameraAngleTarget + 45.0f;
						if (zCameraAngleTarget < 0)
						{
							zCameraAngleTarget = 315;
						}
						cameraRotatingCW = true;
						cameraRotatingCCW = false;
					}
				}
				else if (event.key.keysym.sym == SDLK_LEFT)
				{
					if (!cameraRotatingCCW)
					{
						zCameraAngleTarget += 90.0f;
						zCameraAngleHalfwayToTarget = zCameraAngleTarget - 45.0f;
						cameraRotatingCCW = true;
						cameraRotatingCW = false;
					}
				}
				else if (event.key.keysym.sym == SDLK_UP)
				{
					if (!cameraZoomingIn)
					{
						cameraZoomTarget += 50;
						if (cameraZoomTarget <= 500)
						{
							cameraZoomingIn = true;
							cameraZoomingOut = false;
						}
						else cameraZoomTarget -= 50;
					}
					
				}
				else if (event.key.keysym.sym == SDLK_DOWN)
				{
					if (!cameraZoomingOut)
					{
						cameraZoomTarget -= 50;
						if (cameraZoomTarget >= 0)
						{
							cameraZoomingIn = false;
							cameraZoomingOut = true;
						}
						else cameraZoomTarget += 50;
					}
				}
			}
			break;
		case SDL_KEYUP:
			if (event.key.keysym.sym == SDLK_SPACE)
			{
				keyhold[sn_Shoot] = false;
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				keyhold[sn_UL] = false;
				keyhold[sn_FUL] = false;
			}
			else if (event.key.keysym.sym == SDLK_w)
			{
				keyhold[sn_UR] = false;
				keyhold[sn_FUR] = false;
			}
			else if (event.key.keysym.sym == SDLK_d)
			{
				keyhold[sn_LR] = false;
				keyhold[sn_FLR] = false;
			}
			else if (event.key.keysym.sym == SDLK_s)
			{
				keyhold[sn_LL] = false;
				keyhold[sn_FLL] = false;
			}
			
			break;
		default:
			break;
	}
}

void sn_Phase_Title::animate()
{
	Uint32 ticksPassed;
	game->updateTicksPassed();
	ticksPassed = game->getTicksPassed();
	
	//Time based events
	time = time + ticksPassed;
	
	Uint32 startTime, endTime;
	float transitionTime; //will hold amount 
	
	udg.animate(ticksPassed);
	
	if (time > 500 && erasingTitle == false && showingTitle == false)
	{
		showingTitle = true;
		timeTitleStarted = 500;
	}
	
	if (showingTitle)
	{
		
		if (keyhold[sn_UL] == true)
		{
			game->pStage->controlPlayer(sn_UL);
		}
		else if (keyhold[sn_UR] == true)
		{
			game->pStage->controlPlayer(sn_UR);
		}
		else if (keyhold[sn_LR] == true)
		{
			game->pStage->controlPlayer(sn_LR);
		}
		else if (keyhold[sn_LL] == true)
		{
			game->pStage->controlPlayer(sn_LL);
		}
		else if (keyhold[sn_FUL] == true)
		{
			game->pStage->controlPlayer(sn_FUL);
		}
		else if (keyhold[sn_FUR] == true)
		{
			game->pStage->controlPlayer(sn_FUR);
		}
		else if (keyhold[sn_FLR] == true)
		{
			game->pStage->controlPlayer(sn_FLR);
		}
		else if (keyhold[sn_FLL] == true)
		{
			game->pStage->controlPlayer(sn_FLL);
		}
		
		
		game->pStage->animate( ticksPassed );
		
		if (game->pStage->isFailed())
		{
			game->reloadStage();
			//playerTries -= 1;
			//if (playerTries < 0) game->gameover = true;
		}
		
		if (game->pStage->isCompleted())
		{
			game->loadNextStage();
		}
		
		if (cameraRotatingCW || cameraRotatingCCW)
		{
			/*if (zCameraAngle.distToTarget( zCameraRotationSpeed * ticksPassed, zCameraAngleHalfwayToTarget ) < 0
				&& resorted == false)
			{
				game->pStage->resort();
				resorted = true;
			}
*/
			if (zCameraAngle.rotateTowardsTarget( zCameraRotationSpeed * ticksPassed, zCameraAngleTarget))
			{
				cameraRotatingCCW = false;
				cameraRotatingCW = false;
				//resorted = false;
			}
		}
		if (cameraZoomingIn)
		{
			cameraZoom += cameraZoomSpeed * ticksPassed;
			if (cameraZoom > cameraZoomTarget)
			{
				cameraZoom = cameraZoomTarget;
				cameraZoomingIn = false;
			}
		}
		else if (cameraZoomingOut)
		{
			cameraZoom -= cameraZoomSpeed * ticksPassed;
			if (cameraZoom < cameraZoomTarget)
			{
				cameraZoom = cameraZoomTarget;
				cameraZoomingOut = false;
			}
		}
		//animate the title of the phase
		//"stretch in"
		startTime = timeTitleStarted; endTime = timeTitleStarted+500; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			//the total amount to change (over whole time) is divided total time of transition and multiplied by how many ticks passed 
			//then that portion (which is the fraction needed for THIS update) is subtracted from the current value
			titleScaleX -= 47.0f/transitionTime*ticksPassed;
		}
		if (time > endTime) titleScaleX = 3.0f; //take care of inaccuracy of method
		
		//glow the title
		startTime = timeTitleStarted+500; endTime = timeTitleStarted+1000; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			if (titleGlow == 0.0f)
			{
				titleGlow = 1.0f;
			}
			titleGlow -= 1.0/transitionTime*ticksPassed;
		}
		if (time > endTime) 
		{
			titleGlow = 0.0f;
		}
	}
	
	if (erasingTitle)
	{
		//raise and vanish title
		startTime = timeTitleEnded; endTime = timeTitleEnded+500; transitionTime = (float)(endTime - startTime - 1);
		if (time > startTime && time < endTime)
		{
			titleY -= 200.0f/transitionTime*ticksPassed; 
			titleAlpha -= 0.8f/transitionTime*ticksPassed;
			if (titleAlpha < 0.0f) titleAlpha = 0.0f;
			titleScaleY += 30.0f/transitionTime*ticksPassed;
		}
		if (time >= endTime) {
			titleY = 0.0;
			titleAlpha = 0.0f;
			titleScaleY = 4.0f;
			end();
		}
	}
}

void sn_Phase_Title::draw()
{
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	//***standard settings for rendering***
	//***if changed in other functions  ***
	//***they should then also be reset ***
	//***from those same functions      ***
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	//***    end standard settings      ***
	
	//***setup 3D projection
	gluLookAt(0,0,500-cameraZoom,0,0,0,0,1,0);
	
	//***begin 3D drawing
	
	game->pStage->draw(zCameraAngle.get());
	
	//***end 3D drawing
	
	glPopMatrix();
	
	//***setup ortho projection for 2D overlay
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, game->gameResX, game->gameResY, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***begin 2D drawing
	
	if (showingTitle || erasingTitle)
	{
		drawTitle();
	}
	
	startButton.draw();
	optionsButton.draw();
	quitButton.draw();
	editButton.draw();

	//blue chip pointer
	/*
	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	glColor4f(0.4f, 0.5f, 1.0f, 0.8f);
	glBegin(GL_TRIANGLES);
	glVertex3f(game->pointerX, game->pointerY, 0);
	glVertex3f(game->pointerX + 30, game->pointerY + 30, 0);
	glVertex3f(game->pointerX + 15, game->pointerY + 30, 0);
	glEnd();
	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glPopMatrix();
	*/
	
	glPushMatrix();
	glTranslatef(103, 605, 0);
	udg.draw(100,0,1);
	glPopMatrix();
		
	glPushMatrix();
	glTranslatef(512,140,0);
	titleSegList.draw(0,1.5,0,1);
	glPopMatrix();
	/*
	glPushMatrix();
	glTranslatef(530,32,0);
	titleSegList.draw(1,100,0,1);
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(280,20,0);
	titleSegList.draw(0,100,0,1);
	glPopMatrix();
	*/

	
	drawPointer();
	
	//****end 2D drawing
	
	//***restore matrices
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


void sn_Phase_Title::drawTitle()
{
 /*	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glColor4f(1.0f,0.0f,0.0f,0.5f);
	glBegin(GL_QUADS);
	glVertex3f(10,10,0);
	glVertex3f(10,700,0);
	glVertex3f(950,700,0);
	glVertex3f(950,10,0);
	glEnd();
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
  */
}

void sn_Phase_Title::drawPointer()
{
}