#include "sn_Player.h"
#include <iostream>
//#include "SDL_OpenGL.h"
using namespace std;

void sn_Player::init(sn_TextureCollection * pTextureCollection)
{
	
	
	pTextures = pTextureCollection;
	
	ballAnimList.init(pTextures);
	ballAnimList.load("playerball.ans");
	ballSprite.init(&ballAnimList);
	ballAwaySprite.init(&ballAnimList);
	ballSprite.startSequence("rollToward");
	ballAwaySprite.startSequence("rollAway");
	ballSprite.pause();
	ballAwaySprite.pause();
	
	eyeAnimList.init(pTextures);
	eyeAnimList.load("playereyes.ans");
	eyeSprite.init(&eyeAnimList);
	eyeSprite.startSequence("stillToward");
	eyeAwaySprite.init(&eyeAnimList);
	eyeAwaySprite.startSequence("stillAway");

	
	x = 1;
	y = 1;
	z = 1;
	targetX = x;
	targetY = y;
	targetZ = z;
	deltaX = 0;
	deltaY = 0;
	deltaZ = 0;
	moving = false;
	bouncingBack = false;
	facing = 2;
	layer = 50;
	
	falling = false;
	
	size = 1.0;
	
	collider.init(x,y,z+size/2,0.8,0.8,0.8);
}

void sn_Player::moveTo( float absX, float absY, float absZ )
{
	x = absX; y = absY; z = absZ;
	targetX = x; targetY = y; targetZ = z;
	moving = false;
	bouncingBack = false;
	falling = false;
}

void sn_Player::draw( float rotation )
{
	//drawAlpha(rotation);
}

void sn_Player::drawAlpha( float rotation )
{
	int rotationID = (int)(rotation / 90);
	int drawingDir = 0;
	float scale = size;
	
	if (rotationID == facing)
	{
		drawingDir = 0;
	}
	else if (rotationID == facing + 1 || rotationID == facing - 3)
	{
		drawingDir = 3;
	}
	else if (rotationID == facing + 2 || rotationID == facing - 2)
	{
		drawingDir = 2;
	}
	else if (rotationID == facing + 3 || rotationID == facing - 1)
	{
		drawingDir = 1;
	}
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	if (drawingDir == 3) //towards left
	{
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. ***
		glTranslatef(0,0,0.6);
		
		glPushMatrix();
		glTranslatef(0.0,0.2*scale*1.5,0.0);
		ballSprite.draw(scale, false, false);
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(-0.25*scale, 0.6, 0.0);
		eyeSprite.draw();
		glPopMatrix();
	}
	else if (drawingDir == 2) // towards right
	{
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		glTranslatef(0,0,0.6);

		glPushMatrix();
		glTranslatef(0.0,0.2*scale*1.5,0.0);
		ballSprite.draw(scale, true, false);
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(0.25*scale, 0.6, 0.0);
		eyeSprite.draw(1.0, true, false);
		glPopMatrix();
	}
	else if (drawingDir == 1) //away right
	{
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		glTranslatef(0,0,0.6);
		
		glPushMatrix();
		glTranslatef(0.20*scale, 0.75, 0.0);
		eyeAwaySprite.draw(1.0, true, false);
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(0.0,0.2*scale*1.5,0.0);
		ballAwaySprite.draw(scale, false, false);
		glPopMatrix();
	}
	else if (drawingDir == 0) //away left
	{
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		glTranslatef(0,0,0.6);
		
		glPushMatrix();
		glTranslatef(-0.20*scale, 0.75, 0.0);
		eyeAwaySprite.draw();
		glPopMatrix();
		
		glPushMatrix();
		glTranslatef(0.0,0.2*scale*1.5,0.0);
		ballAwaySprite.draw(scale, true, false);
		glPopMatrix();
	}
	
	glPopMatrix();
}

void sn_Player::setTarget(float tx, float ty, float tz)
{
	if (!(tx == x && ty == y && tz == z))
	{
		moving = true;
		ballSprite.unpause();
		ballAwaySprite.unpause();
		targetX = tx;
		targetY = ty;
		targetZ = tz;
		deltaX = (targetX - x)/120;
		deltaY = (targetY - y)/120;
		deltaZ = (targetZ - z)/120;
		oldX = x;
		oldY = y;
		oldZ = z;
		if (falling) 
		{
			//deltaZ = 0; 
			//deltaX = deltaX*0.5; 
			//deltaY = deltaY*0.5;
		}
	}
}

void sn_Player::setTargetRelative(float dx, float dy, float dz)
{
	if (!(dx == 0 && dy == 0 && dz == 0))
	{
		moving = true;
		ballSprite.unpause();
		ballAwaySprite.unpause();
		targetX = x + dx;
		targetY = y + dy;
		targetZ = z + dz;
		deltaX = (targetX - x)/120;
		deltaY = (targetY - y)/120;
		deltaZ = (targetZ - z)/120;
		oldX = x;
		oldY = y;
		oldZ = z;
		if (falling) 
		{
			deltaZ = 0; 
			deltaX = deltaX*0.5; 
			deltaY = deltaY*0.5;
		}
	}
}

void sn_Player::setBounceTargetRelative(float dx, float dy, float dz)
{
	if (!(dx == 0 && dy == 0 && dz == 0))
	{
		moving = true;
		ballSprite.unpause();
		ballAwaySprite.unpause();
		bouncingBack = true;
		targetX = x + dx*0.5;
		targetY = y + dy*0.5;
		targetZ = z + dz*0.5;
		deltaX = (targetX - x)/60;
		deltaY = (targetY - y)/60;
		deltaZ = (targetZ - z)/60;
		oldX = x;
		oldY = y;
		oldZ = z;
		if (falling) 
		{
			targetX = x + dx * 0.75f;
			targetY = y + dy * 0.75f;
			oldZ = z+dz;
			oldX = x + dx*0.5;
			oldY = y + dy*0.5;
			deltaZ = 0; 
			deltaX = deltaX*0.5; 
			deltaY = deltaY*0.5;
		}
	}
}
	

bool sn_Player::isActive()
{
	return moving;
}

void sn_Player::changeExpression( int expression )
{
	currentExpression = expression;
	if (expression == sn_sad)
	{
		eyeSprite.startSequence("sadToward");
		eyeAwaySprite.startSequence("sadAway");
	}
	else if (expression == sn_confident)
	{
		eyeSprite.startSequence("stillToward");
		eyeAwaySprite.startSequence("stillAway");
	}
	else if (expression == sn_scared)
	{
		eyeSprite.startSequence("scaredToward");
		eyeAwaySprite.startSequence("stillAway");
	}
	else if (expression == sn_happy)
	{
		eyeSprite.startSequence("happyToward");
		eyeAwaySprite.startSequence("happyAway");
	}
	else
	{
		eyeSprite.startSequence("stillToward");
		eyeAwaySprite.startSequence("stillAway");
	}
}
		

void sn_Player::spit()
{
	eyeSprite.setLoop(false);
	eyeSprite.startSequence("spitToward");
	eyeAwaySprite.setLoop(false);
	eyeAwaySprite.startSequence("spitAway");
	spitting = true;
}

void sn_Player::animate( unsigned int ticksPassed )
{
	if (spitting && eyeSprite.atEnd())
	{
		spitting = false;
		eyeSprite.setLoop(true);
		eyeSprite.startSequence("stillToward");
		eyeAwaySprite.setLoop(true);
		eyeAwaySprite.startSequence("stillAway");
	}
		
	eyeSprite.animate(ticksPassed);
	eyeAwaySprite.animate(ticksPassed);
	ballSprite.animate(ticksPassed);
	ballAwaySprite.animate(ticksPassed);
	
	if (moving && !bouncingBack)
	{
		x += deltaX * ticksPassed;
		if (deltaX > 0 && x > targetX) {x = targetX; deltaX = 0;}
		if (deltaX < 0 && x < targetX) {x = targetX; deltaX = 0;}
		
		y += deltaY * ticksPassed;
		if (deltaY > 0 && y > targetY) {y = targetY; deltaY = 0;}
		if (deltaY < 0 && y < targetY) {y = targetY; deltaY = 0;}
		
		if (falling) {deltaZ -= 0.0004;}
		z += deltaZ * ticksPassed;
		if (deltaZ > 0 && z > targetZ) {z = targetZ; deltaZ = 0;}
		if (deltaZ < 0 && z < targetZ) {z = targetZ; deltaZ = 0;}
		
		if (falling && deltaZ == 0) //cheat
		{
			x = targetX;
			y = targetY;
			deltaX = 0;
			deltaY = 0;
		}
		
		if (deltaX == 0 && deltaY == 0 && deltaZ == 0)
		{
			moving = false;
			falling = false;
			ballSprite.pause();
			ballAwaySprite.pause();
		}
	}
	else if (moving && bouncingBack)
	{
		x += deltaX * ticksPassed;
		if (deltaX > 0 && x > targetX) {x = targetX; deltaX = 0;}
		if (deltaX < 0 && x < targetX) {x = targetX; deltaX = 0;}
		
		y += deltaY * ticksPassed;
		if (deltaY > 0 && y > targetY) {y = targetY; deltaY = 0;}
		if (deltaY < 0 && y < targetY) {y = targetY; deltaY = 0;}
		
		if (falling) {deltaZ -= 0.0004;}
		z += deltaZ * ticksPassed;
		if (deltaZ > 0 && z > targetZ) {z = targetZ; deltaZ = 0;}
		if (deltaZ < 0 && z < targetZ) {z = targetZ; deltaZ = 0;}
	
		if (falling && deltaZ == 0) //cheat
		{
			x = targetX;
			y = targetY;
			deltaX = 0;
			deltaY = 0;
		}
		
		if (deltaX == 0 && deltaY == 0 && deltaZ == 0)
		{
			bouncingBack = false;
			setTarget(oldX, oldY, oldZ);
			ballSprite.unpause();
			ballAwaySprite.unpause();
		}
	}
	
	collider.moveTo(x,y,z+0.5);
}