#include "sn_Projectile.h"
#include <iostream>

void sn_Projectile::init(sn_TextureCollection * pTextureCollection)
{
	pTextures = pTextureCollection;
	
	ballAnimList.init(pTextures);
	ballAnimList.load("playerball.ans");
	ballSprite.init(&ballAnimList);
	ballAwaySprite.init(&ballAnimList);
	ballSprite.startSequence("rollToward");
	ballAwaySprite.startSequence("rollAway");
	ballSprite.pause();
	ballAwaySprite.pause();
	
	x = 0;
	y = 0;
	z = 0;
	
	targetX = x;
	targetY = y;
	targetZ = z;
	deltaX = 0;
	deltaY = 0;
	deltaZ = 0;
	moving = false;
	
	facing = 0;
	layer = 50;
	
	speed = 0.01;
	size = 0.40;
	
	collider.init(x,y,z,size*0.25,size*0.25,size*0.25);
}

void sn_Projectile::draw( float rotation )
{
	//drawAlpha(rotation);
}

void sn_Projectile::drawAlpha( float rotation )
{
	if (!moving) return; //only draw projectiles that move so unused ones remain invisible
	
	int rotationID = (int)(rotation / 90);
	int drawingDir = 0;
	float scale = size;
	
	if (rotationID == facing)
	{
		drawingDir = 0;
	}
	else if (rotationID == facing + 1 || rotationID == facing - 3)
	{
		drawingDir = 3;
	}
	else if (rotationID == facing + 2 || rotationID == facing - 2)
	{
		drawingDir = 2;
	}
	else if (rotationID == facing + 3 || rotationID == facing - 1)
	{
		drawingDir = 1;
	}
	
	if (drawingDir == 3) //towards left
	{
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. ***
		//glTranslatef(0,0,0.6);
		
		glPushMatrix();
		//glTranslatef(0.0,0.6*scale*1.5,0.0);
		ballSprite.draw(scale, false, false);
		glPopMatrix();
		
		glPopMatrix();
	}
	else if (drawingDir == 2) // towards right
	{
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		//glTranslatef(0,0,0.6);
		
		glPushMatrix();
		//glTranslatef(0.0,0.6*scale*1.5,0.0);
		ballSprite.draw(scale, true, false);
		glPopMatrix();

		glPopMatrix();
	}
	else if (drawingDir == 1) //away right
	{
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		//glTranslatef(0,0,0.6);
		
		glPushMatrix();
		//glTranslatef(0.0,0.6*scale*1.5,0.0);
		ballAwaySprite.draw(scale, false, false);
		glPopMatrix();
		
		glPopMatrix();
	}
	else if (drawingDir == 0) //away left
	{
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glRotatef(-rotation,0.0f,0.0f,1.0f);
		glRotatef(60.0f,1.0f,0.0f,0.0f); //60 degree angle could change. *** 
		//glTranslatef(0,0,0.6);
		
		glPushMatrix();
		//glTranslatef(0.0,0.6*scale*1.5,0.0);
		ballAwaySprite.draw(scale, true, false);
		glPopMatrix();
		
		glPopMatrix();
	}
}

void sn_Projectile::launch(float xStart, float yStart, float zStart, int compassDir)
{
	x = xStart;
	y = yStart;
	z = zStart;
	
	collider.moveTo(x,y,z);
	
	if (compassDir == 0)
	{
		deltaY = speed;
		deltaX = 0.0f;
	}
	else if (compassDir == 1)
	{
		deltaY = 0.0f;
		deltaX = speed;
	}
	else if (compassDir == 2)
	{
		deltaY = -speed;
		deltaX = 0.0f;
	}
	else if (compassDir == 3)
	{
		deltaY = 0.0f;
		deltaX = -speed;
	}
	else
	{
		return;
	}
	
	ballSprite.unpause();
	ballAwaySprite.unpause();
	
	moving = true;
}

void sn_Projectile::registerHit()
{
	moving = false;
	ballSprite.pause();
	ballAwaySprite.pause();
}

bool sn_Projectile::isActive()
{
	return moving;
}

void sn_Projectile::animate( unsigned int ticksPassed )
{
	if (!moving) return;
	
	ballSprite.animate(ticksPassed);
	ballAwaySprite.animate(ticksPassed);

	oldX = x;
	oldY = y;
	oldZ = z;
	
	x += deltaX * ticksPassed;
	//if (deltaX > 0 && x > targetX) {x = targetX; deltaX = 0;}
	//if (deltaX < 0 && x < targetX) {x = targetX; deltaX = 0;}
	
	y += deltaY * ticksPassed;
	//if (deltaY > 0 && y > targetY) {y = targetY; deltaY = 0;}
	//if (deltaY < 0 && y < targetY) {y = targetY; deltaY = 0;}
	
	z += deltaZ * ticksPassed;
	//if (deltaZ > 0 && z > targetZ) {z = targetZ; deltaZ = 0;}
	//if (deltaZ < 0 && z < targetZ) {z = targetZ; deltaZ = 0;}
	
	collider.moveTo(x,y,z);
	
	if (deltaX == 0 && deltaY == 0 && deltaZ == 0)
	{
		moving = false;
		ballSprite.pause();
		ballAwaySprite.pause();
	}
}