#ifndef SN_PUSHABLEBLOCK
#define SN_PUSHABLEBLOCK

#include "sn_3DmodelCollection.h"
#include "sn_StageElement.h"
#include "sn_MovementArray.h"
#include "sn_MovementDeltas.h"
#include <string>
using namespace std;

class sn_PushableBlock : sn_StageElement
{
public:
	void init(sn_3DmodelCollection * p3DModelCollection, sn_MovementArray * pMA,
			  string modelFilename, string movementMapFilename);
	virtual void draw( float rotation );
	virtual void drawAlpha( float rotation );
	void moveTo( float x, float y, float z );
	void moveBy( float xOffset, float yOffset, float zOffset);

	bool isActive();
	bool hasArrived();

	void animate( unsigned int ticksPassed );
	
	void receivePush( int direction );
	sn_Collider getCollider();
	
	void setFalling( bool tf );
	bool isFalling();
	
	void fallOffStage();
	bool isFallingOffStage();
	
	void setGone( bool tf );
	bool isGone();
	
	void restoreOriginalMovementDeltas();
	void applyMovementDeltas();
	void recordOriginalMovementDeltas();
	
	float getX() const {return x;}
	float getY() const {return y;}
	float getZ() const {return z;}
		
private:
	float height;
	float halfheight;
	

	float targetX, targetY, targetZ;
	float deltaX, deltaY, deltaZ;
	float oldX, oldY, oldZ;
	float moveSpeed;
	
	int modelID;
	sn_3DmodelCollection * pModels;
	
	bool falling;
	bool fallingOffStage;
	bool gone;
	
	bool moving;
	bool arrived;
	
	sn_MovementArray * pMovementArray;
	sn_MovementDeltas blockMovementDeltas;
	sn_MovementDeltas originalMovementDeltas;
	
	void updateMovementDeltas();
};

#endif