#include "sn_PushableBlockCollection.h"

void sn_PushableBlockCollection::init( sn_3DmodelCollection * p3DModelCollection, sn_MovementArray * pMA, vector<sn_StageElement*> * hSE )
{
	pModels = p3DModelCollection;
	pMovementArray = pMA;
	hStageElements = hSE;
	clear();
}

void sn_PushableBlockCollection::clear()
{
	for (vector<sn_PushableBlock*>::iterator bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		delete (*bitr);
		(*bitr) == NULL;
	}
	blocks.clear();
}

int sn_PushableBlockCollection::load( const string &filename )
{
	ifstream datafile;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/stages/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		clear();
		modelFilenames.clear();
		movementMapFilenames.clear();
		string tempFilename;
		
		getline(datafile, tempFilename);
		
		while (tempFilename != "t x y z l f")
		{
			modelFilenames.push_back(tempFilename);
			getline(datafile, tempFilename);
			movementMapFilenames.push_back(tempFilename);
			getline(datafile, tempFilename);
		}
		
		int blockType;
		float bx, by, bz;
		int l,f;
		while (datafile >> blockType)
		{
			if (blockType < modelFilenames.size() && blockType >= 0)
			{
				datafile >> bx; datafile >> by; datafile >> bz; datafile >> l; datafile >> f;
				addNew(bx, by, bz, blockType);
			}
			else
			{
				//the type doesn't match the array so we can't use this row of data 
				datafile >> bx; datafile >> by; datafile >> bz; datafile >> l; datafile >> f;
			}
		}
		datafile.close();
		return true;
	}
	else
	{
		return false;
	}
	return false;	
}

int sn_PushableBlockCollection::save(const string &filename)
{
	ofstream datafile;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/stages/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::out);
	
	if (datafile.is_open())
	{
		for (int i=0; i < (int)modelFilenames.size(); ++i)
		{
			datafile << modelFilenames[i] << endl;
			datafile << movementMapFilenames[i] << endl;
		}
		
		datafile << "t x y z l f" << endl;
		
		for (vector<sn_PushableBlock*>::const_iterator itr = blocks.begin();
			 itr != blocks.end(); itr++)
		{
			datafile << 0 << ' '; //****THIS NEEDS TO BE CHANGED IN ORDER TO KEEP TRACK OF MORE TYPES OF BLOCKS
			datafile << (*itr)->getX() << ' ';
			datafile << (*itr)->getY() << ' ';
			datafile << (*itr)->getZ() << ' ';
			datafile << 5 << ' ';
			datafile << 0 << endl;
		}
		
		datafile.close();
		
		return true;
	}
	else
	{
		return false;
	}
		
}

sn_PushableBlock* sn_PushableBlockCollection::addNew( float x, float y, float z, int blockType )
{
	vector<sn_PushableBlock*>::iterator bitr;
	
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//restore originals that were replaced when blocks had moved where they WERE
		(*bitr)->restoreOriginalMovementDeltas();
	}
	
	sn_PushableBlock* newBlock = new sn_PushableBlock;
	newBlock->init( pModels, pMovementArray, modelFilenames[blockType], movementMapFilenames[blockType]);
	newBlock->moveTo( x, y, z );
	blocks.push_back(newBlock);

	//add it into the stageElements list so it can be drawn
	hStageElements->push_back((sn_StageElement*)newBlock);
	cout << blocks.size();
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//record unmodified deltas where block is before writing over
		(*bitr)->recordOriginalMovementDeltas();
	}
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//write over with the deltas appropriate for the block
		(*bitr)->applyMovementDeltas();
	}
	
	return newBlock;
	
}

bool sn_PushableBlockCollection::eraseArea( float x, float y, float z)
{
	bool erased = false;
	vector<sn_PushableBlock*>::iterator bitr = blocks.begin();
	while( bitr != blocks.end() )
	{
		if ((*bitr)->getX() == x && (*bitr)->getY() == y && (*bitr)->getZ() == z)
		{
			(*bitr)->restoreOriginalMovementDeltas();
			
			//remove this block from the stage elements so it doesn't attempt to draw it
			for (vector<sn_StageElement*>::iterator eitr = hStageElements->begin();
				 eitr != hStageElements->end(); ++eitr)
			{
				if ((sn_StageElement*)(*bitr) == (sn_StageElement*)(*eitr))
				{
					eitr = hStageElements->erase(eitr);
				}
			}
			
			//free up memory that this block took up
			delete (*bitr);
			(*bitr) = NULL;
			
			//remove from the block list now
			bitr = blocks.erase(bitr);
			
			erased = true;
		}
		else
			++bitr;
	}
	return erased;
}

//if at least one block has just moved, we need to update ALL of the movementDeltas that are effected by ALL
//of the blocks so that the movementDeltas don't pick up other blocks as part of their original movementDeltas
//and then writing them back later overwriting information from the scenery's permanent movementArray
void sn_PushableBlockCollection::updateMovementDeltas()
{
	vector<sn_PushableBlock*>::iterator bitr;
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//restore originals that were replaced when blocks had moved where they WERE
		(*bitr)->restoreOriginalMovementDeltas();
	}
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//record unmodified
		(*bitr)->recordOriginalMovementDeltas();
	}
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		(*bitr)->applyMovementDeltas();
	}
}

void sn_PushableBlockCollection::animate( unsigned int ticksPassed )
{
	sn_Collider blockProjectionCollider;
	bool atLeastOneHasArrived = false;
	vector<sn_PushableBlock*>::iterator bitr;
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		(*bitr)->animate( ticksPassed );
	}
	
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		if ((*bitr)->isGone()) continue;
		//when a block has reached its destination we have to establish it at its new location
		if ((*bitr)->hasArrived() && !(*bitr)->isActive())
		{
			
			//we will need to update the movementDeltas
			atLeastOneHasArrived = true;

			//check for hanging over empty space
			blockProjectionCollider = (*bitr)->getCollider();
			blockProjectionCollider.moveBy(0.0f,0.0f,-0.5);
			bool supported = false;
			for (vector<sn_StageElement*>::iterator itr = hStageElements->begin();
				 itr != hStageElements->end(); ++itr)
			{
				if ((*bitr) == (sn_PushableBlock*)(*itr)) continue; //skip self when comparing
				if (blockProjectionCollider.collisionWith((*itr)->collider))
				{
					supported = true;
					(*bitr)->setFalling(false);
					break;
				}
			}
			
			if (!supported)
			{
				//if we are on the bottom row we need to remove this block
				//it has fallen off the stage
				if ((*bitr)->getZ() <= 0)
				{
					if ((*bitr)->getZ() <= -20)
					{
						(*bitr)->setGone(true);
						continue;
					}
					if (!(*bitr)->isFallingOffStage()) (*bitr)->restoreOriginalMovementDeltas();
					
					(*bitr)->fallOffStage();
				}
				
				(*bitr)->setFalling(true);
				(*bitr)->moveBy(0,0,-1);

			}
		}
		

	}
	//when a block arrives it's time to apply its movementDeltas in its new home
	//we reapply ALL of the blocks to avoid an "overlap" error.
	if (atLeastOneHasArrived)
	{
		updateMovementDeltas();
	}
}

//returns an iterator pointing to end if no colliding block is found
vector<sn_PushableBlock*>::iterator sn_PushableBlockCollection::findCollidingBlock( sn_Collider &c)
{
	vector<sn_PushableBlock*>::iterator bitr;
	vector<sn_PushableBlock*>::iterator collidingBlock = blocks.end();
	for (bitr = blocks.begin(); bitr != blocks.end() && collidingBlock == blocks.end(); ++bitr)
	{
		if (c.collisionWith((*bitr)->getCollider()))
		{
			collidingBlock = bitr;
		}		
	}
	return collidingBlock;
}

//returns an iterator pointing to end if no colliding block is found
vector<sn_PushableBlock*>::iterator sn_PushableBlockCollection::findCollidingBlock( vector<sn_PushableBlock*>::iterator &pBlock)
{
	vector<sn_PushableBlock*>::iterator bitr;
	vector<sn_PushableBlock*>::iterator collidingBlock = blocks.end();
	for (bitr = blocks.begin(); bitr != blocks.end() && collidingBlock == blocks.end(); ++bitr)
	{
		if (pBlock == bitr) continue; //don't include pBlock in collision check!
		if ((*pBlock)->getCollider().collisionWith((*bitr)->getCollider()))
		{
			collidingBlock = bitr;
		}		
	}
	return bitr;
}

vector<sn_PushableBlock*>::iterator sn_PushableBlockCollection::getBeginItr()
{
	return blocks.begin();
}

vector<sn_PushableBlock*>::iterator sn_PushableBlockCollection::getEndItr()
{
	return blocks.end();
}

void sn_PushableBlockCollection::dropTest()
{
	sn_Collider blockProjectionCollider;
	vector<sn_PushableBlock*>::iterator bitr;
	for (bitr = blocks.begin(); bitr != blocks.end(); ++bitr)
	{
		//always skip blocks that have vanished from the stage
		if ((*bitr)->isGone()) continue;
		//test all blocks that aren't already moving
		if (!(*bitr)->isActive())
		{
			//check for hanging over empty space
			blockProjectionCollider = (*bitr)->getCollider();
			blockProjectionCollider.moveBy(0.0f,0.0f,-0.5);
			bool supported = false;
			for (vector<sn_StageElement*>::iterator itr = hStageElements->begin();
				 itr != hStageElements->end(); ++itr)
			{
				if ((*bitr) == (sn_PushableBlock*)(*itr)) continue; //skip self when comparing
				if (blockProjectionCollider.collisionWith((*itr)->collider))
				{
					supported = true;
					(*bitr)->setFalling(false);
					break;
				}
			}
			
			if (!supported)
			{
				//if we are on the bottom row we need to remove this block
				//it has fallen off the stage
				if ((*bitr)->getZ() <= 0)
				{
					if ((*bitr)->getZ() <= -20)
					{
						(*bitr)->setGone(true);
						continue;
					}
					if (!(*bitr)->isFallingOffStage()) (*bitr)->restoreOriginalMovementDeltas();
					
					(*bitr)->fallOffStage();
				}
				
				(*bitr)->setFalling(true);
				(*bitr)->moveBy(0,0,-1);
			}
		}
	}
}
