#include "sn_recording.h"

sn_Recording::sn_Recording()
{
	playheadTime = 0;
	secondaryPlayheadTime = 0;
}

void sn_Recording::record( unsigned int ticksPassed )
{
	playheadTime += ticksPassed;
	secondaryPlayheadTime += ticksPassed;
}

void sn_Recording::recordAction( int action )
{
	sn_recordingStep tempStep = { 1, action, playheadTime };
	steps.push_back(tempStep);
	playheadTime = 0;
	secondaryPlayheadTime = 0;
}

void sn_Recording::recordSecondaryAction( int action )
{
	sn_recordingStep tempStep = { 0, action, secondaryPlayheadTime };
	steps.push_back(tempStep);
	secondaryPlayheadTime = 0;
}

//-1 returned means no new action
int sn_Recording::playAndGetAction( unsigned int ticksPassed )
{
	playheadTime += ticksPassed; 
	secondaryPlayheadTime += ticksPassed;
	if (itr == steps.end()) return -99;
	if (itr->primary == true && playheadTime >= itr->timeStamp)
	{
		int actionRead = itr->action;
		++itr;
		playheadTime = 0;
		secondaryPlayheadTime = 0;
		return actionRead;
	}
	//either we aren't up to the time for the action or the next action will be a secondary action
	//so we just keep the playheads going and no action is taken
	return -1; //-1 means no action
}

int sn_Recording::playAndGetSecondaryAction( unsigned int ticksPassed )
{
	if (itr == steps.end()) return -99;
	playheadTime += ticksPassed;
	secondaryPlayheadTime += ticksPassed;
	if ( itr->primary == false && secondaryPlayheadTime >= itr->timeStamp)
	{
		int actionRead = itr->action;
		++itr;
		secondaryPlayheadTime = 0;
		return actionRead;
	}
	//either we aren't to the time for the action or the next action will be a primary action
	//so we just keep the playheads going and no action is taken.
	return -1; //-1 means no action
}

int sn_Recording::load( string filename )
{
	steps.clear();
	
	ifstream datafile;
	
#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
#else
	string path = "data/recordings/" + filename;
#endif
	
	datafile.open(path.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		sn_recordingStep tempStep;
		while (datafile >> tempStep.primary)
		{
			datafile >> tempStep.action;
			datafile >> tempStep.timeStamp;
			steps.push_back(tempStep);
		}
		datafile.close();
		
		reset();
		return true;
	}
	else
	{
		return false;
	}
}

int sn_Recording::save( string filename ) const
{
	ofstream datafile;
	
#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
#else
	string path = "data/recordings/" + filename;
#endif
	
	datafile.open(path.c_str(), ios::out);
	
	if (datafile.is_open())
	{
		for (vector<sn_recordingStep>::const_iterator sitr = steps.begin(); sitr != steps.end(); ++sitr)
		{
			datafile << sitr->primary << ' ' << sitr->action << ' ' << sitr->timeStamp << endl;
		}
		datafile.close();
		return true;
	}
	else
	{
		return false;
	}
	
}

bool sn_Recording::isAtEnd()
{
	return itr == steps.end();
}

void sn_Recording::reset()
{
	playheadTime = 0;
	itr = steps.begin();
}