#include "sn_SpaceGrid.h"
#include "sn_Game.h"
#include <fstream>
using namespace std;

sn_SpaceGrid::sn_SpaceGrid()
{
	currentCameraAngle = -1;
	pModels = NULL;
}

sn_SpaceGrid::sn_SpaceGrid(sn_3DmodelCollection * models)
{
	init(models);
}

void sn_SpaceGrid::clearGrid()
{
	for (vector<sn_Space*>::iterator itr = grid.begin();
		 itr != grid.end(); ++itr)
	{
		delete (*itr);
		(*itr) = NULL;
	}
	
	grid.clear();
}

sn_SpaceGrid::~sn_SpaceGrid()
{
	clearGrid();
}

void sn_SpaceGrid::init(sn_3DmodelCollection * models)
{
	pModels = models;
}


int sn_SpaceGrid::load(const string &filename)
{
	currentCameraAngle = -1;
	clearGrid();
	snowSpaceIndices.clear();
	exitSpaceIndices.clear();
	
	ifstream datafile;
	
	#if defined(__APPLE__)
	filepath = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	filepath = "data/stages/" + filename;
	#endif
	
	datafile.open(filepath.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		unsigned int sizeX, sizeY, sizeZ;
		
		datafile >> sizeX;
		datafile >> sizeY;
		datafile >> sizeZ;
		
		movementArray.resize(sizeX, sizeY, sizeZ);
		
		datafile.ignore(80, '\n');
		
		sn_MovementDeltas tempDelta;
		string tempFilename;
		
		getline(datafile, tempFilename);
		
		int i = 0;
		
		while (tempFilename != "t x y z l f")
		{
			//load model needed for this stage
			if (pModels->load(tempFilename))
				spaceTypeList.push_back(pModels->getModelIndex(tempFilename)); //first time loaded? keep track of indexes in order
			indexBySpaceTypeList[spaceTypeList.back()] = i;
			//keep track of the indices to the different snow type models
			if (tempFilename == "flatsnow.mdl")
			{ flatSnowType = spaceTypeList[i]; }
			else if (tempFilename == "rampsnow.mdl")
			{ rampSnowType = spaceTypeList[i]; }
			else if (tempFilename == "flatsnowdirt.mdl")
			{ flatSnowGoneType = spaceTypeList[i]; }
			else if (tempFilename == "rampsnowdirt.mdl")
			{ rampSnowGoneType = spaceTypeList[i]; }
			else if (tempFilename == "exit_open.mdl")
			{ exitOpenType = spaceTypeList[i]; }
			//else if (tempFilename == "exit_locked.mdl")
			//{ exitClosedType = spaceTypeList[i]; }
			
			
			//get map of movement deltas for this particular model
			getline(datafile, tempFilename);
			deltasList.push_back(tempDelta);
			deltasList.back().init(&movementArray);
			deltasList.back().load(tempFilename);
			
			++i;
			getline(datafile, tempFilename);
		}
	
		sn_Space* pTempSpace = new sn_Space;
		pTempSpace->init(pModels);
		i = 0;
		int spaceTypeLookupIndex;
		while (datafile >> spaceTypeLookupIndex)
		{
			pTempSpace->spaceType = spaceTypeList[spaceTypeLookupIndex];
			
			if (pTempSpace->spaceType == flatSnowType || pTempSpace->spaceType == rampSnowType)
				snowSpaceIndices.push_back(i); //keep a separate list of indices to all snow spaces
			
			if (pTempSpace->spaceType == exitOpenType) //|| pTempSpace->spaceType == exitClosedType )
				exitSpaceIndices.push_back(i);
			
			datafile >> pTempSpace->x;
			datafile >> pTempSpace->y;
			datafile >> pTempSpace->z;
			datafile >> pTempSpace->layer;
			datafile >> pTempSpace->facing;
			deltasList[spaceTypeLookupIndex].applyAt(pTempSpace->x, pTempSpace->y, 
													 pTempSpace->z, pTempSpace->facing );
			
			pTempSpace->collider = pModels->getCollider(pTempSpace->spaceType); //get collision size and OFFSET from .mdl file
			pTempSpace->collider.moveBy(pTempSpace->x, pTempSpace->y, pTempSpace->z); //move it into the same position as the space
			
			grid.push_back(pTempSpace);
			pTempSpace = new sn_Space;
			pTempSpace->init(pModels);
			++i;
		}
		
		delete pTempSpace; //this method leaves an extra sn_Space so we delete it
		pTempSpace = NULL; //just in case
		
		datafile.close();
		
		movementArray.setBorder();
		
		return true;
	}
	else
	{
		sn_Space* pTempSpace;
		pModels->load("snowflat.mdl");
		float xi, yi;
		for (yi = 0; yi<20; yi+=2)
		{
			for (xi = 0; xi<20; xi+=2)
			{
				pTempSpace = new sn_Space;
				pTempSpace->spaceType = 0;
				pTempSpace->x = xi;
				pTempSpace->y = yi;
				pTempSpace->z = 0;
				pTempSpace->layer = 0;
				pTempSpace->facing = 0;
				grid.push_back(pTempSpace);
				movementArray.setMovement(xi,yi,0,5,snmt::blocked); //5 is for down direction
			}
		}
		
		movementArray.setBorder();
		
		return false;
	}
}

int sn_SpaceGrid::save(const string &filename)
{
	ifstream originalFile;
	ofstream newFile;
	string newFilepath;
	string standardFilepath;
	
	#if defined(__APPLE__)
	newFilepath = GAMETITLE + ".app/Contents/Resources/" + filename;
	standardFilepath = GAMETITLE + ".app/Contents/Resources/standard.grid";
	#else
	newFilepath = "data/stages/" + filename;
	standardFilepath = "data/stages/standard.grid";
	#endif
	
	newFile.open(newFilepath.c_str(), ios::out);
	originalFile.open(standardFilepath.c_str(), ios::in);
	
	if (originalFile.is_open())
	{
		string tempString;
		
		newFile << movementArray.getXSize() << ' ';
		newFile << movementArray.getYSize() << ' ';
		newFile << movementArray.getZSize() << endl;
		getline(originalFile, tempString);
		
		int i = 0;
		
		do 
		{
			getline(originalFile, tempString);
			newFile << tempString << endl;
		} while (tempString != "t x y z l f");
		
		originalFile.close();
		
		for (vector<sn_Space*>::const_iterator itr=grid.begin();
			 itr != grid.end(); ++itr)
		{
			newFile << (*itr)->spaceType << ' ';
			newFile << (*itr)->x << ' ';
			newFile << (*itr)->y << ' ';
			newFile << (*itr)->z << ' ';
			newFile << (*itr)->layer << ' ';
			newFile << (*itr)->facing << ' ';
			newFile << endl;
		}
		
		newFile.close();
		
		return true;
	}
	else
	{
		return false;
	}
	
}

sn_Space* sn_SpaceGrid::placeNewSpace( int spaceType, float x, float y, float z, int facing)
{
	sn_Space* pTempSpace = new sn_Space;
	pTempSpace->init(pModels);

	pTempSpace->spaceType = spaceTypeList[spaceType];
	pTempSpace->x = x;
	pTempSpace->y = y;
	pTempSpace->z = z;
	pTempSpace->layer = 1;
	pTempSpace->facing = facing;
		
	if (pTempSpace->spaceType == flatSnowType || pTempSpace->spaceType == rampSnowType)
		snowSpaceIndices.push_back((int)grid.size()); //keep a separate list of indices to all snow spaces
		
	if (pTempSpace->spaceType == exitOpenType) //|| pTempSpace->spaceType == exitClosedType )
		exitSpaceIndices.push_back((int)grid.size());
		
	deltasList[spaceType].applyAt( (int)x, (int)y, (int)z, facing );
		
	pTempSpace->collider = pModels->getCollider(pTempSpace->spaceType); //get collision size and OFFSET from .mdl file
	pTempSpace->collider.moveBy(x, y, z); //move it into the same position as the space
		
	grid.push_back(pTempSpace);
	
	return pTempSpace;
	
	//movementArray.setBorder();
}

void sn_SpaceGrid::eraseArea( float x, float y, float z )
{
	vector<sn_Space*>::iterator itr = grid.begin();
	while (itr != grid.end())
	{
		if ((*itr)->x == x && (*itr)->y == y && (*itr)->z == z)
		{
			deltasList[(*itr)->spaceType].applyFreeAt(x,y,z,(*itr)->facing);
			delete (*itr);
			(*itr) = NULL;
			itr = grid.erase(itr);
			
		}
		else
			++itr;
	}
}

void sn_SpaceGrid::resize(float x, float y, float z)
{
	movementArray.resize((int)x, (int)y, (int)z);
	for (vector<sn_Space*>::const_iterator itr=grid.begin();
		 itr != grid.end(); ++itr)
	{
		deltasList[indexBySpaceTypeList[(*itr)->spaceType]].applyAt((*itr)->x, (*itr)->y, (*itr)->z, (*itr)->facing );
	}
	movementArray.setBorder();
}

snmt::eMoveType sn_SpaceGrid::getMovement( int x, int y, int z, int dir )
{
	return movementArray.getMovement(x,y,z,dir);
}

vector<sn_Space*>::const_iterator sn_SpaceGrid::getGridBeginItr()
{
	return grid.begin();
}

vector<sn_Space*>::const_iterator sn_SpaceGrid::getGridEndItr()
{
	return grid.end();
}

void sn_SpaceGrid::drawMovement()
{
	movementArray.draw();
}

int sn_SpaceGrid::tryToAbsorbSnow( float x, float y, float z )
{
	int found = false;
	int i;
	for (i=0; i < snowSpaceIndices.size(); ++i)
	{
		if (grid[snowSpaceIndices[i]]->x == x &&
			grid[snowSpaceIndices[i]]->y == y &&
			grid[snowSpaceIndices[i]]->z == z )
		{
			found = true;
			if ( grid[snowSpaceIndices[i]]->spaceType == flatSnowType )
			{ grid[snowSpaceIndices[i]]->spaceType = flatSnowGoneType; }
			else if (grid[snowSpaceIndices[i]]->spaceType == rampSnowType )
			{ grid[snowSpaceIndices[i]]->spaceType = rampSnowGoneType; }
			else found = false;
		}
	}
	return found;
}

bool sn_SpaceGrid::exitIsAt( float x, float y, float z )
{
	bool found = false;
	int i;
	for (i=0; i < (int)exitSpaceIndices.size(); ++i)
	{
		if (grid[exitSpaceIndices[i]]->x == x &&
			grid[exitSpaceIndices[i]]->y == y &&
			grid[exitSpaceIndices[i]]->z == z )
		{
			found = true;
		}
	}
	return found;
}

float sn_SpaceGrid::getXMax() const
{
	return float(movementArray.getXSize()-1);
}

float sn_SpaceGrid::getYMax() const
{
	return float(movementArray.getYSize()-1);
}

float sn_SpaceGrid::getZMax() const
{
	return float(movementArray.getZSize()-1);
}

int sn_SpaceGrid::getModelID( int index )
{
	return spaceTypeList[index];
}

int sn_SpaceGrid::getMaxModelIndex()
{
	return spaceTypeList.size() - 1;
}

sn_MovementArray * sn_SpaceGrid::getMovementArrayPtr()
{
	return &movementArray;
}
