#ifndef SN_SPACEGRID
#define SN_SPACEGRID

#include "sn_3DmodelCollection.h"
#include <vector>
#include <string>
#include <iostream>
#include "sn_Space.h"
#include "sn_MovementArray.h"
#include "sn_MovementDeltas.h"

using namespace std;

class sn_SpaceGrid
{
public:
	sn_SpaceGrid();
	sn_SpaceGrid(sn_3DmodelCollection * models);
	void clearGrid();
	~sn_SpaceGrid();
	void init(sn_3DmodelCollection * models);
	int load(const string &filename);
	
	int save(const string &filename);
	sn_Space* placeNewSpace( int spaceType, float x, float y, float z, int facing);
	void eraseArea( float x, float y, float z );
	void resize(float x, float y, float z);
	
	snmt::eMoveType getMovement( int x, int y, int z, int dir );
	int tryToAbsorbSnow( float x, float y, float z );
	bool exitIsAt( float x, float y, float z );
	bool snowFlatIsAt(float x, float y, float z);
	
	float getXMax() const;
	float getYMax() const;
	float getZMax() const;
	
	int getModelID( int index );
	int getMaxModelIndex();
	
	vector<sn_Space*>::const_iterator getGridBeginItr();
	vector<sn_Space*>::const_iterator getGridEndItr();

	void drawMovement(); //for debugging only
	
	sn_MovementArray * getMovementArrayPtr();
	
private:
	string filepath;
	vector<sn_Space*>grid;
	sn_MovementArray movementArray;
	sn_3DmodelCollection * pModels;
	int currentCameraAngle;
	
	vector<sn_MovementDeltas> deltasList;
	vector<int> spaceTypeList;
	map<int, int> indexBySpaceTypeList;
	
	vector<int>snowSpaceIndices;
	
	int flatSnowType, rampSnowType;
	int flatSnowGoneType, rampSnowGoneType;
	
	vector<int>exitSpaceIndices;
	
	int exitOpenType, exitClosedType;
};

typedef vector<sn_Space*>::const_iterator gridItrType;

#endif