#include "sn_Sprite.h"

sn_Sprite::sn_Sprite()
{
	currFrame = 0;
	currTotalFrames = 0;
	currDelayTime = 0;
	currDuration = 0;
	animSeqList = NULL;
	loop = true;
	reachedEnd = false;
	paused = false;
}

void sn_Sprite::init (sn_AnimationSeqList * animationSequenceList)
{
	animSeqList = animationSequenceList;
}

void sn_Sprite::startSequence (string animationSequenceName)
{	
	reachedEnd = false;
	currFrame = 0;
	currDelayTime = 0;
	currSequenceName = animationSequenceName;
	currTotalFrames = animSeqList->getLength(animationSequenceName);
	currDuration = animSeqList->getDuration(animationSequenceName,0);
}

void sn_Sprite::animate( unsigned int ticksPassed )
{
	if (reachedEnd) return;
	if (paused) return;
	currDelayTime += ticksPassed;
	if (currDelayTime >= currDuration)
	{
		currDelayTime = 0;
		++currFrame;
		
		if (currFrame >= currTotalFrames)
		{
			if (loop)
			{
				currFrame = 0;
			}
			else
			{
				reachedEnd = true;
				--currFrame;
			}
		}
		currDuration = animSeqList->getDuration(currSequenceName,currFrame);
	}
}

void sn_Sprite::draw()
{
	animSeqList->draw(currSequenceName, currFrame);
}

void sn_Sprite::draw(float scale, bool horizFlip, bool vertFlip)
{
	animSeqList->draw(currSequenceName, currFrame, scale, horizFlip, vertFlip);
}

bool sn_Sprite::atEnd() const {return reachedEnd;}

void sn_Sprite::setLoop(bool trueOrFalse)
{
	loop = trueOrFalse;
}

void sn_Sprite::pause()
{
	paused = true;
}

void sn_Sprite::unpause()
{
	paused = false;
}