#include "sn_Stage.h"
#include <fstream>
using namespace std;

sn_Stage::~sn_Stage()
{
	if (wallDrawDL > 0) glDeleteLists(wallDrawDL,1);
	for (vector<sn_Player*>::iterator eitr=enemies.begin();
		 eitr != enemies.end(); ++eitr)
	{
		delete (*eitr);
	}
}

void sn_Stage::init( sn_TextureCollection * pTextureCollection)
{
	pTextures = pTextureCollection;
	models.init(pTextureCollection);
	grid.init(&models);

	currentStage = 0;
	recording = false;
	playingBack = false;
	
	wallDrawDL = 0;
	
	currentModelIndex = 0;
	
	title = "new stage!";
	
}

int sn_Stage::load( string filename, int stage )
{
	editPlayerX = 1; editPlayerY = 1; editPlayerZ = 1; editPlayerF = 2;
	currentStage = stage;
	completed = false;
	failed = false;
	
	int loaded;
	
	//clear out data from last load
	snowballs.clear();
	pStageElements.clear();
	enemies.clear();
	if (wallDrawDL > 0 ) {glDeleteLists(wallDrawDL,1); wallDrawDL = 0;}
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/stages/" + filename;
	#endif
	
	ifstream datafile;
	datafile.open(path.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		string tempStr;
		
		//first load grid (.grd)
		tempStr = g_removeExtension(filename) + ".grid";
		grid.load(tempStr);
		
		//now load pushable blocks for grid
		sn_MovementArray* pMA = grid.getMovementArrayPtr();
		blocks.init( &models, pMA, &pStageElements );
		tempStr = g_removeExtension(filename) + ".blk";
		blocks.load(tempStr);
		
		//load title
		getline(datafile, title);
		
		//load player coordinates
		float pX, pY, pZ, pf; //player vertex and facing direction (0-3)
		getline(datafile, tempStr); //eat up the player heading
		getline(datafile, tempStr); //eat up label heading
		datafile >> pX;
		datafile >> pY;
		datafile >> pZ;
		datafile >> pf;
		player.init( pTextures );
		player.moveTo(pX,pY,pZ);
		player.facing = (int)pf;
		
		//for the editors sake:
		editPlayerX = pX;
		editPlayerY = pY;
		editPlayerZ = pZ;
		editPlayerF = pf;
		
		//load enemies if there are any
		datafile.ignore(80,'\n');
		if (getline(datafile, tempStr)) //if there is an enemies heading here we continue
		{
			getline(datafile, tempStr); //eat up the label heading
			
			float eT, eX, eY, eZ, eL, eF;
			sn_Player tempEnemy;
			
			datafile >> eT;
			do
			{
				datafile >> eX;
				datafile >> eY;
				datafile >> eZ;
				datafile >> eL;
				datafile >> eF;
				
				enemies.push_back(new sn_Player);
				enemies.back()->init( pTextures );
				enemies.back()->moveTo(eX, eY, eZ);
				enemies.back()->layer = (int)eL;
				enemies.back()->facing = (int)eF;
			} while (datafile >> eT);
		}
		
		
		datafile.close();
		loaded = true;
	}
	else
	{
		datafile.close();
		grid.load("test.grid");
		sn_MovementArray* pMA = grid.getMovementArrayPtr();
		blocks.init( &models, pMA, &pStageElements );
		blocks.load("test.blk");
		player.init( pTextures );
		player.moveTo(1,1,1);
		loaded = true;
	}
	
	//add the grid pointers to sn_Space data into a list of generic StageElement pointers for later sorting
	//vector<sn_Space*>::const_iterator endItr = grid.getGridEndItr();
	
    for (vector<sn_Space*>::const_iterator itr = grid.getGridBeginItr();
		 itr != grid.getGridEndItr(); ++itr)
	{
		pStageElements.push_back((sn_StageElement*)(*itr));
	}
	
	pStageElements.push_back((sn_StageElement*)(&player));
	
	for (vector<sn_Player*>::const_iterator eitr = enemies.begin();
		 eitr != enemies.end(); ++eitr)
	{
		pStageElements.push_back((sn_StageElement*)(*eitr));
	}
	
	snowballs.resize(10);
	
	for (int i=0; i<(int)snowballs.size(); ++i)
	{
		snowballs[i].init( pTextures );
		pStageElements.push_back((sn_StageElement*)(&snowballs[i]));
	}
	
	//initialize shadow
	pTextures->loadRepeating("shadow.png");
	shadowTextureID = pTextures->getID("shadow.png");
	
	pTextures->loadRepeating("cliffside_wall.png");
	wallTextureID = pTextures->getID("cliffside_wall.png");
	
	pTextures->loadRepeating("cliffside_inwall.png");
	inWallTextureID = pTextures->getID("cliffside_inwall.png");
	
	font.init(pTextures);
	font.load("testWallFont256.png");
	font.setScale(3.0f);
	font.setCentered(false);
	
	createWallsDisplayList();
	
	currentCameraAngle = -1;
	drawMovementArray = false;
	drawColliders = false;
	chanceToJump = false;
	completed = false;
	

	
	return loaded;
}

int sn_Stage::save( const string &filename )
{
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename + ".stg";
	#else
	string path = "data/stages/" + filename + ".stg";
	#endif
	
	ofstream datafile;
	datafile.open(path.c_str(), ios::out);
	
	if (datafile.is_open())
	{
		datafile << title << endl;
		datafile << "player" << endl;
		datafile << "x y z f" << endl;
		datafile << editPlayerX << ' ';
		datafile << editPlayerY << ' ';
		datafile << editPlayerZ << ' ';
		datafile << editPlayerF << endl;

		if (enemies.size() > 0)
		{
			datafile << "enemies" << endl;
			datafile << "t x y z l f" << endl;
			

			for (vector<sn_Player*>::const_iterator eitr = enemies.begin();
				 eitr != enemies.end(); ++eitr)
			{
				datafile << 0 << ' ';
				datafile << (*eitr)->x << ' ';
				datafile << (*eitr)->y << ' ';
				datafile << (*eitr)->z << ' ';
				datafile << (*eitr)->layer << ' ';
				datafile << (*eitr)->facing << endl;
			}
		}
		
		datafile.close();
		
		grid.save( filename + ".grid" );
		blocks.save( filename + ".blk" );
		
		return true;
	}
	else
	{
		return false;
	}
}

void sn_Stage::resort( float cameraAngle )
{
	int isoAngle;
	isoAngle = (int)(cameraAngle / 90);
	
	currentCameraAngle = isoAngle;
	switch(isoAngle)
	{
		case 0:
			sort(pStageElements.begin(),pStageElements.end(),compareAngle45);
			break;
		case 1:
			sort(pStageElements.begin(),pStageElements.end(),compareAngle135);
			break;
		case 2:
			sort(pStageElements.begin(),pStageElements.end(),compareAngle225);	
			break;
		case 3:
			sort(pStageElements.begin(),pStageElements.end(),compareAngle315);
			break;
	}
}

void sn_Stage::draw( float cameraAngle )
{
	int isoAngle;
	isoAngle = (int)(cameraAngle / 90);
	
	//if the angle has changed we need to resort the grid
	if (isoAngle != currentCameraAngle )
	{
		currentCameraAngle = isoAngle;
		switch(isoAngle)
		{
			case 0:
				sort(pStageElements.begin(),pStageElements.end(),compareAngle45);
				break;
			case 1:
				sort(pStageElements.begin(),pStageElements.end(),compareAngle135);
				break;
			case 2:
				sort(pStageElements.begin(),pStageElements.end(),compareAngle225);	
				break;
			case 3:
				sort(pStageElements.begin(),pStageElements.end(),compareAngle315);
				break;
		}
	}
	

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//rotate Camera
	glRotatef(-60.0f,1.0f,0.0f,0.0f);
	glRotatef(cameraAngle,0.0f,0.0f,1.0f);	
	glTranslatef(-player.x,-player.y, -player.z - 1.0f);

	drawWalls();
	
	//now that the grid is sorted in draw order we can traverse it
	for (vector<sn_StageElement*>::const_iterator itr = pStageElements.begin();
		 itr != pStageElements.end(); ++itr)
	{
		glPushMatrix();
		glTranslatef((*itr)->x, (*itr)->y, (*itr)->z);
		(*itr)->draw( cameraAngle );
		glPopMatrix();
	}
	
	glDepthMask(GL_FALSE);
		
	for (vector<sn_StageElement*>::const_iterator itr = pStageElements.begin();
		 itr != pStageElements.end(); ++itr)
	{
		glPushMatrix();
		glTranslatef((*itr)->x, (*itr)->y, (*itr)->z);
		(*itr)->drawAlpha( cameraAngle );
		glPopMatrix();
	}
	
	glDepthMask(GL_TRUE);
	for (vector<sn_StageElement*>::const_iterator itr = pStageElements.begin();
		 itr != pStageElements.end(); ++itr)
	{
		glPushMatrix();
		glTranslatef((*itr)->x, (*itr)->y, (*itr)->z);
		(*itr)->drawAlpha( cameraAngle );
		glPopMatrix();
	}
	
	/*for (vector<sn_Player>::iterator eitr = enemies.begin();
		 eitr != enemies.end(); eitr++)
	{
		eitr->drawAlpha( cameraAngle );
	}
	*/


	
	/*
	//draw shadow
	sn_Collider shadowCollider;
	shadowCollider.init(player.x,player.y,player.z,0.5,0.5,0.5);
	float zShadowCollision;
	bool foundSurface = false;
	for (zShadowCollision = player.z; zShadowCollision > 0.5f; zShadowCollision -= 0.5f)
	{
		shadowCollider.moveTo(player.x,player.y,zShadowCollision+0.1);
		for (gridItrType itr = grid.getGridBeginItr(); 
			 itr != grid.getGridEndItr() && !foundSurface; ++itr)
		{
			if (shadowCollider.collisionWith((*itr)->collider))
			{
				foundSurface = true;
				while (shadowCollider.collisionWith((*itr)->collider))
				{
					zShadowCollision += 0.01;
					shadowCollider.moveBy(0,0,0.01);
				}
			}
		}
		if (foundSurface == true) break;
	}
	
	float halfshadow = player.size * 0.5f;
	glEnable(GL_TEXTURE_2D);
	//glDisable(GL_DEPTH_TEST);
	glPushMatrix();
	glTranslatef(player.x,player.y,zShadowCollision+0.1);
	glBindTexture( GL_TEXTURE_2D, shadowTextureID);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0); glVertex3f(-halfshadow,-halfshadow,0.0f); 
	glTexCoord2f(1,0); glVertex3f(halfshadow,-halfshadow,0.0f);
	glTexCoord2f(1,1); glVertex3f(halfshadow,halfshadow,0.0f);
	glTexCoord2f(0,1); glVertex3f(-halfshadow,halfshadow,0.0f);
	glEnd();
	glPopMatrix();
	*/
	
	//draw player
	glDepthMask(GL_FALSE);
	glEnable(GL_DEPTH_TEST);

	glPushMatrix();
	glTranslatef(player.x, player.y, player.z);
	player.drawAlpha( cameraAngle );
	glPopMatrix();
	
	if (drawColliders)
	{
		for (vector<sn_StageElement*>::const_iterator itr = pStageElements.begin();
			 itr != pStageElements.end(); ++itr)
		{
			(*itr)->drawCollider();
		}
	}
	
	if (drawMovementArray)
	{
		grid.drawMovement();
	}
	
	//restore standard render settings
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	
	//restore modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
}

const string * sn_Stage::getCaption() const
{
	return &title;
}

void sn_Stage::toggleDrawColliders()
{
	if (drawColliders == false) drawColliders = true;
	else drawColliders = false;
}

void sn_Stage::toggleDrawMovementArray()
{
	if (drawMovementArray == false) drawMovementArray = true;
	else drawMovementArray = false;
}

void sn_Stage::animate ( unsigned int ticksPassed )
{
	bool playerWasActive = player.isActive();
	
	for (vector<sn_Player*>::iterator eitr = enemies.begin();
		 eitr != enemies.end(); eitr++)
	{
		(*eitr)->animate(ticksPassed);
	}
	
	//SNOWBALLS
	//test for snowball collisions
	for (int i = 0; i < (int)snowballs.size(); ++i)
	{
		if (snowballs[i].isActive())
		{
			snowballs[i].animate(ticksPassed);
			if (snowballs[i].collider.collisionWithBounds(0,grid.getXMax(),
														  0,grid.getYMax(),
														  0,grid.getZMax()))
			{
				snowballs[i].registerHit();
			}
			else
			{
				for (gridItrType itr = grid.getGridBeginItr(); itr != grid.getGridEndItr(); ++itr)
				{
					if (snowballs[i].collider.collisionWith((*itr)->collider))
					{
						snowballs[i].registerHit();
						break;
					}
				}
			}
		}
	}
	
	//PLAYER
	//if we have reached a point where the player can make a choice start recording how long it takes
	if (recording && player.isActive() == false) 
		tape.record(ticksPassed);
	
	//animate player image	
	player.animate( ticksPassed );
	
	//keep player in bounds of stage
	if (player.x > grid.getXMax()) player.x = grid.getXMax();
	if (player.y > grid.getYMax()) player.y = grid.getYMax();
	if (player.z > grid.getZMax()) player.z = grid.getZMax();
	if (player.x < 0.0f) player.x = 0.0f;
	if (player.y < 0.0f) player.y = 0.0f;
	if (player.z < 0.0f) 
	{
		failed = true; //falling off screen means you lose a try
		return;
	}

	//If player has just been given a chance to jump off an edge and didnt, just fall off
	if (chanceToJump && getMovementAhead(player,5) == snmt::free)
	{
		movePlayer( player.facing, false );
		chanceToJump = false;
	}
	
	//If player just stopped
	if (playerWasActive && player.isActive() == false)
	{
		//align player to nearest whole number
		player.x = g_roundToWhole(player.x);
		player.y = g_roundToWhole(player.y);
		player.z = g_roundToWhole(player.z);
		
		blocks.dropTest(); //plyaer may have moved out from under a block so test them them for a drop
		
		//check if player landed on an exit (which will load a new level)
		if (grid.exitIsAt(player.x, player.y, player.z))
		{
			completed = true;
			return;
		}
		
		//check for player sitting on air
		if (grid.getMovement((int)player.x,(int)player.y,(int)player.z,5) == snmt::free)
		{
			movePlayer( 5, false); //down, uncontrolled
		}
		else
		{
			//check if player is standing on the edge of a cliff (ahead of him)
			if (getMovementAhead(player,5) == snmt::free)
			{
				chanceToJump = true;
			}
			//check if player is standing on the edge of a cliff (beside him)
			else if (getMovementOnLeft(player,5) == snmt::free)
			{
				//fall off the edge
				movePlayer( getLeftTurn(player.facing), false); //move player to his left
				player.facing = getRightTurn(player.facing); //rotate him back so he doesn't face the way he slides
			}
			else if (getMovementOnRight(player, 5) == snmt::free)
			{
				//fall off the edge
				movePlayer( getRightTurn(player.facing), false);
				player.facing = getLeftTurn(player.facing); //rotate him back so he doesn't face the way he slides
			}
			else
			{
				//check if player is standing on the edge of a cliff (behind him)
				//and also if player is standing on "half" the edge of a cliff behind him
				float behindX = player.x+getDeltaX(getBehindTurn(player.facing));
				float behindY = player.y+getDeltaY(getBehindTurn(player.facing));
				if ( getMovementBehind(player, 5) == snmt::free
					 || grid.getMovement(behindX + getDeltaX(getLeftTurn(player.facing)), 
										 behindY + getDeltaY(getLeftTurn(player.facing)), player.z, 5) == snmt::free
					 || grid.getMovement(behindX + getDeltaX(getRightTurn(player.facing)), 
										 behindY + getDeltaY(getRightTurn(player.facing)), player.z, 5) == snmt::free
					 )
				{
					//if so, move ahead to stay on solid ground.
					movePlayer( player.facing, false);
				}
				//check if player is standing on "half" the edge of a cliff ahead of him
				float aheadX = player.x+getDeltaX(player.facing);
				float aheadY = player.y+getDeltaY(player.facing);
				if ( grid.getMovement(aheadX + getDeltaX(getLeftTurn(player.facing)), 
									  aheadY + getDeltaY(getLeftTurn(player.facing)), player.z, 5) == snmt::free
					 || grid.getMovement(aheadX + getDeltaX(getRightTurn(player.facing)), 
										 aheadY + getDeltaY(getRightTurn(player.facing)), player.z, 5) == snmt::free
					 )
				{
					//if so, move ahead on edge, eventually slipping off of it uncontrollably
					movePlayer( player.facing, false);
				}
				
			}
		}
	}
	

	
	//now that all the automatic stuff is checked, if we are playing back tape, let it do its thing
	if (playingBack)
	{
		playBack(ticksPassed);
	}
	
	//BLOCKS
	blocks.animate( ticksPassed );
}

void sn_Stage::playBack( unsigned int ticksPassed )
{
	int control;
	if (player.isActive() == false)
	{
		control = tape.playAndGetAction(ticksPassed);
	}
	else
	{
		control = tape.playAndGetSecondaryAction(ticksPassed);
	}
	
	if (control > -1 && control < sn_totalPlayerControls)
		controlPlayer(control);
	
	if (tape.isAtEnd()) 
	{
		playingBack = false; 
	}
}

void sn_Stage::controlPlayer(int control)
{	
	//the values these numbers represent are defined at top of sn_global.h
	if (control >= 5 && control <=8)
	{
		if (!player.isActive())
		{
			//convert control direction into compass direction
			int dir = ((control-5) + currentCameraAngle) % 4;
			//then change facing
			player.facing = dir;
			
			//record
			if (recording) tape.recordAction(control);
		}
	}
	else if (control == sn_Shoot)
	{
		int i = 0;
		while (snowballs[i].isActive() && i < (int)snowballs.size()) ++i;
		if (i == (int)snowballs.size()) 
		{
			//all snowballs that are possible to be shot are in the air
			return;
		}
		else if (player.size > 1.0) //shoot as long as snowball is not at minimum size
		{
			player.spit();
			player.size -= 0.05;
			snowballs[i].launch(player.x, player.y, player.z+0.5, player.facing);
		}
		
		//record
		//shooting a snowball can be time critical but since it can happen DURING other
		//player movements it must be considered secondary in that case.
		if (recording)
		{	
			if (player.isActive() == false)
				tape.recordAction(control); 
			else
				tape.recordSecondaryAction(control);
		}
		
	}
	else if (control < 4 && !player.isActive())
	{
		//deal with a movement control
		//first, convert movement pressed to compass direction
		int dir = (control + currentCameraAngle) % 4;
		movePlayer( dir, true );
		
		//record
		if (recording) tape.recordAction(control);
	}
	else if (control >= 10)
	{
		player.changeExpression( control - 10);
		
		//record
		//changing expression is not time critical so it is secondary 
		//unless happening while player is active
		if (recording)
		{	
			if (player.isActive() == false)
				tape.recordAction(control); 
			else
				tape.recordSecondaryAction(control);
		}
	}
}

void sn_Stage::movePlayer( int dir, bool controlled )
{
	chanceToJump = false; //just getting to this function allows a chance for moving
	
	//the movementArray in grid to see where player can move from here
	snmt::eMoveType movement;
	movement = grid.getMovement((int)player.x, (int)player.y, (int)player.z, dir);
	
	if (dir == 5 && !controlled)
	{
		player.setTargetRelative(0, 0, -1);
		lastBumpDirection = -1; //no possibility of bumping if falling straight down
		return;
	}
	
	if (movement == snmt::blocked)
	{
		return;
	}
	else
	{
		//establish z movment
		float deltaX = 0, deltaY = 0, deltaZ = 0;
		
		if (movement == snmt::rampUp)
		{
			deltaZ = 1;
		}
		else if (movement == snmt::rampDown)
		{
			deltaZ = -1;
		}
		else 
		{
			deltaZ = 0;
		}
		
		if (movement == snmt::free || movement == snmt::rampUp || movement == snmt::rampDown || movement == snmt::eblocked)
		{
			if (grid.tryToAbsorbSnow(player.x,player.y,player.z))
			{
				player.size += 0.05;
			}
			
			if (dir < 4)
			{
				player.facing = dir;
				deltaX = getDeltaX(dir);
				deltaY = getDeltaY(dir);
			}
		}
		
		//now deal with special cases:
		
		//if we are headed to an area where this is no floor, attempt to jump
		snmt::eMoveType destinationMovement = grid.getMovement((int)(player.x+deltaX), (int)(player.y+deltaY), (int)player.z, dir);
		if (destinationMovement != snmt::rampDown && deltaZ != -1)
		{
			if ((grid.getMovement((int)(player.x+deltaX), (int)(player.y+deltaY), (int)player.z, 5 ) == snmt::free) &&
				(destinationMovement == snmt::free || destinationMovement == snmt::eblocked))
			{
				
				//also check to see if a block is in the way (possibly just starting to fall)
				sn_Collider playerColliderProjection = player.collider;
				playerColliderProjection.moveBy(deltaX*2, deltaY*2, 0.0f);
				vector<sn_PushableBlock*>::const_iterator pushedBlock;
				pushedBlock = blocks.findCollidingBlock( playerColliderProjection );
				
				//no collisions means initiate a fall or jump
				if (pushedBlock == blocks.getEndItr()) 
				{
					if (controlled) //if button is held down jump
					{
						deltaX = deltaX * 2;
						deltaY = deltaY * 2;
						player.falling = true;
						deltaZ = -1;
					}
					else //otherwise just fall
					{
						deltaX = deltaX;
						deltaY = deltaY;
						player.falling = true;
						deltaZ = -1;
					}
				}
			}
		}
		
		//if where we are moving to is blocked then we need to induce a bump back routine
		if (grid.getMovement((int)(player.x+deltaX), (int)(player.y+deltaY), (int)(player.z+deltaZ), dir) == snmt::blocked)
		{
			if(dir != lastBumpDirection) //if we have just bumped don't repeat
			{
				player.setBounceTargetRelative(deltaX, deltaY, deltaZ);
				lastBumpDirection = dir;
			}
			return;
		}
		else //test if we are pushing against any blocks
		{
			sn_Collider playerColliderProjection;
			playerColliderProjection = player.collider;
			playerColliderProjection.moveBy(deltaX, deltaY, deltaZ);
			
			vector<sn_PushableBlock*>::iterator pushedBlock;
			pushedBlock = blocks.findCollidingBlock( playerColliderProjection );
			
			if (pushedBlock != blocks.getEndItr())
			{
				//if we are pushing against a block we need to see if it can move
				int numberOfBlocksPushable = (int)(player.size * 1.5); //if you are big you can push more than one block at a time
				
				if (!pushBlock(deltaX, deltaY, deltaZ, pushedBlock, numberOfBlocksPushable))
				{
					if (dir != lastBumpDirection) //if we have just bumped, don't repeat
					{
						player.setBounceTargetRelative(deltaX, deltaY, deltaZ);
						lastBumpDirection = dir;
					}
					return;
				}
			}
		}
		
		//if we get here we have a regular move and we set in motion
		player.setTargetRelative(deltaX, deltaY, deltaZ);
		lastBumpDirection = -1; //no bumping occurs if we make a regular move
	}
}

bool sn_Stage::playerReadyToMove()
{
	return !player.isActive();
}

bool sn_Stage::isCompleted()
{
	return completed;
}

bool sn_Stage::isFailed()
{
	return failed;
}

bool sn_Stage::pushBlock(float deltaX, float deltaY, float deltaZ, vector<sn_PushableBlock*>::iterator blockItr, int numberOfBlocksToPush)
{	
	if (numberOfBlocksToPush <= 0) return false;

	bool freeToPush = true;
	
	//check if the block runs into the anything else.
	
	//can't push a falling or moving block or it will wedge itself into other objects
	if ((*blockItr)->isFalling() || (*blockItr)->isActive())
	{
		freeToPush = false;
		return false;
	}
	
	//check for block's collision with other spaces in grid
	sn_Collider blockColliderProjection;
	blockColliderProjection = (*blockItr)->getCollider();
	blockColliderProjection.moveBy(deltaX, deltaY, deltaZ);
	for (gridItrType itr = grid.getGridBeginItr(); itr != grid.getGridEndItr(); ++itr)
	{
		if ((*itr)->collider.collisionWith(blockColliderProjection))
		{
			freeToPush = false;
			return false;
		}
	}
	
	//check for block's collision with grid boundaries
	if (blockColliderProjection.collisionWithBounds(0,grid.getXMax(),
													0,grid.getYMax(),
													0,grid.getZMax()))
	{
		freeToPush = false;
		return false;
	}
	
	//check for block's collision with other blocks
	sn_Collider blockColliderOnTopProjection;
	blockColliderOnTopProjection = (*blockItr)->getCollider();
	blockColliderOnTopProjection.moveBy(0,0,1.0);
	
	for (vector<sn_PushableBlock*>::iterator j = blocks.getBeginItr(); j != blocks.getEndItr(); ++j) 
	{
		if ( j == blockItr ) continue;
		if (blockColliderOnTopProjection.collisionWith((*j)->getCollider()))
		{
			if (pushBlock(deltaX, deltaY, deltaZ, j, numberOfBlocksToPush-1))
			{
				numberOfBlocksToPush--;
				if (numberOfBlocksToPush <= 0) return false;
			}
		}
		if (blockColliderProjection.collisionWith((*j)->getCollider()))
		{
			if (!pushBlock(deltaX, deltaY, deltaZ, j, numberOfBlocksToPush-1))
			{
				freeToPush = false;
				return false;
				break;
			}
		}
	}
	
	if (freeToPush)
	{
		(*blockItr)->receivePush(player.facing);
		return true;
	}
	else
	{
		return false;
	}
	
	return false;
}	

bool sn_Stage::compareAngle225 (const sn_StageElement* a, const sn_StageElement* b)
{
	if (a->x + a->y < b->x + b->y)
	{
		return true;
	}
	else if (a->x + a->y > b->x + b->y)
	{
		return false;
	}
	else if (a->x < b->x)
	{
		return true;
	}
	else if (a->x > b->x)
	{
		return false;
	}
	else if (a->z < b->z)
	{
		return true;
	}
	else if (a->z > b->z)
	{
		return false;
	}
	else if (a->layer < b->layer)
	{
		return true;
	}
	return false;
}

bool sn_Stage::compareAngle135 (const sn_StageElement* a, const sn_StageElement* b)
{
	if (a->x - a->y > b->x - b->y)
	{
		return true;
	}
	else if (a->x - a->y < b->x - b->y)
	{
		return false;
	}
	else if (a->y > b->y)
	{
		return true;
	}
	else if (a->y < b->y)
	{
		return false;
	}
	else if (a->z < b->z)
	{
		return true;
	}
	else if (a->z > b->z)
	{
		return false;
	}
	else if (a->layer < b->layer)
	{
		return true;
	}
	return false;
}

bool sn_Stage::compareAngle45 (const sn_StageElement* a, const sn_StageElement* b)
{
	if (a->x + a->y > b->x + b->y)
	{
		return true;
	}
	else if (a->x + a->y < b->x + b->y)
	{
		return false;
	}
	else if (a->x < b->x)
	{
		return true;
	}
	else if (a->x > b->x)
	{
		return false;
	}
	else if (a->z < b->z)
	{
		return true;
	}
	else if (a->z > b->z)
	{
		return false;
	}
	else if (a->layer < b->layer)
	{
		return true;
	}
	else if (a->layer > b->layer)
	{
		return false;
	}
	return false;
}

bool sn_Stage::compareAngle315 (const sn_StageElement* a, const sn_StageElement* b)
{
	if (a->y - a->x > b->y - b->x)
	{
		return true;
	}
	else if (a->y - a->x < b->y - b->x)
	{
		return false;
	}
	else if (a->x < b->x)
	{
		return true;
	}
	else if (a->x > b->x)
	{
		return false;
	}
	else if (a->z < b->z)
	{
		return true;
	}
	else if (a->z > b->z)
	{
		return false;
	}
	else if (a->layer < b->layer)
	{
		return true;
	}
	return false;
}

inline float sn_Stage::getDeltaX(int dir)
{
	switch (dir)
	{
	case 0:
		return 0.0f;
		break;
	case 1:
		return 1.0f;
	case 2:
		return 0.0f;
	case 3:
		return -1.0f;
	case 4:
		return 0.0f;
	case 5:
		return 0.0f;
	default:
		return 0.0f;
	}
}

inline float sn_Stage::getDeltaY(int dir)
{
	switch (dir)
	{
		case 0:
			return 1.0f;
			break;
		case 1:
			return 0.0f;
		case 2:
			return -1.0f;
		case 3:
			return 0.0f;
		case 4:
			return 0.0f;
		case 5:
			return 0.0f;
		default:
			return 0.0f;
	}
}

inline float sn_Stage::getDeltaZ(int dir)
{
	switch (dir)
	{
		case 0:
			return 0.0f;
			break;
		case 1:
			return 0.0f;
		case 2:
			return 0.0f;
		case 3:
			return 0.0f;
		case 4:
			return 1.0f;
		case 5:
			return -1.0f;
		default:
			return 0.0f;
	}
}

inline snmt::eMoveType sn_Stage::getMovementAhead(sn_StageElement &element, int dir)
{
	return grid.getMovement(int(element.x+getDeltaX(element.facing)),
							int(element.y+getDeltaY(element.facing)), 
							int(element.z), dir); 
}

inline snmt::eMoveType sn_Stage::getMovementOnRight(sn_StageElement &element, int dir)
{
	int tempFacing = ( element.facing + 1)%4;
	return grid.getMovement(int(element.x+getDeltaX(tempFacing)),
							int(element.y+getDeltaY(tempFacing)), 
							int(element.z), dir); 
}

inline snmt::eMoveType sn_Stage::getMovementBehind(sn_StageElement &element, int dir)
{
	int tempFacing = ( element.facing + 2)%4;
	return grid.getMovement(int(element.x+getDeltaX(tempFacing)),
							int(element.y+getDeltaY(tempFacing)), 
							int(element.z), dir); 
}

inline snmt::eMoveType sn_Stage::getMovementOnLeft(sn_StageElement &element, int dir)
{
	int tempFacing = ( element.facing + 3)%4;
	return grid.getMovement(int(element.x+getDeltaX(tempFacing)),
							int(element.y+getDeltaY(tempFacing)), 
							int(element.z), dir); 
}

inline int sn_Stage::getRightTurn( int dir )
{
	return (dir+1)%4;
}

inline int sn_Stage::getBehindTurn( int dir )
{
	return (dir+2)%4;
}

inline int sn_Stage::getLeftTurn( int dir )
{
	return (dir+3)%4;
}

void sn_Stage::createWallsDisplayList()
{
	if (wallDrawDL > 0) glDeleteLists(wallDrawDL,1);
	wallDrawDL = glGenLists(1);
	
	glNewList(wallDrawDL, GL_COMPILE);
	
	float xmo = grid.getXMax() + 0.25;
	float ymo = grid.getYMax() + 0.25;
	float xm = grid.getXMax(); 
	float ym = grid.getYMax();
	float zm = grid.getZMax()*2;
	float xmpart = xm/2.0f;
	float zmpart = zm/4.0f;
	float ympart = ym/2.0f;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	glBindTexture( GL_TEXTURE_2D, wallTextureID);
	//north wall
	glBegin(GL_QUADS);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(0,0); glVertex3f(-0.01,ym,0);
	glTexCoord2f(xmpart,0); glVertex3f(xm,ym,0);
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f(xmpart,zmpart); glVertex3f(xm,ym,zm);
	glTexCoord2f(0,zmpart); glVertex3f(-0.01,ym,zm);
	
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f(0,zmpart); glVertex3f(-0.01,ym,-zm); 
	glTexCoord2f(xmpart,zmpart); glVertex3f(xm,ym,-zm);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(xmpart,0); glVertex3f(xm,ym,0);
	glTexCoord2f(0,0); glVertex3f(-0.01,ym,0);
	
	//east wall
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( 0, 0 ); glVertex3f( xm, ym, 0 );
	glTexCoord2f( ympart, 0 ); glVertex3f( xm, 0, 0 );
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f( ympart, zmpart ); glVertex3f( xm, 0, zm );	
	glTexCoord2f( 0, zmpart ); glVertex3f( xm, ym, zm );
	
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f( 0, zmpart ); glVertex3f( xm, ym, -zm );
	glTexCoord2f( ympart, zmpart ); glVertex3f( xm, 0, -zm );
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( ympart,0 ); glVertex3f( xm, 0, 0 );	
	glTexCoord2f( 0, 0 ); glVertex3f( xm, ym, 0 );
	
	//south wall
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(0,0); glVertex3f(xm,0,0);
	glTexCoord2f(xmpart,0); glVertex3f(-0.01,0,0);
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f(xmpart,zmpart); glVertex3f(-0.01,0,zm);
	glTexCoord2f(0,zmpart); glVertex3f(xm,0,zm);
	
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f(0,zmpart); glVertex3f(xm,0,-zm);
	glTexCoord2f(xmpart,zmpart); glVertex3f(-0.01,0,-zm);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(xmpart,0); glVertex3f(-0.01,0,0);
	glTexCoord2f(0,0); glVertex3f(xm,0,0);
	
	//west wall
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f( ympart, zmpart ); glVertex3f( -0.01, ym, zm );
	glTexCoord2f( 0, zmpart ); glVertex3f( -0.01, 0, zm );
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( 0, 0 ); glVertex3f( -0.01, 0, 0 );	
	glTexCoord2f( ympart, 0 ); glVertex3f( -0.01, ym, 0 );
	
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( ympart, 0 ); glVertex3f( -0.01, ym, 0 );
	glTexCoord2f( 0, 0 ); glVertex3f( -0.01, 0, 0 );
	glColor4f(1.0f,1.0f,1.0f,0.0f);
	glTexCoord2f( 0, zmpart ); glVertex3f( -0.01, 0, -zm );	
	glTexCoord2f( ympart, zmpart ); glVertex3f( -0.01, ym, -zm );
	
	
	glEnd();
	
	//glDisable(GL_TEXTURE_2D);
	//glColor4f(0.2f, 0.12f, 0.0f, 1.0);
	glBindTexture(GL_TEXTURE_2D, inWallTextureID);
	glBegin(GL_QUADS);
	
	//east shelf
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( ympart, 0.25 ); glVertex3f( xmo-0.25, ymo, 0 );
	glTexCoord2f( 0, 0.25 ); glVertex3f( xmo-0.25, -0.25, 0 );
	glTexCoord2f( 0, 0 ); glVertex3f( xmo, -0.25, 0 );
	glTexCoord2f( ympart, 0 ); glVertex3f( xmo, ymo, 0 );
	//north shelf
	glTexCoord2f(0,0); glVertex3f(xmo,ymo,0);
	glTexCoord2f(xmpart,0); glVertex3f(-0.25,ymo,0);
	glTexCoord2f(xmpart,0.25); glVertex3f(-0.25,ymo-0.25,0);
	glTexCoord2f(0,0.25); glVertex3f(xmo,ymo-0.25,0);
	//west shelf
	glTexCoord2f( 0, 0 ); glVertex3f( -0.25, ymo, 0 );
	glTexCoord2f( ympart, 0 ); glVertex3f( -0.25, -0.25, 0 );
	glTexCoord2f( ympart, 0.25 ); glVertex3f( 0.01, -0.25, 0 );	
	glTexCoord2f( 0, 0.25 ); glVertex3f( 0.01, ymo, 0 );
	//south shelf
	glTexCoord2f(xmpart, 0.25); glVertex3f(xmo,0.01,0);
	glTexCoord2f(0,0.25); glVertex3f(-0.25,0.01,0);
	glTexCoord2f(0,0); glVertex3f(-0.25,-0.25,0);
	glTexCoord2f(xmpart,0); glVertex3f(xmo,-0.25,0);

	//south overhang
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(xmpart,zmpart); glVertex3f(xmo,-0.25,0);
	glTexCoord2f(0,zmpart); glVertex3f(-0.25,-0.25,0);
	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
	glTexCoord2f(0,0); glVertex3f(-0.25,-0.25,-zm);
	glTexCoord2f(xmpart,0); glVertex3f(xmo,-0.25,-zm);
	//west overhang
	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
	glTexCoord2f( 0, 0 ); glVertex3f( -0.25, ymo, -zm );
	glTexCoord2f( ympart, 0 ); glVertex3f( -0.25, -0.25, -zm );
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( ympart, zmpart ); glVertex3f( -0.25, -0.25, 0 );	
	glTexCoord2f( 0, zmpart ); glVertex3f( -0.25, ymo, 0 );
	//north overhang
	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
	glTexCoord2f(0,0); glVertex3f(xmo,ymo,-zm);
	glTexCoord2f(xmpart,0); glVertex3f(-0.25,ymo,-zm);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f(xmpart,zmpart); glVertex3f(-0.25,ymo,0);
	glTexCoord2f(0,zmpart); glVertex3f(xmo,ymo,0);
	//east overang
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glTexCoord2f( ympart, zmpart ); glVertex3f( xmo, ymo, 0 );
	glTexCoord2f( 0, zmpart ); glVertex3f( xmo, -0.25, 0 );
	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
	glTexCoord2f( 0, 0 ); glVertex3f( xmo, -0.25, -zm );	
	glTexCoord2f( ympart, 0 ); glVertex3f( xmo, ymo, -zm );

	glEnd();
	glPopMatrix();
	
	//restore standard render settings
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	
	glEndList();	
}

void sn_Stage::drawWalls()
{
	glCallList(wallDrawDL);
	
	//draw level number on wall
	glPushMatrix();
	glTranslatef(3.0f, grid.getYMax() - 0.01f,  4.0f);
	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	font.type( 0,0,0, ".%d.", currentStage );
	glPopMatrix();
}

void sn_Stage::toggleRecording()
{
	if (!recording)
	{
		recording = true;
		tape.reset();
	}
	else
	{
		recording = false;
		tape.save("title.rcd");
	}
}

void sn_Stage::startPlayback()
{
	if (!playingBack)
	{
		if (recording)
		{
			recording = false;
			tape.save("title.rcd");
		}
		playingBack = true;
		tape.load("title.rcd");
	}
}

void sn_Stage::stopPlayback()
{
	playingBack = false;
	tape.reset();
}

bool sn_Stage::isRecording() const
{
	return recording;
}

bool sn_Stage::isPlayingBack() const
{
	return playingBack;
}

void sn_Stage::controlPlayerEdit(int control)
{
	if (!player.isActive())
	{
		//the values these numbers represent are defined at top of sn_global.h
		if (control >= 5 && control <=8)
		{
				//convert control direction into compass direction
				int dir = ((control-5) + currentCameraAngle) % 4;
				//then change facing
				player.facing = dir;
		}
		else if (control == sn_Shoot)
		{
			sn_Space* pointerToNewSpace;
			pointerToNewSpace = grid.placeNewSpace(currentModelIndex, player.x, player.y, player.z, player.facing);
			pStageElements.push_back((sn_StageElement*)pointerToNewSpace);
		}
		else if (control < 4)
		{
			//deal with a movement control
			//first, convert movement pressed to compass direction
			int dir = (control + currentCameraAngle) % 4;
			player.facing = dir;
			
			float deltaX = getDeltaX(dir);
			float deltaY = getDeltaY(dir);
			float projX = player.x + deltaX;
			float projY = player.y + deltaY;
			if (projX > grid.getXMax()-1) 
			{
				//if you hit limit extend limits of level
				grid.resize(grid.getXMax()+2, grid.getYMax()+1, grid.getZMax()+1);
				createWallsDisplayList();
				deltaX = 0;
			}
			if (projX < 1) deltaX = 0;
			if (projY > grid.getYMax()-1) 
			{
				grid.resize(grid.getXMax()+1, grid.getYMax()+2, grid.getZMax()+1);
				createWallsDisplayList();
				deltaY = 0;
			}
			if (projY < 1) deltaY = 0;
			if (!(deltaX == 0 && deltaY == 0))
				player.setTargetRelative( getDeltaX(dir), getDeltaY(dir), 0 );
		}
		else if (control == sn_Up || control == sn_Down)
		{
			int dir = 0;
			if (control == sn_Up) dir = 4;
			else if (control == sn_Down) dir = 5;
			
			float deltaZ = getDeltaZ(dir);
			float projZ = player.z + deltaZ;
			if (projZ > grid.getZMax()-1) 
			{
				grid.resize(grid.getXMax()+1, grid.getYMax()+1, grid.getZMax()+2);
				createWallsDisplayList();
				deltaZ = 0;
			}
			if (projZ < 0) deltaZ = 0;
			if (deltaZ != 0)
			player.setTargetRelative(0, 0, getDeltaZ(dir));
		}
		else if (control == sn_NextModel)
		{
			currentModelIndex += 1;
			if (currentModelIndex > grid.getMaxModelIndex())
			{
				currentModelIndex = grid.getMaxModelIndex();
			}
		}
		else if (control == sn_LastModel)
		{
			currentModelIndex -= 1;
			if (currentModelIndex < 0) currentModelIndex = 0;
		}
		else if (control == sn_SaveStage)
		{	
			save( "temp" );
		}
		else if (control == sn_EraseBlock)
		{
			if (!blocks.eraseArea(player.x, player.y, player.z))
			{
				//if there is no block, try to erase enemies
				if (!removeEnemyFromArea(player.x, player.y, player.z))
				{
					//if there are no enemies, try to erase stage underneath
					eraseStageElements(player.x, player.y, player.z);
					grid.eraseArea(player.x, player.y, player.z);
				}
			}
		}
		else if (control == sn_AddPushableBlock)
		{
			blocks.addNew(player.x, player.y, player.z, 0);
		}
		else if (control == sn_AddPlayerStart)
		{
			editPlayerX = player.x;
			editPlayerY = player.y;
			editPlayerZ = player.z;
			editPlayerF = player.facing;
		}
		else if (control == sn_AddOtherSnowball)
		{
			addEnemy(0, player.x, player.y, player.z, 5, player.facing);
		}
		else if (control == sn_LoadStage)
		{
			load("temp.stg", 1);
		}
		else if (control == sn_PlayEditedStage)
		{
			save( "temp" );
		}
	}
	
}

void sn_Stage::animateEdit( unsigned int ticksPassed )
{
	bool playerWasActive = player.isActive();
	player.animate( ticksPassed );
	//If player just stopped
	if (playerWasActive && player.isActive() == false)
	{
		//align player to nearest whole number
		player.x = g_roundToWhole(player.x);
		player.y = g_roundToWhole(player.y);
		player.z = g_roundToWhole(player.z);
	}
	
}

void sn_Stage::drawEdit( float cameraAngle )
{
	draw(cameraAngle);
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	int isoAngle;
	isoAngle = (int)(cameraAngle / 90);
	
	//rotate Camera
	glRotatef(-60.0f,1.0f,0.0f,0.0f);
	glRotatef(cameraAngle,0.0f,0.0f,1.0f);	
	glTranslatef(-player.x,-player.y, -player.z - 1.0f);
	
	glColor4f(1.0f, 0.0f, 0.0f, 0.5f);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_LINE_STRIP);
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(player.x, player.y, 0);
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(player.x, player.y, grid.getZMax());
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(grid.getXMax()-1, player.y, player.z);
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(player.x, grid.getYMax()-1, player.z);
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(0, player.y, player.z);
	glVertex3f(player.x, player.y, player.z);
	glVertex3f(player.x, 0, player.z);
	glEnd();
	
	glPopMatrix();
	
	glEnable(GL_TEXTURE_2D);
	
	glPushMatrix();
	//glDisable(GL_DEPTH_TEST);
	//glDepthMask(GL_FALSE);
	glColor4f(1.0f, 1.0f, 1.0f, 0.8f);
	
	glRotatef(-60.0f,1.0f,0.0f,0.0f);
	glRotatef(cameraAngle,0.0f,0.0f,1.0f);
	glTranslatef(0,0,-1);	
	models.draw(grid.getModelID(currentModelIndex),player.facing);
	models.drawAlpha(grid.getModelID(currentModelIndex),player.facing);
	glPopMatrix();
	
	//restore standard render settings
	glEnable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void sn_Stage::eraseStageElements( float x, float y, float z)
{
	vector<sn_StageElement*>::iterator sitr = pStageElements.begin();
	while ( sitr != pStageElements.end())
	{
		if ((*sitr)->x == x && (*sitr)->y == y && (*sitr)->z == z)
		{
			sitr = pStageElements.erase(sitr);
		}
		else
			++sitr;
	}
}

void sn_Stage::addEnemy(float type, float x, float y, float z, int layer, int facing)
{
		enemies.push_back(new sn_Player);
		enemies.back()->init( pTextures );
		enemies.back()->moveTo(x, y, z);
		enemies.back()->layer = layer;
		enemies.back()->facing = facing;
		//add to the list of items to display in stage
		pStageElements.push_back((sn_StageElement*)enemies.back());
}

bool sn_Stage::removeEnemyFromArea(float x, float y, float z)
{
	bool erased = false;
	vector<sn_Player*>::iterator itr = enemies.begin();
	while( itr != enemies.end() )
	{
		if ((*itr)->x == x && (*itr)->y == y && (*itr)->z == z)
		{			
			//remove this block from the stage elements so it doesn't attempt to draw it
			for (vector<sn_StageElement*>::iterator eitr = pStageElements.begin();
				 eitr != pStageElements.end(); ++eitr)
			{
				if ((sn_StageElement*)(*itr) == (sn_StageElement*)(*eitr))
				{
					eitr = pStageElements.erase(eitr);
				}
			}
			
			//free up memory that this block took up
			delete (*itr);
			(*itr) = NULL;
			
			//remove from the block list now
			itr = enemies.erase(itr);
			
			erased = true;
		}
		else
			++itr;
	}
	return erased;
}