#ifndef SN_STAGE
#define SN_STAGE

#include <string>
#include "sn_TextureCollection.h"
#include "sn_SpaceGrid.h"
#include "sn_Player.h"
#include "sn_Projectile.h"
#include "sn_PushableBlockCollection.h"

#include "sn_FontTyper.h"
#include "sn_Recording.h"

using namespace std;

class sn_Stage
{
public:
	~sn_Stage();
	void init( sn_TextureCollection * pTextureCollection);
	int load( string filename, int stage );
	int save( const string &filename );
	void draw( float cameraAngle );
	void animate ( unsigned int ticksPassed );
	
	void resort( float camerangle );
	
	void controlPlayer(int control);
	bool playerReadyToMove();
	
	bool isCompleted();
	bool isFailed();
	
	void toggleDrawColliders();
	void toggleDrawMovementArray();
	
	void toggleRecording();
	void startPlayback();
	void stopPlayback();
	bool isRecording() const;
	bool isPlayingBack() const;
	
	const string * getCaption() const;
	
	//editing functions
	void controlPlayerEdit(int control);
	void animateEdit( unsigned int ticksPassed );
	void drawEdit( float cameraAngle );

private:
	sn_TextureCollection * pTextures;
	sn_3DmodelCollection models;
	sn_SpaceGrid grid;
	vector<sn_StageElement*> pStageElements;
	int currentCameraAngle;
	
	sn_Player player;
	vector <sn_Projectile> snowballs;
	
	vector <sn_Player*> enemies;
	
	sn_PushableBlockCollection blocks;
	
	GLuint shadowTextureID;
	GLuint wallTextureID;
	GLuint inWallTextureID;
	GLuint wallDrawDL;
	void createWallsDisplayList();
	
	int lastBumpDirection;
	
	bool drawMovementArray;
	bool drawColliders;
	
	void movePlayer( int dir, bool controlled );
	bool pushBlock(float deltaX, float deltaY, float deltaZ, vector<sn_PushableBlock*>::iterator blockItr, int numberOfBlocksToPush);
	
	bool chanceToJump; //used to allow an extra cycle for the snowball to move (jump) if about to fall off an edge of a cliff
	
	bool completed;
	bool failed;
	
	int currentStage;
	
	sn_Recording tape;
	bool recording;
	bool playingBack;
	void playBack( unsigned int ticksPassed );
	
	//edit variables
	int currentModelIndex;
	float editPlayerX, editPlayerY, editPlayerZ, editPlayerF;
	
	static bool compareAngle45 (const sn_StageElement* a, const sn_StageElement* b);
	static bool compareAngle135 (const sn_StageElement* a, const sn_StageElement* b);
	static bool compareAngle225 (const sn_StageElement* a, const sn_StageElement* b);
	static bool compareAngle315 (const sn_StageElement* a, const sn_StageElement* b);
	
	inline float getDeltaX(int compassDir);
	inline float getDeltaY(int compassDir);
	inline float getDeltaZ(int compassDir);
	
	inline snmt::eMoveType getMovementAhead(sn_StageElement &element, int dir);
	inline snmt::eMoveType getMovementOnRight(sn_StageElement &element, int dir);
	inline snmt::eMoveType getMovementBehind(sn_StageElement &element, int dir);
	inline snmt::eMoveType getMovementOnLeft(sn_StageElement &element, int dir);
	
	inline int getRightTurn( int dir );
	inline int getBehindTurn( int dir );
	inline int getLeftTurn( int dir );
	
	sn_FontTyper font;
	string title;
	
	void drawWalls();
	
	//editing functions
	void eraseStageElements( float x, float y, float z);
	void addEnemy(float type, float x, float y, float z, int layer, int facing);
	bool removeEnemyFromArea(float x, float y, float z);
	

};

#endif
