#include "sn_TextureCollection.h"
#include "sn_Game.h"
#include <iostream>
using namespace std;

int sn_TextureCollection::load(const string &filename)
{
	load(filename, false);
}

int sn_TextureCollection::loadRepeating(const string &filename)
{
	load(filename, true);
}

int sn_TextureCollection::load(const string &filename, bool repeating)
{
	if (textureID.find(filename) != textureID.end())
	{
		//there is already of texture with this filename loaded so return without loading
		return 0;
	}
	else
	{
		//attempt to load the png named filename
		const int ST_PNG_HEADER_SIZE = 8;
		int bpp;
		
		#if defined(__APPLE__)
		string path = GAMETITLE + ".app/Contents/Resources/" + filename;
		#else
		string path = "data/textures/" + filename;
		#endif
		
		
		FILE *PNG_file = fopen(path.c_str(), "rb");
		if (PNG_file == NULL)
		{
			cerr << "error opening";
			return 0;
		}
		
		GLubyte PNG_header[ST_PNG_HEADER_SIZE];
		
		fread(PNG_header, 1, ST_PNG_HEADER_SIZE, PNG_file);
		if (png_sig_cmp(PNG_header, 0, ST_PNG_HEADER_SIZE) != 0)
		{
			cerr << "Incorrect header signature";
			
			return 0;
		}
		
		png_structp PNG_reader
			= png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
		if (PNG_reader == NULL)
		{
			fclose(PNG_file);
			cerr << "couldn't create read struct";
			return 0;
		}
		
		png_infop PNG_info = png_create_info_struct(PNG_reader);
		if (PNG_info == NULL)
		{
			png_destroy_read_struct(&PNG_reader, NULL, NULL);
			fclose(PNG_file);
			cerr << "couldn't create png info struct";
			return 0;
		}
		
		png_infop PNG_end_info = png_create_info_struct(PNG_reader);
		if (PNG_end_info == NULL)
		{
			png_destroy_read_struct(&PNG_reader, &PNG_info, NULL);
			fclose(PNG_file);
			cerr << "end info null";
			return 0;
		}
		
		if (setjmp(png_jmpbuf(PNG_reader)))
		{
			png_destroy_read_struct(&PNG_reader, &PNG_info, &PNG_end_info);
			fclose(PNG_file);
			cerr << "setjmp";
			return (0);
		}
		
		png_init_io(PNG_reader, PNG_file);
		png_set_sig_bytes(PNG_reader, ST_PNG_HEADER_SIZE);
		
		png_read_info(PNG_reader, PNG_info);
		
		png_uint_32 imageW, imageH;
		imageW = png_get_image_width(PNG_reader, PNG_info);
		imageH = png_get_image_height(PNG_reader, PNG_info);
		
		png_uint_32 bit_depth, color_type;
		bit_depth = png_get_bit_depth(PNG_reader, PNG_info);
		bpp = bit_depth * 4; 		//bit_depth seems to return 8 when 32 bit???
		
		color_type = png_get_color_type(PNG_reader, PNG_info);
		
		if (color_type == PNG_COLOR_TYPE_PALETTE)
		{
			png_set_palette_to_rgb(PNG_reader);
		}
		
		if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8) 
		{
			png_set_gray_1_2_4_to_8(PNG_reader);
		}
		
		if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
		{
			png_set_gray_to_rgb(PNG_reader);
		}
		
		if (png_get_valid(PNG_reader, PNG_info, PNG_INFO_tRNS))
		{
			png_set_tRNS_to_alpha(PNG_reader);
		}
		else
		{
			png_set_filler(PNG_reader, 0xff, PNG_FILLER_AFTER);
		}
		
		if (bit_depth == 16)
		{
			png_set_strip_16(PNG_reader);
		}
		
		png_read_update_info(PNG_reader, PNG_info);
		
		png_byte* imageData = (png_byte*)malloc(4 * imageW * imageH);
		png_byte** PNG_rows = (png_byte**)malloc(imageH * sizeof(png_byte*));
		
		unsigned int row;
		for (row = 0; row < imageH; ++row)
		{
			PNG_rows[imageH - 1 - row] = imageData + (row * 4 * imageW);
		}
		
		png_read_image(PNG_reader, PNG_rows);
		
		free(PNG_rows);
		PNG_rows = NULL;
		
		png_destroy_read_struct(&PNG_reader, &PNG_info, &PNG_end_info);
		fclose(PNG_file);
		
		//Check to see if imageW and imagH are powers of 2
		if (!((imageW & (imageW - 1)) == 0) || !((imageH & (imageH - 1)) == 0))
		{
			free(imageData);
			cerr << "image imageW and or imageH not power of two";
			return 0;
		}
		
		GLuint tempTextureID;
		glGenTextures(1, &tempTextureID);				// Generate 1 OpenGL texture IDs
		
		/*glBindTexture(GL_TEXTURE_2D, tempTextureID);			// Bind Our Texture
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// GL_LINEAR ("smooth") or GL_NEAREST (not "smooth") Filtered
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Filtered
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		*/

		glGenTextures(1, &tempTextureID); //textureID is a member of iso_FrameTable class
		
		glBindTexture(GL_TEXTURE_2D, tempTextureID);
		
		gluBuild2DMipmaps(GL_TEXTURE_2D,
						  GL_RGBA,
						  imageW,
						  imageH,
						  GL_RGBA,
						  GL_UNSIGNED_BYTE,
						  imageData);
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		
		if (repeating)
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		}
		else
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		}
		
		GLuint type=GL_RGBA;   					// Set The Default GL Mode To RBGA (32 BPP)
		
		if (bpp == 24)						// Was The TGA 24 Bits
		{
			type = GL_RGB;							// If So Set The 'type' To GL_RGB
		}
		
		glTexImage2D(GL_TEXTURE_2D, 0, type, imageW, imageH,
					 0, type, GL_UNSIGNED_BYTE, imageData);

		free(imageData);
		imageData = NULL;
		
		textureID[filename] = tempTextureID;
		
		return true;
	}
	
}


GLuint sn_TextureCollection::getID(const string &filename)
{
	if (textureID.find(filename) != textureID.end())
		return textureID[filename];
	else
		return 0; //openGL starts first texture at 1?  So, this should work...
}

void sn_TextureCollection::reset()
{
	textureID.clear();
}