#include "sn_TextureSegmentList.h"

sn_TextureSegmentList::sn_TextureSegmentList()
{
	texID = 0;
	
	segments.resize(4);
	segments[0].texX1 = 0;
	segments[0].texY1 = 0;
	segments[0].texX2 = 0.50;
	segments[0].texY2 = 0.50;
	segments[0].relX1 = -0.5;
	segments[0].relY1 = -0.5;
	segments[0].relX2 = 0.5;
	segments[0].relY2 = 0.5;
	
	segments[1].texX1 = 0.50;
	segments[1].texY1 = 0;
	segments[1].texX2 = 1.0;
	segments[1].texY2 = 0.50;
	segments[1].relX1 = -0.5;
	segments[1].relY1 = -0.5;
	segments[1].relX2 = 0.5;
	segments[1].relY2 = 0.5;
	
	segments[2].texX1 = 0.0;
	segments[2].texY1 = 0.50;
	segments[2].texX2 = 0.50;
	segments[2].texY2 = 1.0;
	segments[2].relX1 = -0.5;
	segments[2].relY1 = -0.5;
	segments[2].relX2 = 0.5;
	segments[2].relY2 = 0.5;
	
	segments[3].texX1 = 0.50;
	segments[3].texY1 = 0.50;
	segments[3].texX2 = 1.00;
	segments[3].texY2 = 1.00;
	segments[3].relX1 = -0.5;
	segments[3].relY1 = -0.5;
	segments[3].relX2 = 0.5;
	segments[3].relY2 = 0.5;
}

void sn_TextureSegmentList::init(GLuint textureID)
{
	texID = textureID;
}

int sn_TextureSegmentList::load(string filename)
{
	ifstream datafile;
	
	#if defined(__APPLE__)
	string path = GAMETITLE + ".app/Contents/Resources/" + filename;
	#else
	string path = "data/textures/" + filename;
	#endif
	
	datafile.open(path.c_str(), ios::in);
	
	if (datafile.is_open())
	{
		segments.clear();
		
		string type;
		getline(datafile, type);
		if (type == "const") //size of each segment is constant
		{
			segments.clear(); //get rid of default settings
			
			int textureW, textureH;
			datafile >> textureW;
			datafile >> textureH;
			int segmentW, segmentH;
			datafile >> segmentW;
			datafile >> segmentH;
			float relW, relH;
			datafile >> relW;
			datafile >> relH;
			int horizontalSegments = textureW/segmentW;
			int verticalSegments = textureH/segmentH;
			float texXStep = 1.0/(float)horizontalSegments;
			float texYStep = 1.0/(float)verticalSegments;
			float rx2 = relW * 0.5;
			float ry2 = relH * 0.5;
			float rx1 = -rx2;
			float ry1 = -ry2;
			
			sn_textureSegment tempSegment;
			int x, y;
			//textures are loaded "upside down" with y going up unlike screen coordinates that has y going down
			//so texY must start at high number and go down to get expected behavior
			float texX = 0.0, texY = (verticalSegments-1)*texYStep;
			for (y=0; y<verticalSegments; ++y)
			{
				for (x=0; x<horizontalSegments; ++x)
				{
					tempSegment.relX1 = rx1; tempSegment.relY1 = ry1;
					tempSegment.relX2 = rx2; tempSegment.relY2 = ry2;
					tempSegment.texX1 = texX; tempSegment.texY1 = texY;
					tempSegment.texX2 = texX + texXStep; tempSegment.texY2 = texY + texYStep;
					segments.push_back(tempSegment);
					texX += texXStep;
				}
				texY -= texYStep;
				texX = 0.0;
			}
			datafile.close();
			return true;
				
		}
		else if (type == "var") //sizes of segments are variable
		{
			segments.clear(); //get rid of default segments
			
			sn_textureSegment tempSegment;
			
			float textureW, textureH;
			datafile >> textureW;
			datafile >> textureH;
			
			float NumberOfPixelsEqualToOneHeight;
			datafile >> NumberOfPixelsEqualToOneHeight;
			pixelHeight = NumberOfPixelsEqualToOneHeight;
			////all pixel measurements can be multiplied by pixelRatio to keep it relative to OpenGL coordinates
			float pixelRatio = 1.0f/NumberOfPixelsEqualToOneHeight; 
			
			float textureRatioX = 1.0f/textureW;
			float textureRatioY = 1.0f/textureH;
			
			//data in data file for each segment:
			float ulX, ulY, lrX, lrY; //absolute coordinates of upper left point and lower right point
			float centerX, centerY; //absolute coordinates of center
			

			
			char dotSeparator;
			int i = 0;
			do
			{
				datafile >> ulX;
				datafile >> ulY;
				datafile >> lrX;
				datafile >> lrY;
				datafile >> centerX;
				datafile >> centerY;
				
				tempSegment.texX1 = ulX * textureRatioX;
				tempSegment.texY1 = 1.0f - (ulY * textureRatioY); // subtracted from 1 because texture's y is inverted compared to screen
				tempSegment.texX2 = lrX * textureRatioX;
				tempSegment.texY2 = 1.0f - (lrY * textureRatioY); // subtracted from 1 because texture's y is inverted compared to screen
				
				tempSegment.relX1 = (ulX - centerX) * pixelRatio;
				tempSegment.relY1 = -(ulY - centerY) * pixelRatio; // negative because texture's y is inverted compared to screen
				tempSegment.relX2 = (lrX - centerX) * pixelRatio;
				tempSegment.relY2 = -(lrY - centerY) * pixelRatio; // negative because texture's y is inverted compared to screen
				
				segments.push_back(tempSegment);
				++i;
				
			} while (datafile >> dotSeparator);
			
			datafile.close();
			return true;
		}
		else
		{
			cout << "segment file format incorrect... needs 'const' or 'var'" << endl;
			datafile.close();
			return false;
		}
	}
	else
	{
		cout << "Error on segment file opening" << endl;
		return false;
	}
	
	return false;
			
}

int sn_TextureSegmentList::draw( int segmentIndex) const
{
	if (segmentIndex >= segments.size() || segmentIndex < 0) 
	{ 
		cerr<< "tried to access textureSegment " << segmentIndex << " texture ID: " << texID << endl; 
		return false; 
	}
	
	int i = segmentIndex;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, texID);

	glBegin(GL_QUADS);
	glTexCoord2f(segments[i].texX1, segments[i].texY1); glVertex3f(segments[i].relX1, segments[i].relY1, 0.0f);
	glTexCoord2f(segments[i].texX1, segments[i].texY2); glVertex3f(segments[i].relX1, segments[i].relY2, 0.0f);
	glTexCoord2f(segments[i].texX2, segments[i].texY2); glVertex3f(segments[i].relX2, segments[i].relY2, 0.0f);
	glTexCoord2f(segments[i].texX2, segments[i].texY1); glVertex3f(segments[i].relX2, segments[i].relY1, 0.0f);
	
	glEnd();
	
	glPopMatrix();
	
	return true;
}

int sn_TextureSegmentList::draw( int segmentIndex, float scale, bool horizFlip, bool vertFlip) const
{
	if (segmentIndex >= segments.size() || segmentIndex < 0)  return false;
	int i = segmentIndex;
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glScalef(scale,scale,scale);
	glBindTexture(GL_TEXTURE_2D, texID);
	
	if (!horizFlip && !vertFlip)
	{
		glBegin(GL_QUADS);
		glTexCoord2f(segments[i].texX1, segments[i].texY1); glVertex3f(segments[i].relX1, segments[i].relY1, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY2); glVertex3f(segments[i].relX1, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY2); glVertex3f(segments[i].relX2, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY1); glVertex3f(segments[i].relX2, segments[i].relY1, 0.0f);
		glEnd();
	}
	else if (horizFlip && vertFlip)
	{
		glBegin(GL_QUADS);
		glTexCoord2f(segments[i].texX2, segments[i].texY2); glVertex3f(segments[i].relX1, segments[i].relY1, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY1); glVertex3f(segments[i].relX1, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY1); glVertex3f(segments[i].relX2, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY2); glVertex3f(segments[i].relX2, segments[i].relY1, 0.0f);
		glEnd();
	}
	else if (horizFlip)
	{
		glBegin(GL_QUADS);
		glTexCoord2f(segments[i].texX2, segments[i].texY1); glVertex3f(segments[i].relX1, segments[i].relY1, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY2); glVertex3f(segments[i].relX1, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY2); glVertex3f(segments[i].relX2, segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY1); glVertex3f(segments[i].relX2, segments[i].relY1, 0.0f);
		glEnd();
	}
	else if (vertFlip)
	{
		glTranslatef(0,1,0);
		glBegin(GL_QUADS);
		glTexCoord2f(segments[i].texX1, segments[i].texY1); glVertex3f(segments[i].relX1, -segments[i].relY1, 0.0f);
		glTexCoord2f(segments[i].texX1, segments[i].texY2); glVertex3f(segments[i].relX1, -segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY2); glVertex3f(segments[i].relX2, -segments[i].relY2, 0.0f);
		glTexCoord2f(segments[i].texX2, segments[i].texY1); glVertex3f(segments[i].relX2, -segments[i].relY1, 0.0f);
		glEnd();
	}
		
	glPopMatrix();
	
	return true;
}

int sn_TextureSegmentList::getSize() const
{
	return segments.size();
}

float sn_TextureSegmentList::getWidth( int segmentIndex ) const
{
	if (segmentIndex >= segments.size() || segmentIndex < 0) return 0.0f;
	
	return (segments[segmentIndex].relX2 - segments[segmentIndex].relX1);
}

float sn_TextureSegmentList::getPixelHeight() const
{	
	return pixelHeight;
}
