#include <stdio.h>
#include <iostream>
using namespace std;
//use SDL to include the correct OpenGL files for each platform
#include "SDL_OpenGL.h"
#include "SDL.h"
#include "SDL_mixer.h"
//#include "g_RNG.h"
#include "sn_global.h"
#include "sn_Screen.h"
#include "g_MusicBox.h"
#include "g_Noisemaker.h"
#include "sn_Game.h"
#include "sn_GamePhase.h"
#include "sn_Phase_Title.h"
#include "sn_Phase_Play.h"
#include "sn_Phase_Options.h"
#include "sn_Prefs.h"
#include "sn_textureCollection.h"
#include "sn_3Dmodel.h"
#include "sn_Phase_Edit.h"

void beginOrtho(sn_Game &game);
void endOrtho();

int main(int argc, char *argv[])
{	
	//*setup video

	sn_Screen screen;

	sn_Prefs pref;//sb_Prefs uses a file to keep track of settings
	//		   ( game res, screen res, bit depth  v (0 means keep current depth), windowed or not, name of window)
	screen.init(1024,768,  pref.getWidth(),pref.getHeight(),0,				pref.getUseWindow(),       GAMETITLE.c_str());
    //*Pre game loop initialization BEGIN
	SDL_Event event;  //will hold events as they are polled
	int done; //set to true to end main loop

    SDL_ShowCursor(true); //show the standard mouse pointer
    SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL); //delay 0 means Disable Key Repeat
	sn_Game game; //game has gamewide variables including data objects (like musicBox and noisemaker)
    if (!pref.getUseMusic())
	{
		game.musicBox.mute();
	}
	if (!pref.getUseSound())
	{
		game.noisemaker.mute();
	}
	game.setResolution(screen.getW(), screen.getH());
	//The game Phases control all the mechanics in the game and how the game is displayed.
	sn_Phase_Title phase_Title( &game );
	sn_Phase_Play phase_Play( &game );
	sn_Phase_Options phase_Options( &game );
	sn_Phase_Edit phase_Edit( &game );
	
	sn_GamePhase * currentPhase;
	currentPhase = &phase_Title;  //set the currentPhase by pointing to the phase object we want
	
	//variables the main game loop needs to maintain
    Sint32 pointerX, pointerY, pointerRelX, pointerRelY; //to track mouse movement
    int commandKeyIsPressed = false;

	pointerX = 0; //start mouse pointer off at 0,0 at start in case event doesn't update it
	pointerY = 0;
	
	int paused = false;
	int focusPaused = false;
	sn_Button pauseLabel;
	pauseLabel.init( &game.typer, game.gameResX/2.0f, game.gameResY/2.0f, kSBBT_label, "Paused");
	pauseLabel.moveBy(-pauseLabel.getWidth()/2.0f, 0.0f);
	
	//*Pre game loop initialization END
	
	currentPhase->reset();
	currentPhase->start();
	done = false;
	
    //*Main Game Loop BEGIN	
	while ( !done )
    {
		if (game.gameover)
		{
			//done = true;
			continue;
			
			game.gameover = false;
			currentPhase = &phase_Title;
			currentPhase->reset();
			currentPhase->start();
			 
		}
				
		if (game.chooseOptions)
		{
			game.chooseOptions = false;
			currentPhase = &phase_Options;
			currentPhase->reset();
			currentPhase->start();
		}
		
		if (game.quit)
		{
			done = true;
			continue;
		}
		
		if (game.edit)
		{
			game.edit = false;
			currentPhase = &phase_Edit;
			currentPhase->reset();
			currentPhase->start();
		}
		
		if (game.changeScreen)
		{
			game.changeScreen = false;
			screen.resize(game.screenResX, game.screenResY, 0, game.useWindow);
			game.textures.reset();
			game.typer.init(&game.textures);
			game.resetStage();
			game.typer.load("main_font.png");
		}
		
		if (game.playEditedStage)
		{
			currentPhase = &phase_Play;
			currentPhase->reset();
			currentPhase->start();
			game.playEditedStage = false;
		}
		
		if (currentPhase->isDone())
		{
			if (currentPhase == &phase_Title)
			{
				currentPhase = &phase_Play;
			}
			else if (currentPhase == &phase_Options)
			{
				currentPhase = &phase_Title;
			}
			else if (currentPhase == &phase_Play)
			{
				//reset all of the variables
				game.reset();
				currentPhase = &phase_Title;
			}
			//switch currentPhase to next in line
			currentPhase->reset();
			currentPhase->start();
		}
		
        //reset the relative movement to 0
        pointerRelX = 0;
        pointerRelY = 0;
		
		//anything that needs to be set before the event loop in game object is done here
		game.preEvent();
        
        //* Poll events BEGIN
        while ( SDL_PollEvent(&event) )
        {
			//each phase has it's own way of processing events
			game.processEvent( event ); //needs to be done to update the pointer (all phases rely on game's pointer coordinates)
			if (!paused && !focusPaused)
				currentPhase->processEvent( event );
            
			switch (event.type)
            {
                case SDL_MOUSEMOTION:
                    pointerX = event.motion.x;
                    pointerY = event.motion.y;
                    pointerRelX = event.motion.xrel;
                    pointerRelY = event.motion.yrel;
                    break;
                case SDL_MOUSEBUTTONDOWN:
					if (event.button.button == SDL_BUTTON_LEFT)
					{
					}
					if (event.button.button == SDL_BUTTON_RIGHT)
					{
					}

                    break;
                case SDL_MOUSEBUTTONUP:
                    break;
                case SDL_KEYDOWN:
					if (event.key.keysym.mod & KMOD_CTRL || commandKeyIsPressed)
					{
						switch (event.key.keysym.sym)
						{
							//apple(command)-Q or control q quits game
							case SDLK_q:
								done = true;
								break;
							default:
								break;
						}
					}
					if (event.key.keysym.sym == SDLK_ESCAPE )//|| event.key.keysym.sym == SDLK_p)
					{
						if (currentPhase != &phase_Options) //pausing on the Options screen can cause problems
						{
							if (!paused)
							{
								paused = true;
								currentPhase->pause();
							}
							else
							{
								paused = false;
								currentPhase->unpause();
							}
						}

						
					}
					//command/meta key isn't considered a modifier key, so we take care of it manually
					if (event.key.keysym.sym == SDLK_LMETA || event.key.keysym.sym == SDLK_RMETA)
					{
						commandKeyIsPressed = true;
					}
					//the alt/option key
					if (event.key.keysym.sym == SDLK_LALT || event.key.keysym.sym == SDLK_RALT)
					{
					}
                    break;
                case SDL_KEYUP:
                    if (event.key.keysym.sym == SDLK_LMETA || event.key.keysym.sym == SDLK_RMETA)
                    {
                        commandKeyIsPressed = false;
                    }
                    if (event.key.keysym.sym == SDLK_LALT || event.key.keysym.sym == SDLK_RALT)
                    {
                    }
                    if (event.key.keysym.sym == SDLK_SPACE)
                    {
                    }
                    break;
				case SDL_ACTIVEEVENT:
					if (currentPhase != &phase_Options) //pausing on the Options screen can cause problems
					{
						if (event.active.state == SDL_APPINPUTFOCUS)
						{
							if (event.active.gain == 0)//if we lost input focus
							{
								if (!focusPaused)
								{
									focusPaused = true;
									if (!paused)
										currentPhase->pause();
								}
							}
							else
							{
								if (focusPaused)
								{
									focusPaused = false;
									if (!paused)
										currentPhase->unpause();
								}

							}
						}
					}
					break;
                case SDL_QUIT:
                    done = true;
                    break;
                default:
                    break;
            }
        }
		//*Poll Events END
		
	    //*update animation and game mechanics
		//the phase mechanics are internally timed to determine how much change from update to update
		if (!paused && !focusPaused)
			currentPhase->animate(); //each phase object has it's own animate function
        
	    //*update visuals
		//***may need to compensate for VERY SLOW framerates so it isn't drawn if it takes too long to get here
		glDepthMask(GL_TRUE);  //depth buffer won't clear if this is false
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);// Clear Screen And Depth Buffer
		
		if (!paused && !focusPaused)//if (!paused && !focusPaused)	
		{	
			currentPhase->draw(); //each phase object has it's own draw function
		}
		else
		{
		    currentPhase->draw();
		    beginOrtho(game);
		    //cover everything with a blue/white haze
			glLoadIdentity();
			glDisable(GL_TEXTURE_2D);
			glColor4f(0.9f,0.9f,1.0f,0.9f);
			glBegin(GL_QUADS);
			glVertex2f(2.0f,2.0f);
			glVertex2f(game.gameResX-2, 2.0f);
			glVertex2f(game.gameResX-2, game.gameResY - 2);
			glVertex2f(2.0f, game.gameResY - 2);
			glEnd();
			//return to standard render settings
			glEnable(GL_TEXTURE_2D);
			glColor4f(1.0f,1.0f,1.0f,1.0f);
   		    
            pauseLabel.draw();
			endOrtho();
		}

		glFlush();		   // Flush The GL Rendering Pipeline
		
		SDL_GL_SwapBuffers( ); // Render it all to the screen
    }
    //*Main Game Loop END

    /* Clean up the SDL library */
	SDL_Quit();
    return 0;
}

void beginOrtho(sn_Game &game)
{
    //***setup ortho projection for 2D overlay
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, game.gameResX, game.gameResY, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	//***begin 2D drawing
	glDisable(GL_DEPTH_TEST);
}

void endOrtho()
{	
	//****end 2D drawing
	
	//***restore matrices
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}    
